@@ -1,5 +1,5 @@
 /*
- * $Id: action.c,v 1.79 2001/09/25 14:30:39 tal197 Exp $
+ * $Id: action.c,v 1.94 2001/12/28 14:47:36 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2001, the ROX-Filer team.
@@ -24,7 +24,7 @@
  */
 
 /* Allow use of GtkText widget */
-#define GTK_ENABLE_BROKEN
+/* #define GTK_ENABLE_BROKEN */
 
 #include "config.h"
 
@@ -44,17 +44,21 @@
 #include "support.h"
 #include "gui_support.h"
 #include "filer.h"
+#include "display.h"
 #include "main.h"
 #include "options.h"
 #include "modechange.h"
 #include "find.h"
 #include "dir.h"
 #include "icon.h"
+#include "mount.h"
 
 /* Parent->Child messages are one character each:
  *
- * Q/Y/N 	Quiet/Yes/No button clicked
+ * Y/N 		Yes/No button clicked
  * F		Force deletion of non-writeable items
+ * Q		Quiet toggled
+ * E		Entry text changed
  */
 
 #define SENSITIVE_YESNO(gui_side, state)	\
@@ -74,8 +78,9 @@ struct _GUIside
 	int 		from_child;	/* File descriptor */
 	FILE		*to_child;
 	int 		input_tag;	/* gdk_input_add() */
-	GtkWidget 	*vbox, *log, *window, *dir, *log_hbox;
-	GtkWidget	*quiet, *yes, *no;
+	GtkWidget 	*log;
+	GtkWidget 	*vbox, *window, *dir, *log_hbox, *flag_box;
+	GtkWidget	*quiet, *yes, *no, *quiet_flag;
 	int		child;		/* Process ID */
 	int		errors;
 	gboolean	show_info;	/* For Disk Usage */
@@ -132,8 +137,10 @@ static gboolean send_error();
 static gboolean send_dir(char *dir);
 static gboolean read_exact(int source, char *buffer, ssize_t len);
 static void do_mount(guchar *path, gboolean mount);
-static void add_toggle(GUIside *gui_side, guchar *label, guchar *code,
-			gboolean active);
+static GtkWidget *add_toggle(GUIside *gui_side,
+			     guchar *label, guchar *tip,
+			     guchar *code,
+			     gboolean active);
 static gboolean reply(int fd, gboolean ignore_quiet);
 static gboolean remove_pinned_ok(GList *paths);
 
@@ -211,6 +218,10 @@ void show_condition_help(gpointer data)
 		GtkWidget *text, *vbox, *button, *hbox, *frame;
 		
 		help = gtk_window_new(GTK_WINDOW_DIALOG);
+#ifdef GTK2
+		gtk_window_set_type_hint(GTK_WINDOW(help),
+				    GDK_WINDOW_TYPE_HINT_DIALOG);
+#endif
 		gtk_container_set_border_width(GTK_CONTAINER(help), 10);
 		gtk_window_set_title(GTK_WINDOW(help),
 				_("Find expression reference"));
@@ -236,7 +247,7 @@ _("'*.htm', '*.html'      (finds HTML files)\n"
 "mtime after 1 day ago and size > 1Mb   (big, and recently modified)\n"
 "'CVS' prune, isreg                     (a regular file not in CVS)\n"
 "IsReg system(grep -q fred \"%\")         (contains the word 'fred')"));
-		gtk_widget_set_style(text, fixed_style);
+		gtk_widget_set_name(text, "fixed-font");
 		gtk_misc_set_padding(GTK_MISC(text), 4, 4);
 		gtk_label_set_justify(GTK_LABEL(text), GTK_JUSTIFY_LEFT);
 		gtk_container_add(GTK_CONTAINER(frame), text);
@@ -308,6 +319,10 @@ static void show_chmod_help(gpointer data)
 		GtkWidget *text, *vbox, *button, *hbox, *sep;
 		
 		help = gtk_window_new(GTK_WINDOW_DIALOG);
+#ifdef GTK2
+		gtk_window_set_type_hint(GTK_WINDOW(help),
+				    GDK_WINDOW_TYPE_HINT_DIALOG);
+#endif
 		gtk_container_set_border_width(GTK_CONTAINER(help), 10);
 		gtk_window_set_title(GTK_WINDOW(help),
 				_("Permissions command reference"));
@@ -401,7 +416,12 @@ static void message_from_child(gpointer 	 data,
 {
 	char buf[5];
 	GUIside	*gui_side = (GUIside *) data;
-	GtkWidget *log = gui_side->log;
+#ifdef GTK2
+	GtkTextBuffer *text_buffer;
+	GtkTextIter end;
+
+	text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(gui_side->log));
+#endif
 
 	if (read_exact(source, buf, 4))
 	{
@@ -477,11 +497,24 @@ static void message_from_child(gpointer 	 data,
 			else if (*buffer == '!')
 				gui_side->errors++;
 
-			gtk_text_insert(GTK_TEXT(log),
+#ifdef GTK2
+			gtk_text_buffer_get_end_iter(text_buffer, &end);
+			gtk_text_buffer_insert_with_tags_by_name(text_buffer,
+					&end, buffer + 1, message_len - 1,
+					*buffer == '!' ? "red" : NULL, NULL);
+			/* Causes a crash sometimes!
+			gtk_text_view_scroll_to_iter(
+				GTK_TEXT_VIEW(gui_side->log),
+				&end,
+				0.0, FALSE, 0, 0);
+				*/
+#else
+			gtk_text_insert(GTK_TEXT(gui_side->log),
 					NULL,
 					*buffer == '!' ? &red : NULL,
 					NULL,
 					buffer + 1, message_len - 1);
+#endif
 			g_free(buffer);
 			return;
 		}
@@ -507,8 +540,12 @@ static void message_from_child(gpointer 	 data,
 			report = g_strdup_printf(_("There were %d errors.\n"),
 							gui_side->errors);
 
-		gtk_text_insert(GTK_TEXT(log), NULL, &red, NULL,
+#ifdef GTK2
+		gtk_text_buffer_insert_at_cursor(text_buffer, report, -1);
+#else
+		gtk_text_insert(GTK_TEXT(gui_side->log), NULL, &red, NULL,
 				report, -1);
+#endif
 
 		g_free(report);
 	}
@@ -609,6 +646,35 @@ static gboolean send_error(void)
 	return send();
 }
 
+static void quiet_clicked(GtkWidget *button, GUIside *gui_side)
+{
+	GtkToggleButton *quiet_flag = GTK_TOGGLE_BUTTON(gui_side->quiet_flag);
+
+	if (!gui_side->to_child)
+		return;
+
+	gtk_widget_set_sensitive(gui_side->quiet, FALSE);
+
+	if (gtk_toggle_button_get_active(quiet_flag))
+		return;		/* (shouldn't happen) */
+
+	gtk_toggle_button_set_active(quiet_flag, TRUE);
+
+	if (GTK_WIDGET_SENSITIVE(gui_side->yes))
+		gtk_button_clicked(GTK_BUTTON(gui_side->yes));
+}
+
+/* Send 'Quiet' if possible, 'Yes' otherwise */
+static void find_return_pressed(GtkWidget *button, GUIside *gui_side)
+{
+	if (GTK_WIDGET_SENSITIVE(gui_side->quiet))
+		gtk_button_clicked(GTK_BUTTON(gui_side->quiet));
+	else if (GTK_WIDGET_SENSITIVE(gui_side->yes))
+		gtk_button_clicked(GTK_BUTTON(gui_side->yes));
+	else
+		gdk_beep();
+}
+
 static void button_reply(GtkWidget *button, GUIside *gui_side)
 {
 	char *text;
@@ -621,8 +687,42 @@ static void button_reply(GtkWidget *button, GUIside *gui_side)
 	fputc(*text, gui_side->to_child);
 	fflush(gui_side->to_child);
 
-	if (*text == 'Y' || *text == 'N' || *text == 'Q')
+	if (*text == 'Y' || *text == 'N')
 		SENSITIVE_YESNO(gui_side, FALSE);
+	if (*text == 'Q')
+	{
+		gtk_widget_set_sensitive(gui_side->quiet,
+			!gtk_toggle_button_get_active(
+				GTK_TOGGLE_BUTTON(gui_side->quiet_flag)));
+	}
+}
+
+static void read_new_entry_text(void)
+{
+	int	len;
+	char	c;
+	GString	*new;
+
+	new = g_string_new(NULL);
+
+	for (;;)
+	{
+		len = read(from_parent, &c, 1);
+		if (len != 1)
+		{
+			fprintf(stderr, "read() error: %s\n",
+					g_strerror(errno));
+			_exit(1);	/* Parent died? */
+		}
+
+		if (c == '\n')
+			break;
+		g_string_append_c(new, c);
+	}
+
+	g_free(new_entry_string);
+	new_entry_string = new->str;
+	g_string_free(new, FALSE);
 }
 
 static void process_flag(char flag)
@@ -641,6 +741,9 @@ static void process_flag(char flag)
 		case 'B':
 			o_brief = !o_brief;
 			break;
+		case 'E':
+			read_new_entry_text();
+			break;
 		default:
 			g_string_sprintf(message,
 					"!ERROR: Bad message '%c'\n", flag);
@@ -679,34 +782,6 @@ static void check_flags(void)
 	}
 }
 
-static void read_new_entry_text(void)
-{
-	int	len;
-	char	c;
-	GString	*new;
-
-	new = g_string_new(NULL);
-
-	for (;;)
-	{
-		len = read(from_parent, &c, 1);
-		if (len != 1)
-		{
-			fprintf(stderr, "read() error: %s\n",
-					g_strerror(errno));
-			_exit(1);	/* Parent died? */
-		}
-
-		if (c == '\n')
-			break;
-		g_string_append_c(new, c);
-	}
-
-	g_free(new_entry_string);
-	new_entry_string = new->str;
-	g_string_free(new, FALSE);
-}
-
 /* Read until the user sends a reply. If ignore_quiet is TRUE then
  * the user MUST click Yes or No, else treat quiet on as Yes.
  * If the user needs prompting then does send().
@@ -715,16 +790,14 @@ static gboolean reply(int fd, gboolean ignore_quiet)
 {
 	ssize_t len;
 	char retval;
-	gboolean asked = FALSE;
 
-	while (ignore_quiet || !quiet)
+	if (quiet && !ignore_quiet)
+		return TRUE;
+
+	send();
+
+	while (1)
 	{
-		if (!asked)
-		{
-			send();
-			asked = TRUE;
-		}
-
 		len = read(fd, &retval, 1);
 		if (len != 1)
 		{
@@ -735,14 +808,6 @@ static gboolean reply(int fd, gboolean ignore_quiet)
 
 		switch (retval)
 		{
-			case 'Q':
-				quiet = !quiet;
-				if (ignore_quiet)
-				{
-					g_string_assign(message, "?");
-					send();
-				}
-				break;
 			case 'Y':
 				g_string_sprintf(message, "' %s\n", _("Yes"));
 				send();
@@ -751,21 +816,11 @@ static gboolean reply(int fd, gboolean ignore_quiet)
 				g_string_sprintf(message, "' %s\n", _("No"));
 				send();
 				return FALSE;
-			case 'E':
-				read_new_entry_text();
-				break;
 			default:
 				process_flag(retval);
 				break;
 		}
 	}
-
-	if (asked)
-	{
-		g_string_sprintf(message, "' %s\n", _("Quiet"));
-		send();
-	}
-	return TRUE;
 }
 
 static void destroy_action_window(GtkWidget *widget, gpointer data)
@@ -805,17 +860,22 @@ static void destroy_action_window(GtkWidget *widget, gpointer data)
  *
  * If autoq then automatically selects 'Quiet'.
  */
-static GUIside *start_action(gpointer data, ActionChild *func, gboolean autoq)
+static GUIside *start_action_with_options(gpointer data, ActionChild *func,
+					  gboolean autoq,
+					  int force, int brief, int recurse)
 {
 	int		filedes[4];	/* 0 and 2 are for reading */
 	GUIside		*gui_side;
 	int		child;
-	GtkWidget	*vbox, *button, *scrollbar, *actions;
+	GtkWidget	*vbox, *button, *scrollbar, *actions, *text;
 	struct sigaction act;
+#ifdef GTK2
+	GtkWidget	*frame;
+#endif
 
 	if (pipe(filedes))
 	{
-		report_error(PROJECT, g_strerror(errno));
+		report_error("pipe: %s", g_strerror(errno));
 		return NULL;
 	}
 
@@ -823,19 +883,19 @@ static GUIside *start_action(gpointer data, ActionChild *func, gboolean autoq)
 	{
 		close(filedes[0]);
 		close(filedes[1]);
-		report_error(PROJECT, g_strerror(errno));
+		report_error("pipe: %s", g_strerror(errno));
 		return NULL;
 	}
 
-	o_force = option_get_int("action_force");
-	o_brief = option_get_int("action_brief");
-	o_recurse = option_get_int("action_recurse");
+	o_force = force;
+	o_brief = brief;
+	o_recurse = recurse;
 
 	child = fork();
 	switch (child)
 	{
 		case -1:
-			report_error(PROJECT, g_strerror(errno));
+			report_error("fork: %s", g_strerror(errno));
 			return NULL;
 		case 0:
 			/* We are the child */
@@ -884,6 +944,7 @@ static GUIside *start_action(gpointer data, ActionChild *func, gboolean autoq)
 	gtk_container_add(GTK_CONTAINER(gui_side->window), vbox);
 
 	gui_side->dir = gtk_label_new(_("<dir>"));
+	gtk_widget_set_usize(gui_side->dir, 8, -1);
 	gui_side->next_dir = NULL;
 	gtk_misc_set_alignment(GTK_MISC(gui_side->dir), 0.5, 0.5);
 	gtk_box_pack_start(GTK_BOX(vbox), gui_side->dir, FALSE, TRUE, 0);
@@ -891,23 +952,46 @@ static GUIside *start_action(gpointer data, ActionChild *func, gboolean autoq)
 	gui_side->log_hbox = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), gui_side->log_hbox, TRUE, TRUE, 4);
 
-	gui_side->log = gtk_text_new(NULL, NULL);
-	gtk_widget_set_usize(gui_side->log, 400, 100);
-	gtk_box_pack_start(GTK_BOX(gui_side->log_hbox),
-				gui_side->log, TRUE, TRUE, 0);
-	scrollbar = gtk_vscrollbar_new(GTK_TEXT(gui_side->log)->vadj);
+#ifdef GTK2
+	frame = gtk_frame_new(NULL);
+	gtk_box_pack_start(GTK_BOX(gui_side->log_hbox), frame, TRUE, TRUE, 0);
+	
+	text = gtk_text_view_new();
+	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
+	gtk_container_add(GTK_CONTAINER(frame), text);
+
+	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_WORD);
+	gui_side->log = text;
+	scrollbar = gtk_vscrollbar_new(NULL);
+	gtk_widget_set_scroll_adjustments(text, NULL,
+			gtk_range_get_adjustment(GTK_RANGE(scrollbar)));
+	gtk_text_buffer_create_tag(
+			gtk_text_view_get_buffer(GTK_TEXT_VIEW(gui_side->log)),
+			"red", "foreground", "red",
+			NULL);
+	gtk_text_view_set_editable(GTK_TEXT_VIEW(text), FALSE);
+	gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(text), FALSE);
+#else
+	text = gui_side->log = gtk_text_new(NULL, NULL);
+	scrollbar = gtk_vscrollbar_new(GTK_TEXT(text)->vadj);
+	gtk_box_pack_start(GTK_BOX(gui_side->log_hbox), text, TRUE, TRUE, 0);
+#endif
+	gtk_widget_set_usize(text, 400, 100);
+
 	gtk_box_pack_start(GTK_BOX(gui_side->log_hbox),
 				scrollbar, FALSE, TRUE, 0);
 
 	actions = gtk_hbox_new(TRUE, 4);
 	gtk_box_pack_start(GTK_BOX(vbox), actions, FALSE, TRUE, 0);
+	gui_side->flag_box = gtk_hbox_new(FALSE, 16);
+	gtk_box_pack_start(GTK_BOX(vbox), gui_side->flag_box, FALSE, TRUE, 2);
 
-	gui_side->quiet = button = gtk_toggle_button_new_with_label(_("Quiet"));
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), autoq);
-	gtk_object_set_data(GTK_OBJECT(button), "send-code", "Q");
-	gtk_box_pack_start(GTK_BOX(actions), button, TRUE, TRUE, 0);
+	gui_side->quiet = button = gtk_button_new_with_label(_("Quiet"));
+	gtk_misc_set_padding(GTK_MISC(GTK_BIN(button)->child), 12, 0);
+	gtk_box_pack_start(GTK_BOX(gui_side->flag_box), button, FALSE, TRUE, 0);
 	gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			GTK_SIGNAL_FUNC(button_reply), gui_side);
+			GTK_SIGNAL_FUNC(quiet_clicked), gui_side);
+
 	gui_side->yes = button = gtk_button_new_with_label(_("Yes"));
 	gtk_object_set_data(GTK_OBJECT(button), "send-code", "Y");
 	gtk_box_pack_start(GTK_BOX(actions), button, TRUE, TRUE, 0);
@@ -931,19 +1015,30 @@ static GUIside *start_action(gpointer data, ActionChild *func, gboolean autoq)
 						message_from_child,
 						gui_side);
 
+	gui_side->quiet_flag = add_toggle(gui_side,
+				_("Quiet"), _("Don't confirm every operation"),
+				"Q", autoq);
+	gtk_widget_set_sensitive(gui_side->quiet, !autoq);
+
 	return gui_side;
 }
 
+static GUIside *start_action(gpointer data, ActionChild *func, gboolean autoq)
+{
+	return start_action_with_options(data, func, autoq,
+					 option_get_int("action_force"),
+					 option_get_int("action_brief"),
+					 option_get_int("action_recurse"));
+}
+
 /* 			ACTIONS ON ONE ITEM 			*/
 
 /* These may call themselves recursively, or ask questions, etc.
  * TRUE iff the directory containing dest_path needs to be rescanned.
  */
 
-/* dest_path is the dir containing src_path.
- * Updates the global size_tally, file_counter and dir_counter.
- */
-static gboolean do_usage(char *src_path, char *dest_path)
+/* Updates the global size_tally, file_counter and dir_counter */
+static gboolean do_usage(char *src_path, char *unused)
 {
 	struct 		stat info;
 
@@ -975,6 +1070,8 @@ static gboolean do_usage(char *src_path, char *dest_path)
 			g_free(safe_path);
 		}
 	}
+	else
+		file_counter++;
 
 	return FALSE;
 }
@@ -1366,27 +1463,27 @@ static gboolean do_copy2(char *path, char *dest)
 	}
 	else if (S_ISLNK(info.st_mode))
 	{
-		char	target[MAXPATHLEN + 1];
-		int	count;
+		char	*target;
 
 		/* Not all versions of cp(1) can make symlinks,
 		 * so we special-case it.
 		 */
 
-		count = readlink(path, target, sizeof(target) - 1);
-		if (count < 0)
+		target = readlink_dup(path);
+		if (target)
 		{
-			send_error();
-			retval = FALSE;
-		}
-		else
-		{
-			target[count] = '\0';
 			if (symlink(target, dest_path))
 			{
 				send_error();
 				retval = FALSE;
 			}
+
+			g_free(target);
+		}
+		else
+		{
+			send_error();
+			retval = FALSE;
 		}
 	}
 	else
@@ -1397,7 +1494,9 @@ static gboolean do_copy2(char *path, char *dest)
 
 		if (error)
 		{
-			g_string_sprintf(message, _("!ERROR: %s\n"), error);
+			g_string_sprintf(message, _("!%s\nFailed to copy '%s'"),
+					error, path);
+			g_free(error);
 			send();
 			retval = FALSE;
 		}
@@ -1414,6 +1513,7 @@ static gboolean do_move2(char *path, char *dest)
 	char		*argv[] = {"mv", "-f", NULL, NULL, NULL};
 	struct stat	info2;
 	gboolean	is_dir;
+	char            *err;
 
 	check_flags();
 
@@ -1470,7 +1570,17 @@ static gboolean do_move2(char *path, char *dest)
 	argv[2] = path;
 	argv[3] = dest_path;
 
-	if (fork_exec_wait(argv) == 0)
+	err = fork_exec_wait(argv);
+	if (err)
+	{
+		g_string_sprintf(message,
+				_("!%s\nFailed to move %s as %s\n"),
+				err, path, dest_path);
+		send();
+		retval = FALSE;
+		g_free(err);
+	}
+	else
 	{
 		char	*leaf;
 
@@ -1483,19 +1593,12 @@ static gboolean do_move2(char *path, char *dest)
 		g_string_sprintf(message, "+%s", path);
 		g_string_truncate(message, leaf - path + 1);
 		send();
-		if (is_dir) {
+		if (is_dir)
+		{
 			g_string_sprintf(message, "m%s", path);
 			send();
 		}
 	}
-	else
-	{
-		g_string_sprintf(message,
-				_("!ERROR: Failed to move %s as %s\n"),
-				path, dest_path);
-		send();
-		retval = FALSE;
-	}
 
 	return retval;
 }
@@ -1533,17 +1636,10 @@ static gboolean do_move(char *path, char *dest)
 static gboolean do_link(char *path, char *dest)
 {
 	char		*dest_path;
-	char		*leaf;
 
 	check_flags();
 
-	leaf = strrchr(path, '/');
-	if (!leaf)
-		leaf = path;		/* Error? */
-	else
-		leaf++;
-
-	dest_path = make_path(dest, leaf)->str;
+	dest_path = make_dest_path(path, dest);
 
 	if (quiet)
 	{
@@ -1572,6 +1668,7 @@ static gboolean do_link(char *path, char *dest)
 static void do_mount(guchar *path, gboolean mount)
 {
 	char		*argv[3] = {NULL, NULL, NULL};
+	char            *err;
 
 	check_flags();
 
@@ -1596,7 +1693,16 @@ static void do_mount(guchar *path, gboolean mount)
 			return;
 	}
 
-	if (fork_exec_wait(argv) == 0)
+	err = fork_exec_wait(argv);
+	if (err)
+	{
+		g_string_sprintf(message, mount ?
+			_("!%s\nMount failed\n") :
+			_("!%s\nUnmount failed\n"), err);
+		send();
+		g_free(err);
+	}
+	else
 	{
 		g_string_sprintf(message, "M%s", path);
 		send();
@@ -1606,13 +1712,6 @@ static void do_mount(guchar *path, gboolean mount)
 			send();
 		}
 	}
-	else
-	{
-		g_string_sprintf(message, mount ?
-			_("!ERROR: Mount failed\n") :
-			_("!ERROR: Unmount failed\n"));
-		send();
-	}
 }
 
 /*			CHILD MAIN LOOPS			*/
@@ -1622,34 +1721,27 @@ static void do_mount(guchar *path, gboolean mount)
 /* We use a double for total size in order to count beyond 4Gb */
 static void usage_cb(gpointer data)
 {
-	FilerWindow *filer_window = (FilerWindow *) data;
-	Collection *collection = filer_window->collection;
-	DirItem   *item;
-	int	left = collection->number_selected;
-	int	i = -1;
+	GList *paths = (GList *) data;
 	double	total_size = 0;
 	gchar	*tmp;
 
-	dir_counter= file_counter= 0;
+	dir_counter = file_counter = 0;
 
-	send_dir(filer_window->path);
-
-	while (left > 0)
+	for (; paths; paths = paths->next)
 	{
-		i++;
-		if (!collection->items[i].selected)
-			continue;
-		item = (DirItem *) collection->items[i].data;
+		guchar	*path = (guchar *) paths->data;
+
+		send_dir(path);
+
 		size_tally = 0;
-		do_usage(make_path(filer_window->path,
-					item->leafname)->str,
-					filer_window->path);
+		
+		do_usage(path, NULL);
+
 		g_string_sprintf(message, "'%s: %s\n",
-				item->leafname,
-				format_double_size((unsigned long) size_tally));
+				g_basename(path),
+				format_double_size(size_tally));
 		send();
 		total_size += size_tally;
-		left--;
 	}
 
 	g_string_sprintf(message, _("'\nTotal: %s ("),
@@ -1747,29 +1839,18 @@ static void delete_cb(gpointer data)
 
 static void find_cb(gpointer data)
 {
-	FilerWindow *filer_window = (FilerWindow *) data;
-	Collection *collection = filer_window->collection;
-	DirItem   *item;
-	int	left;
-	int	i;
-
-	send_dir(filer_window->path);
+	GList *all_paths = (GList *) data;
+	GList *paths;
 
 	while (1)
 	{
-		left = collection->number_selected;
-		i = -1;
-
-		while (left > 0)
+		for (paths = all_paths; paths; paths = paths->next)
 		{
-			i++;
-			if (!collection->items[i].selected)
-				continue;
-			item = (DirItem *) collection->items[i].data;
-			do_find(make_path(filer_window->path,
-						item->leafname)->str,
-					NULL);
-			left--;
+			guchar	*path = (guchar *) paths->data;
+
+			send_dir(path);
+
+			do_find(path, NULL);
 		}
 
 		g_string_assign(message, _("?Another search?"));
@@ -1785,34 +1866,29 @@ static void find_cb(gpointer data)
 
 static void chmod_cb(gpointer data)
 {
-	FilerWindow *filer_window = (FilerWindow *) data;
-	Collection *collection = filer_window->collection;
-	DirItem   *item;
-	int	left = collection->number_selected;
-	int	i = -1;
+	GList *paths = (GList *) data;
 
-	send_dir(filer_window->path);
-
-	while (left > 0)
+	for (; paths; paths = paths->next)
 	{
-		i++;
-		if (!collection->items[i].selected)
-			continue;
-		item = (DirItem *) collection->items[i].data;
-		if (item->flags & ITEM_FLAG_SYMLINK)
+		guchar	*path = (guchar *) paths->data;
+		struct stat info;
+
+		send_dir(path);
+
+		if (mc_stat(path, &info) != 0)
+			send_error();
+		else if (S_ISLNK(info.st_mode))
 		{
 			g_string_sprintf(message,
 					_("!'%s' is a symbolic link\n"),
-					item->leafname);
+					g_basename(path));
 			send();
 		}
-		else if (do_chmod(make_path(filer_window->path,
-						item->leafname)->str, NULL))
+		else if (do_chmod(path, NULL))
 		{
-			g_string_sprintf(message, "+%s", filer_window->path);
+			g_string_sprintf(message, "+%s", path); /* XXX */
 			send();
 		}
-		left--;
 	}
 	
 	g_string_sprintf(message, _("'\nDone\n"));
@@ -1840,37 +1916,39 @@ static void list_cb(gpointer data)
 	send();
 }
 
-static void add_toggle(GUIside *gui_side, guchar *label, guchar *code,
-			gboolean active)
+static GtkWidget *add_toggle(GUIside *gui_side,
+			     guchar *label, guchar *tip,
+			     guchar *code,
+			     gboolean active)
 {
 	GtkWidget	*check;
 
 	check = gtk_check_button_new_with_label(label);
+	gtk_tooltips_set_tip(tooltips, check, tip, NULL);
 	gtk_object_set_data(GTK_OBJECT(check), "send-code", code);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), active);
 	gtk_signal_connect(GTK_OBJECT(check), "clicked",
 			GTK_SIGNAL_FUNC(button_reply), gui_side);
-	gtk_box_pack_start(GTK_BOX(gui_side->vbox), check, FALSE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(gui_side->flag_box), check, FALSE, TRUE, 0);
+
+	return check;
 }
 
 
 /*			EXTERNAL INTERFACE			*/
 
-void action_find(FilerWindow *filer_window)
+void action_find(GList *paths)
 {
 	GUIside		*gui_side;
-	Collection 	*collection;
 	GtkWidget	*hbox, *label, *scroller;
 	gchar		*titles[2];
 	
 	titles[0] = _("Name");
 	titles[1] = _("Directory");
 
-	collection = filer_window->collection;
-
-	if (collection->number_selected < 1)
+	if (!paths)
 	{
-		report_error(PROJECT, _("You need to select some items "
+		report_error(_("You need to select some items "
 				"to search through"));
 		return;
 	}
@@ -1879,7 +1957,7 @@ void action_find(FilerWindow *filer_window)
 		last_find_string = g_strdup("'core'");
 
 	new_entry_string = last_find_string;
-	gui_side = start_action(filer_window, find_cb, FALSE);
+	gui_side = start_action(paths, find_cb, FALSE);
 	if (!gui_side)
 		return;
 
@@ -1908,7 +1986,7 @@ void action_find(FilerWindow *filer_window)
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 4);
 	gui_side->default_string = &last_find_string;
 	gui_side->entry = gtk_entry_new();
-	gtk_widget_set_style(gui_side->entry, fixed_style);
+	gtk_widget_set_name(gui_side->entry, "fixed-font");
 	gtk_entry_set_text(GTK_ENTRY(gui_side->entry), last_find_string);
 	set_find_string_colour(gui_side->entry, last_find_string);
 	gtk_editable_select_region(GTK_EDITABLE(gui_side->entry), 0, -1);
@@ -1923,29 +2001,24 @@ void action_find(FilerWindow *filer_window)
 
 	gtk_window_set_title(GTK_WINDOW(gui_side->window), _("Find"));
 	gtk_window_set_focus(GTK_WINDOW(gui_side->window), gui_side->entry);
-	gtk_signal_connect_object(GTK_OBJECT(gui_side->entry), "activate",
-			GTK_SIGNAL_FUNC(gtk_button_clicked),
-			GTK_OBJECT(gui_side->quiet));
+	gtk_signal_connect(GTK_OBJECT(gui_side->entry), "activate",
+			GTK_SIGNAL_FUNC(find_return_pressed), gui_side);
 	number_of_windows++;
 	gtk_widget_show_all(gui_side->window);
 }
 
 /* Count disk space used by selected items */
-void action_usage(FilerWindow *filer_window)
+void action_usage(GList *paths)
 {
 	GUIside		*gui_side;
-	Collection 	*collection;
 
-	collection = filer_window->collection;
-
-	if (collection->number_selected < 1)
+	if (!paths)
 	{
-		report_error(PROJECT,
-			_("You need to select some items to count"));
+		report_error(_("You need to select some items to count"));
 		return;
 	}
 
-	gui_side = start_action(filer_window, usage_cb, TRUE);
+	gui_side = start_action(paths, usage_cb, TRUE);
 	if (!gui_side)
 		return;
 
@@ -1959,15 +2032,18 @@ void action_usage(FilerWindow *filer_window)
 /* Mount/unmount listed items (paths).
  * Free the list after this function returns.
  * If open_dir is TRUE and the dir is successfully mounted, open it.
+ * quiet can be -1 for default.
  */
-void action_mount(GList	*paths, gboolean open_dir)
+void action_mount(GList	*paths, gboolean open_dir, int quiet)
 {
 #ifdef DO_MOUNT_POINTS
 	GUIside		*gui_side;
 
+	if (quiet == -1)
+	 	quiet = option_get_int("action_mount");
+
 	mount_open_dir = open_dir;
-	gui_side = start_action(paths, mount_cb,
-					option_get_int("action_mount"));
+	gui_side = start_action(paths, mount_cb, quiet);
 	if (!gui_side)
 		return;
 
@@ -1976,22 +2052,19 @@ void action_mount(GList	*paths, gboolean open_dir)
 	number_of_windows++;
 	gtk_widget_show_all(gui_side->window);
 #else
-	report_error(PROJECT,
+	report_error(
 		_("ROX-Filer does not yet support mount points on your "
 			"system. Sorry."));
 #endif /* DO_MOUNT_POINTS */
 }
 
 /* Deletes all selected items in the window */
-void action_delete(FilerWindow *filer_window)
+void action_delete(GList *paths)
 {
 	GUIside		*gui_side;
-	GList		*paths;
-
-	paths = filer_selected_items(filer_window);
 
 	if (!remove_pinned_ok(paths))
-		goto out;
+		return;
 
 	gui_side = start_action(paths, delete_cb,
 					option_get_int("action_delete"));
@@ -2000,34 +2073,26 @@ void action_delete(FilerWindow *filer_window)
 
 	gtk_window_set_title(GTK_WINDOW(gui_side->window), _("Delete"));
 	add_toggle(gui_side,
-		_("Force - don't confirm deletion of non-writeable items"),
+		_("Force"), _("Don't confirm deletion of non-writeable items"),
 		"F", option_get_int("action_force"));
 	add_toggle(gui_side,
-		_("Brief - only log directories being deleted"),
+		_("Brief"), _("Only log directories being deleted"),
 		"B", option_get_int("action_brief"));
 
 	number_of_windows++;
 	gtk_widget_show_all(gui_side->window);
-
-out:
-	g_list_foreach(paths, (GFunc) g_free, NULL);
-	g_list_free(paths);
 }
 
 /* Change the permissions of the selected items */
-void action_chmod(FilerWindow *filer_window)
+void action_chmod(GList *paths)
 {
 	GUIside		*gui_side;
-	Collection 	*collection;
 	GtkWidget	*hbox, *label, *combo;
 	static GList	*presets = NULL;
 
-	collection = filer_window->collection;
-
-	if (collection->number_selected < 1)
+	if (!paths)
 	{
-		report_error(PROJECT,
-			_("You need to select the items "
+		report_error(_("You need to select the items "
 				"whose permissions you want to change"));
 		return;
 	}
@@ -2049,15 +2114,15 @@ void action_chmod(FilerWindow *filer_window)
 	if (!last_chmod_string)
 		last_chmod_string = g_strdup((guchar *) presets->data);
 	new_entry_string = last_chmod_string;
-	gui_side = start_action(filer_window, chmod_cb, FALSE);
+	gui_side = start_action(paths, chmod_cb, FALSE);
 	if (!gui_side)
 		return;
 
 	add_toggle(gui_side,
-		_("Brief - don't list processed files"),
+		_("Brief"), _("Don't list processed files"),
 		"B", option_get_int("action_brief"));
 	add_toggle(gui_side,
-		_("Recurse - also change contents of subdirectories"),
+		_("Recurse"), _("Also change contents of subdirectories"),
 		"R", option_get_int("action_recurse"));
 
 	hbox = gtk_hbox_new(FALSE, 0);
@@ -2092,15 +2157,20 @@ void action_chmod(FilerWindow *filer_window)
 	gtk_widget_show_all(gui_side->window);
 }
 
-/* If leaf is NULL then the copy has the same name as the original */
-void action_copy(GList *paths, char *dest, char *leaf)
+/* If leaf is NULL then the copy has the same name as the original.
+ * quiet can be -1 for default.
+ */
+void action_copy(GList *paths, char *dest, char *leaf, int quiet)
 {
 	GUIside		*gui_side;
 
+	if (quiet == -1)
+		quiet = option_get_int("action_copy");
+
 	action_dest = dest;
 	action_leaf = leaf;
 	action_do_func = do_copy;
-	gui_side = start_action(paths, list_cb, option_get_int("action_copy"));
+	gui_side = start_action(paths, list_cb, quiet);
 	if (!gui_side)
 		return;
 
@@ -2109,15 +2179,20 @@ void action_copy(GList *paths, char *dest, char *leaf)
 	gtk_widget_show_all(gui_side->window);
 }
 
-/* If leaf is NULL then the file is not renamed */
-void action_move(GList *paths, char *dest, char *leaf)
+/* If leaf is NULL then the file is not renamed.
+ * quiet can be -1 for default.
+ */
+void action_move(GList *paths, char *dest, char *leaf, int quiet)
 {
 	GUIside		*gui_side;
 
+	if (quiet == -1)
+		quiet = option_get_int("action_move");
+
 	action_dest = dest;
 	action_leaf = leaf;
 	action_do_func = do_move;
-	gui_side = start_action(paths, list_cb, option_get_int("action_move"));
+	gui_side = start_action(paths, list_cb, quiet);
 	if (!gui_side)
 		return;
 
@@ -2126,11 +2201,14 @@ void action_move(GList *paths, char *dest, char *leaf)
 	gtk_widget_show_all(gui_side->window);
 }
 
-void action_link(GList *paths, char *dest)
+/* If leaf is NULL then the link will have the same name */
+/* XXX: No quiet option here? */
+void action_link(GList *paths, char *dest, char *leaf)
 {
 	GUIside		*gui_side;
 
 	action_dest = dest;
+	action_leaf = leaf;
 	action_do_func = do_link;
 	gui_side = start_action(paths, list_cb, option_get_int("action_link"));
 	if (!gui_side)
@@ -2232,7 +2310,7 @@ static gboolean remove_pinned_ok(GList *paths)
 	}
 	
 	retval = get_choice(PROJECT, message->str,
-				2, _("OK"), _("Cancel")) == 0;
+				2, _("Cancel"), _("OK")) == 1;
 
 	g_string_free(message, TRUE);
 	
@@ -2241,6 +2319,7 @@ static gboolean remove_pinned_ok(GList *paths)
 
 void set_find_string_colour(GtkWidget *widget, guchar *string)
 {
+#ifndef GTK2
 	static GtkStyle *error_style = NULL;
 	FindCondition *cond;
 
@@ -2255,4 +2334,5 @@ void set_find_string_colour(GtkWidget *widget, guchar *string)
 
 	if (cond)
 		find_condition_free(cond);
+#endif
 }
