@@ -1,5 +1,5 @@
 /*
- * $Id: collection.c,v 1.61 2001/09/24 12:34:45 tal197 Exp $
+ * $Id: collection.c,v 1.76 2001/12/19 21:21:29 tal197 Exp $
  *
  * Collection - a GTK+ widget
  * Copyright (C) 2001, the ROX-Filer team.
@@ -30,6 +30,8 @@
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+#include "global.h"
+
 #include "collection.h"
 
 #define MIN_WIDTH 80
@@ -90,14 +92,13 @@ static void collection_init(Collection *object);
 static void collection_destroy(GtkObject *object);
 static void collection_finalize(GtkObject *object);
 static void collection_realize(GtkWidget *widget);
+static void collection_map(GtkWidget *widget);
 static gint collection_paint(Collection 	*collection,
 			     GdkRectangle 	*area);
 static void collection_size_request(GtkWidget 		*widget,
 				    GtkRequisition 	*requisition);
 static void collection_size_allocate(GtkWidget 		*widget,
 			     GtkAllocation 	*allocation);
-static void collection_set_style(GtkWidget *widget,
-                                 GtkStyle *previous_style);
 static void collection_set_adjustment(Collection 	*collection,
 				      GtkAdjustment 	*vadj);
 static void collection_set_arg(	GtkObject *object,
@@ -106,17 +107,7 @@ static void collection_set_arg(	GtkObject *object,
 static void collection_get_arg(	GtkObject *object,
 				GtkArg    *arg,
 				guint     arg_id);
-static void collection_adjustment(GtkAdjustment *adjustment,
-				  Collection    *collection);
-#ifndef GTK2
-static void collection_disconnect(GtkAdjustment *adjustment,
-				  Collection    *collection);
-#endif
-static void set_vadjustment(Collection *collection);
 static gint collection_expose(GtkWidget *widget, GdkEventExpose *event);
-static void scroll_by(Collection *collection, gint diff);
-static gint collection_button_press(GtkWidget      *widget,
-				    GdkEventButton *event);
 static void default_draw_item(GtkWidget *widget,
 				CollectionItem *data,
 				GdkRectangle *area,
@@ -136,7 +127,25 @@ static void cancel_wink(Collection *collection);
 static gint collection_key_press(GtkWidget *widget, GdkEventKey *event);
 static void get_visible_limits(Collection *collection, int *first, int *last);
 static void scroll_to_show(Collection *collection, int item);
+static void collection_item_set_selected(Collection *collection,
+                                         gint item,
+                                         gboolean selected,
+					 gboolean signal);
+#ifdef GTK2
+static gint collection_scroll_event(GtkWidget *widget, GdkEventScroll *event);
+#else
+static gint collection_scroll_event(GtkWidget *widget, GdkEventButton *event);
+#endif
+
 #ifndef GTK2
+static void collection_adjustment(GtkAdjustment *adjustment,
+				  Collection    *collection);
+static void collection_set_style(GtkWidget *widget,
+                                 GtkStyle *previous_style);
+static void collection_disconnect(GtkAdjustment *adjustment,
+				  Collection    *collection);
+static void scroll_by(Collection *collection, gint diff);
+static void set_vadjustment(Collection *collection);
 static void draw_focus(GtkWidget *widget);
 static void collection_draw(GtkWidget *widget, GdkRectangle *area);
 static gint focus_in(GtkWidget *widget, GdkEventFocus *event);
@@ -251,18 +260,25 @@ static void collection_class_init(CollectionClass *class)
 	widget_class->expose_event = collection_expose;
 	widget_class->size_request = collection_size_request;
 	widget_class->size_allocate = collection_size_allocate;
-	widget_class->style_set = collection_set_style;
 
 	widget_class->key_press_event = collection_key_press;
-	widget_class->button_press_event = collection_button_press;
+	
 	widget_class->motion_notify_event = collection_motion_notify;
+	widget_class->map = collection_map;
+#ifdef GTK2
+	widget_class->scroll_event = collection_scroll_event;
+#else
+	widget_class->button_press_event = collection_scroll_event;
+#endif
+
 #ifndef GTK2
+	widget_class->style_set = collection_set_style; /* XXX: Test for 2.0 */
 	widget_class->focus_in_event = focus_in;
 	widget_class->focus_out_event = focus_out;
 	widget_class->draw = collection_draw;
 	widget_class->draw_focus = draw_focus;
 #endif
-
+	
 	object_class->set_arg = collection_set_arg;
 	object_class->get_arg = collection_get_arg;
 
@@ -316,31 +332,30 @@ static void collection_init(Collection *object)
 	object->item_width = 64;
 	object->item_height = 64;
 	object->vadj = NULL;
+#ifndef GTK2
 	object->paint_level = PAINT_OVERWRITE;
 	object->last_scroll = 0;
+#endif
 	object->bg_gc = NULL;
 
-	object->items = g_malloc(sizeof(CollectionItem) * MINIMUM_ITEMS);
+	object->items = g_new(CollectionItem, MINIMUM_ITEMS);
 	object->cursor_item = -1;
 	object->cursor_item_old = -1;
 	object->wink_item = -1;
+	object->wink_on_map = -1;
 	object->array_size = MINIMUM_ITEMS;
 	object->draw_item = default_draw_item;
 	object->test_point = default_test_point;
+	object->free_item = NULL;
 
 	object->auto_scroll = -1;
 	
 	return;
 }
 
-GtkWidget* collection_new(GtkAdjustment *vadj)
+GtkWidget* collection_new(void)
 {
-	if (vadj)
-		g_return_val_if_fail(GTK_IS_ADJUSTMENT(vadj), NULL);
-
-	return GTK_WIDGET(gtk_widget_new(collection_get_type(),
-				"vadjustment", vadj,
-				NULL));
+	return GTK_WIDGET(gtk_widget_new(collection_get_type(), NULL));
 }
 
 /* Note: The draw_item call gives the maximum area that can be
@@ -374,7 +389,9 @@ void collection_set_functions(Collection *collection,
 
 	if (GTK_WIDGET_REALIZED(widget))
 	{
+#ifndef GTK2
 		collection->paint_level = PAINT_CLEAR;
+#endif
 		gtk_widget_queue_clear(widget);
 	}
 }
@@ -391,11 +408,7 @@ static void collection_destroy(GtkObject *object)
 
 	collection = COLLECTION(object);
 
-	if (collection->wink_item != -1)
-	{
-		collection->wink_item = -1;
-		gtk_timeout_remove(collection->wink_timeout);
-	}
+	collection_clear(collection);
 
 	if (collection->auto_scroll != -1)
 	{
@@ -411,8 +424,10 @@ static void collection_destroy(GtkObject *object)
 
 	if (collection->vadj)
 	{
+#ifndef GTK2
 		gtk_signal_disconnect_by_data(GTK_OBJECT(collection->vadj),
 				collection);
+#endif
 		gtk_object_unref(GTK_OBJECT(collection->vadj));
 		collection->vadj = NULL;
 	}
@@ -428,9 +443,25 @@ static void collection_finalize(GtkObject *object)
 
 	collection = COLLECTION(object);
 
+	g_return_if_fail(collection->number_of_items == 0);
+
 	g_free(collection->items);
 }
 
+static void collection_map(GtkWidget *widget)
+{
+	Collection *collection = COLLECTION(widget);
+	
+	if (GTK_WIDGET_CLASS(parent_class)->map)
+		(*GTK_WIDGET_CLASS(parent_class)->map)(widget);
+
+	if (collection->wink_on_map >= 0)
+	{
+		collection_wink_item(collection, collection->wink_on_map);
+		collection->wink_on_map = -1;
+	}
+}
+
 static GdkGC *create_bg_gc(GtkWidget *widget)
 {
 	GdkGCValues values;
@@ -473,7 +504,7 @@ static void collection_realize(GtkWidget *widget)
 
 	attributes_mask = GDK_WA_X | GDK_WA_Y |
 				GDK_WA_VISUAL | GDK_WA_COLORMAP;
-	widget->window = gdk_window_new(widget->parent->window,
+	widget->window = gdk_window_new(gtk_widget_get_parent_window(widget),
 			&attributes, attributes_mask);
 
 	widget->style = gtk_style_attach(widget->style, widget->window);
@@ -485,10 +516,12 @@ static void collection_realize(GtkWidget *widget)
 	if (widget->style->bg_pixmap[GTK_STATE_NORMAL])
 		collection->bg_gc = create_bg_gc(widget);
 
+#ifndef GTK2
 	/* Try to stop everything flickering horribly */
 	gdk_window_set_static_gravities(widget->window, TRUE);
 
 	set_vadjustment(collection);
+#endif
 
 	bg = &widget->style->bg[GTK_STATE_NORMAL];
 	fg = &widget->style->fg[GTK_STATE_NORMAL];
@@ -503,8 +536,20 @@ static void collection_realize(GtkWidget *widget)
 static void collection_size_request(GtkWidget *widget,
 				GtkRequisition *requisition)
 {
+#ifdef GTK2
+	Collection *collection = COLLECTION(widget);
+	int	rows, cols = collection->columns;
+
+	/* We ask for the total size we need; our containing viewport
+	 * will deal with scrolling.
+	 */
+	requisition->width = MIN_WIDTH;
+	rows = (collection->number_of_items + cols - 1) / cols;
+	requisition->height = rows * collection->item_height;
+#else
 	requisition->width = MIN_WIDTH;
 	requisition->height = MIN_HEIGHT;
+#endif
 }
 
 static void collection_size_allocate(GtkWidget *widget,
@@ -512,6 +557,7 @@ static void collection_size_allocate(GtkWidget *widget,
 {
 	Collection 	*collection;
 	int		old_columns;
+	gboolean	cursor_visible = FALSE;
 
 	g_return_if_fail(widget != NULL);
 	g_return_if_fail(IS_COLLECTION(widget));
@@ -519,10 +565,22 @@ static void collection_size_allocate(GtkWidget *widget,
 
 	collection = COLLECTION(widget);
 
+	if (collection->cursor_item != -1)
+	{
+		int	first, last;
+		int	crow = collection->cursor_item / collection->columns;
+
+		get_visible_limits(collection, &first, &last);
+
+		cursor_visible = crow >= first && crow <= last;
+	}
+
 	old_columns = collection->columns;
+#ifndef GTK2
 	if (widget->allocation.x != allocation->x
 		|| widget->allocation.y != allocation->y)
 		collection->paint_level = PAINT_CLEAR;
+#endif
 
 	widget->allocation = *allocation;
 
@@ -536,6 +594,7 @@ static void collection_size_allocate(GtkWidget *widget,
 				allocation->x, allocation->y,
 				allocation->width, allocation->height);
 
+#ifndef GTK2
 		/* Force a redraw if the number of columns has changed
 		 * or we have a background pixmap (!).
 		 */
@@ -546,12 +605,22 @@ static void collection_size_allocate(GtkWidget *widget,
 		}
 
 		set_vadjustment(collection);
+#endif
 
-		if (collection->cursor_item != -1)
+		if (cursor_visible)
 			scroll_to_show(collection, collection->cursor_item);
 	}
+
+#ifdef GTK2
+	if (old_columns != collection->columns)
+	{
+		/* Need to go around again... */
+		gtk_widget_queue_resize(widget);
+	}
+#endif
 }
 
+#ifndef GTK2
 static void collection_set_style(GtkWidget *widget,
                                  GtkStyle *previous_style)
 {
@@ -578,11 +647,16 @@ static void collection_set_style(GtkWidget *widget,
 			collection->bg_gc = create_bg_gc(widget);
 	}
 }
+#endif
 
 static void clear_area(Collection *collection, GdkRectangle *area)
 {
 	GtkWidget	*widget = GTK_WIDGET(collection);
+#ifdef GTK2
+	int		scroll = 0;
+#else
 	int		scroll = collection->vadj->value;
+#endif
 
 	if (collection->bg_gc)
 	{
@@ -607,7 +681,11 @@ static void collection_get_item_area(Collection *collection,
 					GdkRectangle *area)
 
 {
+#ifdef GTK2
+	int		scroll = 0;
+#else
 	int scroll = collection->vadj->value;	/* (round to int) */
+#endif
 
 	area->x = col * collection->item_width;
 	area->y = row * collection->item_height - scroll;
@@ -621,21 +699,22 @@ static void collection_get_item_area(Collection *collection,
 static gint collection_paint(Collection 	*collection,
 			     GdkRectangle 	*area)
 {
-	GdkRectangle	whole, item_area;
-	GtkWidget	*widget;
+	GdkRectangle	item_area;
 	int		row, col;
 	int		item;
-	int		scroll;
+	int		scroll = 0;
 	int		start_row, last_row;
 	int		start_col, last_col;
 	int		phys_last_col;
 	GdkRectangle 	clip;
+#ifndef GTK2
+	GtkWidget	*widget;
+	GdkRectangle	whole;
 	gint     	width, height;
 
-	scroll = collection->vadj->value;
-
 	widget = GTK_WIDGET(collection);
 
+	scroll = collection->vadj->value;
 	gdk_window_get_size(widget->window, &width, &height);
 	whole.x = 0;
 	whole.y = 0;
@@ -652,6 +731,7 @@ static gint collection_paint(Collection 	*collection,
 
 		collection->paint_level = PAINT_NORMAL;
 	}
+#endif
 
 	/* Calculate the ranges to plot */
 	start_row = (area->y + scroll) / collection->item_height;
@@ -694,6 +774,7 @@ static gint collection_paint(Collection 	*collection,
 
 	item = row * collection->columns + col;
 
+	/* g_print("[ paint %d..%d ]\n", row, last_row); */
 	while ((item == 0 || item < collection->number_of_items)
 			&& row <= last_row)
 	{
@@ -771,8 +852,8 @@ static void collection_set_arg(	GtkObject *object,
 	}
 }
 
-static void collection_set_adjustment(  Collection    *collection,
-					GtkAdjustment *vadj)
+static void collection_set_adjustment(Collection    *collection,
+				      GtkAdjustment *vadj)
 {
 	if (vadj)
 		g_return_if_fail (GTK_IS_ADJUSTMENT (vadj));
@@ -782,8 +863,10 @@ static void collection_set_adjustment(  Collection    *collection,
 							 0.0, 0.0, 0.0));
 	if (collection->vadj && (collection->vadj != vadj))
 	{
+#ifndef GTK2
 		gtk_signal_disconnect_by_data(GTK_OBJECT(collection->vadj),
 						collection);
+#endif
 		gtk_object_unref(GTK_OBJECT(collection->vadj));
 	}
 
@@ -793,6 +876,7 @@ static void collection_set_adjustment(  Collection    *collection,
 		gtk_object_ref(GTK_OBJECT(collection->vadj));
 		gtk_object_sink(GTK_OBJECT(collection->vadj));
 
+#ifndef GTK2
 		gtk_signal_connect(GTK_OBJECT(collection->vadj),
 				"changed",
 				(GtkSignalFunc) collection_adjustment,
@@ -801,14 +885,13 @@ static void collection_set_adjustment(  Collection    *collection,
 				"value_changed",
 				(GtkSignalFunc) collection_adjustment,
 				collection);
-#ifndef GTK2
 		/* Is this used for anything? */
 		gtk_signal_connect(GTK_OBJECT(collection->vadj),
 				"disconnect",
 				(GtkSignalFunc) collection_disconnect,
 				collection);
-#endif
 		collection_adjustment(vadj, collection);
+#endif
 	}
 }
 
@@ -831,6 +914,7 @@ static void collection_get_arg(	GtkObject *object,
 	}
 }
 
+#ifndef GTK2
 /* Something about the adjustment has changed */
 static void collection_adjustment(GtkAdjustment *adjustment,
 				  Collection    *collection)
@@ -860,7 +944,6 @@ static void collection_adjustment(GtkAdjustment *adjustment,
 	}
 }
 
-#ifndef GTK2
 static void collection_disconnect(GtkAdjustment *adjustment,
 				  Collection    *collection)
 {
@@ -871,7 +954,6 @@ static void collection_disconnect(GtkAdjustment *adjustment,
 
 	collection_set_adjustment(collection, NULL);
 }
-#endif
 
 static void set_vadjustment(Collection *collection)
 {	
@@ -910,17 +992,6 @@ static void set_vadjustment(Collection *collection)
 	gtk_signal_emit_by_name(GTK_OBJECT(collection->vadj), "changed");
 }
 
-/* Change the adjustment by this amount. Bounded. */
-static void diff_vpos(Collection *collection, int diff)
-{
-	int	value = collection->vadj->value + diff;
-
-	value = CLAMP(value, 0,
-			collection->vadj->upper - collection->vadj->page_size);
-	gtk_adjustment_set_value(collection->vadj, value);
-}
-
-#ifndef GTK2
 static void collection_draw(GtkWidget *widget, GdkRectangle *area)
 {
 	Collection    *collection;
@@ -931,16 +1002,20 @@ static void collection_draw(GtkWidget *widget, GdkRectangle *area)
 
 	collection = COLLECTION(widget);
 
-	/* This doesn't always work - I think Gtk+ may be doing some
-	 * kind of expose-event compression...
-	 * 29/9/2000: Turned expose_events on in copy_area... maybe
-	 * we can use this again? Try after 1.0.0!
-	if (collection->paint_level > PAINT_NORMAL)
-	*/
-		collection_paint(collection, area);
+	collection_paint(collection, area);
 }
 #endif
 
+/* Change the adjustment by this amount. Bounded. */
+static void diff_vpos(Collection *collection, int diff)
+{
+	int	value = collection->vadj->value + diff;
+
+	value = CLAMP(value, 0,
+			collection->vadj->upper - collection->vadj->page_size);
+	gtk_adjustment_set_value(collection->vadj, value);
+}
+
 static gint collection_expose(GtkWidget *widget, GdkEventExpose *event)
 {
 	Collection	*collection;
@@ -951,13 +1026,16 @@ static gint collection_expose(GtkWidget *widget, GdkEventExpose *event)
 
 	collection = COLLECTION(widget);
 
+#ifndef GTK2
 	clear_area(collection, &event->area);
+#endif
 
 	collection_paint(collection, &event->area);
 
 	return FALSE;
 }
 
+#ifndef GTK2
 /* Positive makes the contents go move up the screen */
 static void scroll_by(Collection *collection, gint diff)
 {
@@ -1029,6 +1107,7 @@ static void scroll_by(Collection *collection, gint diff)
 	clear_area(collection, &new_area);
 	collection_paint(collection, NULL);
 }
+#endif
 
 static void resize_arrays(Collection *collection, guint new_size)
 {
@@ -1085,11 +1164,19 @@ static gint collection_key_press(GtkWidget *widget, GdkEventKey *event)
 				MAX((gint) collection->number_of_items - 1, 0));
 			break;
 		case GDK_Page_Up:
-			collection_move_cursor(collection, -10, 0);
+		  {
+		        int first, last;
+		       	get_visible_limits(collection, &first, &last);
+			collection_move_cursor(collection, first - last - 1, 0);
 			break;
+		  }
 		case GDK_Page_Down:
-			collection_move_cursor(collection, 10, 0);
+		  {
+		        int first, last;
+		       	get_visible_limits(collection, &first, &last);
+			collection_move_cursor(collection, last - first + 1, 0);
 			break;
+		  }
 		case GDK_Escape:
 			collection_set_cursor_item(collection, -1);
 			collection_clear_selection(collection);
@@ -1110,11 +1197,15 @@ static gint collection_key_press(GtkWidget *widget, GdkEventKey *event)
 	return TRUE;
 }
 
-static gint collection_button_press(GtkWidget      *widget,
-				    GdkEventButton *event)
+/* Wheel mouse scrolling */
+#ifdef GTK2
+static gint collection_scroll_event(GtkWidget *widget, GdkEventScroll *event)
+#else
+static gint collection_scroll_event(GtkWidget *widget, GdkEventButton *event)
+#endif
 {
 	Collection    	*collection;
-	int		diff;
+	int		diff = 0;
 
 	g_return_val_if_fail(widget != NULL, FALSE);
 	g_return_val_if_fail(IS_COLLECTION(widget), FALSE);
@@ -1122,24 +1213,33 @@ static gint collection_button_press(GtkWidget      *widget,
 
 	collection = COLLECTION(widget);
 
+#ifdef GTK2
+	if (event->direction == GDK_SCROLL_UP)
+		diff = -1;
+	else if (event->direction == GDK_SCROLL_DOWN)
+		diff = 1;
+	else
+		return FALSE;
+#else
 	if (event->button <= 3 || event->type != GDK_BUTTON_PRESS)
 		return FALSE;		/* Only deal with wheel events here */
 		
-	/* Wheel mouse scrolling */
-	diff = 32;
-
 	if (event->button == 4)
-		diff = -diff;
-	else if (event->button != 5)
+		diff = -1;
+	else if (event->button == 5)
+		diff = 1;
+	else
 		return FALSE;
+#endif
 
 	if (diff)
 	{
 		int	old_value = collection->vadj->value;
 		int	new_value = 0;
 		gboolean box = collection->lasso_box;
+		int	step = collection->vadj->page_increment / 2;
 
-		new_value = CLAMP(old_value + diff, 0.0, 
+		new_value = CLAMP(old_value + diff * step, 0.0, 
 				collection->vadj->upper
 				- collection->vadj->page_size);
 		diff = new_value - old_value;
@@ -1150,10 +1250,7 @@ static gint collection_button_press(GtkWidget      *widget,
 				remove_lasso_box(collection);
 				collection->drag_box_y[0] -= diff;
 			}
-			collection->vadj->value = new_value;
-			gtk_signal_emit_by_name(
-					GTK_OBJECT(collection->vadj),
-					"changed");
+			gtk_adjustment_set_value(collection->vadj, new_value);
 			if (box)
 				add_lasso_box(collection);
 		}
@@ -1193,7 +1290,7 @@ static void get_range(int from, int to, int step, short *pos, short *len)
  */
 static void find_lasso_area(Collection *collection, GdkRectangle *area)
 {
-	int	scroll;
+	int	scroll = 0;
 	int	cols = collection->columns;
 	int	dx = collection->drag_box_x[0] - collection->drag_box_x[1];
 	int	dy = collection->drag_box_y[0] - collection->drag_box_y[1];
@@ -1207,7 +1304,9 @@ static void find_lasso_area(Collection *collection, GdkRectangle *area)
 		return;
 	}
 
+#ifndef GTK2
 	scroll = collection->vadj->value;
+#endif
 
 	get_range(collection->drag_box_x[0],
 		  collection->drag_box_x[1],
@@ -1236,9 +1335,12 @@ static void collection_process_area(Collection	 *collection,
 	guint32		stacked_time;
 	int		item;
 	gboolean	changed = FALSE;
+	guint		old_selected;
 
 	g_return_if_fail(fn == GDK_SET || fn == GDK_INVERT);
 
+	old_selected = collection->number_selected;
+
 	stacked_time = current_event_time;
 	current_event_time = time;
 
@@ -1254,9 +1356,12 @@ static void collection_process_area(Collection	 *collection,
 				goto out;
 
 			if (fn == GDK_INVERT)
-				collection_toggle_item(collection, item);
+				collection_item_set_selected(collection, item,
+					!collection->items[item].selected,
+					FALSE);
 			else
-				collection_select_item(collection, item);
+				collection_item_set_selected(collection, item,
+								TRUE, FALSE);
 
 			changed = TRUE;
 			item++;
@@ -1264,6 +1369,15 @@ static void collection_process_area(Collection	 *collection,
 	}
 
 out:
+	if (collection->number_selected && !old_selected)
+		gtk_signal_emit(GTK_OBJECT(collection),
+				collection_signals[GAIN_SELECTION],
+				current_event_time);
+	else if (!collection->number_selected && old_selected)
+		gtk_signal_emit(GTK_OBJECT(collection),
+				collection_signals[LOSE_SELECTION],
+				current_event_time);
+	
 	collection_unblock_selection_changed(collection,
 					current_event_time, changed);
 	current_event_time = stacked_time;
@@ -1370,7 +1484,7 @@ static void scroll_to_show(Collection *collection, int item)
 
 		if (GTK_WIDGET_REALIZED(widget))
 		{
-			gdk_window_get_size(widget->window, NULL, &height);
+			height = collection->vadj->page_size;
 			gtk_adjustment_set_value(collection->vadj,
 				(row + 1) * collection->item_height - height);
 		}
@@ -1383,7 +1497,7 @@ static void scroll_to_show(Collection *collection, int item)
 static void get_visible_limits(Collection *collection, int *first, int *last)
 {
 	GtkWidget	*widget = (GtkWidget *) collection;
-	gint		scroll, height;
+	gint		scroll = 0, height;
 
 	g_return_if_fail(collection != NULL);
 	g_return_if_fail(IS_COLLECTION(collection));
@@ -1397,7 +1511,7 @@ static void get_visible_limits(Collection *collection, int *first, int *last)
 	else
 	{
 		scroll = collection->vadj->value;
-		gdk_window_get_size(widget->window, NULL, &height);
+		height = collection->vadj->page_size;
 
 		*first = MAX(scroll / collection->item_height, 0);
 		*last = (scroll + height - 1) /collection->item_height;
@@ -1507,7 +1621,12 @@ static gboolean as_timeout(Collection *collection)
 	int		diff = 0;
 
 	gdk_window_get_pointer(window, &x, &y, &mask);
-	gdk_window_get_size(window, &w, &h);
+	gdk_window_get_size(window, &w, NULL);
+
+	h = collection->vadj->page_size;
+#ifdef GTK2
+	y -= collection->vadj->value;
+#endif
 
 	if ((x < 0 || x > w || y < 0 || y > h) && !collection->lasso_box)
 	{
@@ -1526,10 +1645,15 @@ static gboolean as_timeout(Collection *collection)
 	return TRUE;
 }
 
-/* Change the selected state of an item */
+/* Change the selected state of an item.
+ * Send GAIN/LOSE signals if 'signal' is TRUE.
+ * Send SELECTION_CHANGED unless blocked.
+ * Updates number_selected and redraws the item.
+ */
 static void collection_item_set_selected(Collection *collection,
                                          gint item,
-                                         gboolean selected)
+                                         gboolean selected,
+					 gboolean signal)
 {
 	g_return_if_fail(collection != NULL);
 	g_return_if_fail(IS_COLLECTION(collection));
@@ -1544,7 +1668,7 @@ static void collection_item_set_selected(Collection *collection,
 	if (selected)
 	{
 		collection->number_selected++;
-		if (collection->number_selected == 1)
+		if (signal && collection->number_selected == 1)
 			gtk_signal_emit(GTK_OBJECT(collection),
 					collection_signals[GAIN_SELECTION],
 					current_event_time);
@@ -1552,7 +1676,7 @@ static void collection_item_set_selected(Collection *collection,
 	else
 	{
 		collection->number_selected--;
-		if (collection->number_selected == 0)
+		if (signal && collection->number_selected == 0)
 			gtk_signal_emit(GTK_OBJECT(collection),
 					collection_signals[LOSE_SELECTION],
 					current_event_time);
@@ -1566,41 +1690,13 @@ static void collection_item_set_selected(Collection *collection,
 /* Remove all objects from the collection */
 void collection_clear(Collection *collection)
 {
-	int	prev_selected;
-
-	g_return_if_fail(IS_COLLECTION(collection));
-
-	if (collection->number_of_items == 0)
-		return;
-
-	if (collection->wink_item != -1)
-	{
-		collection->wink_item = -1;
-		gtk_timeout_remove(collection->wink_timeout);
-	}
-
-	collection_set_cursor_item(collection,
-			collection->cursor_item == -1 ? -1: 0);
-	collection->cursor_item_old = -1;
-	prev_selected = collection->number_selected;
-	collection->number_of_items = collection->number_selected = 0;
-
-	resize_arrays(collection, MINIMUM_ITEMS);
-
-	collection->paint_level = PAINT_CLEAR;
-
-	gtk_widget_queue_clear(GTK_WIDGET(collection));
-
-	if (prev_selected && collection->number_selected == 0)
-		gtk_signal_emit(GTK_OBJECT(collection),
-				collection_signals[LOSE_SELECTION],
-				current_event_time);
+	collection_delete_if(collection, NULL, NULL);
 }
 
 /* Inserts a new item at the end. The new item is unselected, and its
  * number is returned.
  */
-gint collection_insert(Collection *collection, gpointer data)
+gint collection_insert(Collection *collection, gpointer data, gpointer view)
 {
 	int	item;
 	
@@ -1612,35 +1708,37 @@ gint collection_insert(Collection *collection, gpointer data)
 		resize_arrays(collection, item + (item >> 1));
 
 	collection->items[item].data = data;
+	collection->items[item].view_data = view;
 	collection->items[item].selected = FALSE;
 
 	collection->number_of_items++;
 
+#ifdef GTK2
+	gtk_widget_queue_resize(GTK_WIDGET(collection));
+#else
 	if (GTK_WIDGET_REALIZED(GTK_WIDGET(collection)))
-	{
 		set_vadjustment(collection);
-		collection_draw_item(collection,
-				collection->number_of_items - 1,
-				FALSE);
-	}
+#endif
+
+	collection_draw_item(collection, item, FALSE);
 
 	return item;
 }
 
 void collection_unselect_item(Collection *collection, gint item)
 {
-	collection_item_set_selected(collection, item, FALSE);
+	collection_item_set_selected(collection, item, FALSE, TRUE);
 }
 
 void collection_select_item(Collection *collection, gint item)
 {
-	collection_item_set_selected(collection, item, TRUE);
+	collection_item_set_selected(collection, item, TRUE, TRUE);
 }
 
 void collection_toggle_item(Collection *collection, gint item)
 {
 	collection_item_set_selected(collection, item,
-			!collection->items[item].selected);
+			!collection->items[item].selected, TRUE);
 }
 
 /* Select all items in the collection */
@@ -1675,6 +1773,41 @@ void collection_select_all(Collection *collection)
 	EMIT_SELECTION_CHANGED(collection, current_event_time);
 }
 
+/* Toggle all items in the collection */
+void collection_invert_selection(Collection *collection)
+{
+	int		item;
+	
+	g_return_if_fail(collection != NULL);
+	g_return_if_fail(IS_COLLECTION(collection));
+
+	if (collection->number_selected == 0)
+	{
+		collection_select_all(collection);
+		return;
+	}
+	else if (collection->number_of_items == collection->number_selected)
+	{
+		collection_clear_selection(collection);
+		return;
+	}
+
+	for (item = 0; item < collection->number_of_items; item++)
+		collection->items[item].selected =
+			!collection->items[item].selected;
+
+	collection->number_selected = collection->number_of_items -
+				      collection->number_selected;
+	
+	/* Have to redraw everything... */
+#ifndef GTK2
+	collection->paint_level = PAINT_CLEAR;
+#endif
+	gtk_widget_queue_clear(GTK_WIDGET(collection));
+	
+	EMIT_SELECTION_CHANGED(collection, current_event_time);
+}
+
 /* Unselect all items except number item, which is selected (-1 to unselect
  * everything).
  */
@@ -1732,10 +1865,12 @@ void collection_clear_selection(Collection *collection)
 /* Force a redraw of the specified item, if it is visible */
 void collection_draw_item(Collection *collection, gint item, gboolean blank)
 {
-	gint		height;
 	GdkRectangle	area;
 	GtkWidget	*widget;
 	int		row, col;
+#ifndef GTK2
+	gint		height;
+#endif
 
 	g_return_if_fail(collection != NULL);
 	g_return_if_fail(IS_COLLECTION(collection));
@@ -1751,6 +1886,9 @@ void collection_draw_item(Collection *collection, gint item, gboolean blank)
 
 	collection_get_item_area(collection, row, col, &area);
 
+#ifdef GTK2
+	gdk_window_invalidate_rect(widget->window, &area, FALSE);
+#else
 	if (area.y + area.height < 0)
 		return;
 
@@ -1770,6 +1908,7 @@ void collection_draw_item(Collection *collection, gint item, gboolean blank)
 		draw_lasso_box(collection);
 		gdk_gc_set_clip_rectangle(collection->xor_gc, NULL);
 	}
+#endif
 }
 
 void collection_set_item_size(Collection *collection, int width, int height)
@@ -1793,16 +1932,21 @@ void collection_set_item_size(Collection *collection, int width, int height)
 	{
 		gint		window_width;
 
-		collection->paint_level = PAINT_CLEAR;
 		gdk_window_get_size(widget->window, &window_width, NULL);
 		collection->columns = MAX(window_width / collection->item_width,
 					  1);
-
+#ifndef GTK2
+		collection->paint_level = PAINT_CLEAR;
 		set_vadjustment(collection);
+#endif
 		if (collection->cursor_item != -1)
 			scroll_to_show(collection, collection->cursor_item);
 		gtk_widget_queue_draw(widget);
 	}
+
+#ifdef GTK2
+	gtk_widget_queue_resize(GTK_WIDGET(collection));
+#endif
 }
 
 /* Cursor is positioned on item with the same data as before the sort.
@@ -1811,9 +1955,10 @@ void collection_set_item_size(Collection *collection, int width, int height)
 void collection_qsort(Collection *collection,
 			int (*compar)(const void *, const void *))
 {
-	int	cursor, wink, items;
+	int	cursor, wink, items, wink_on_map;
 	gpointer cursor_data = NULL;
 	gpointer wink_data = NULL;
+	gpointer wink_on_map_data = NULL;
 	
 	g_return_if_fail(collection != NULL);
 	g_return_if_fail(IS_COLLECTION(collection));
@@ -1821,6 +1966,15 @@ void collection_qsort(Collection *collection,
 
 	items = collection->number_of_items;
 
+	wink_on_map = collection->wink_on_map;
+	if (wink_on_map >= 0 && wink_on_map < items)
+	{
+		wink_on_map_data = collection->items[wink_on_map].data;
+		collection->wink_on_map = -1;
+	}
+	else
+		wink = -1;
+
 	wink = collection->wink_item;
 	if (wink >= 0 && wink < items)
 	{
@@ -1838,7 +1992,7 @@ void collection_qsort(Collection *collection,
 	
 	qsort(collection->items, items, sizeof(collection->items[0]), compar); 
 
-	if (cursor > -1 || wink > -1)
+	if (cursor > -1 || wink > -1 || wink_on_map > -1)
 	{
 		int	item;
 
@@ -1846,6 +2000,8 @@ void collection_qsort(Collection *collection,
 		{
 			if (collection->items[item].data == cursor_data)
 				collection_set_cursor_item(collection, item);
+			if (collection->items[item].data == wink_on_map_data)
+				collection->wink_on_map = item;
 			if (collection->items[item].data == wink_data)
 			{
 				collection->cursor_item_old = item;
@@ -1855,7 +2011,9 @@ void collection_qsort(Collection *collection,
 		}
 	}
 	
+#ifndef GTK2
 	collection->paint_level = PAINT_CLEAR;
+#endif
 
 	gtk_widget_queue_draw(GTK_WIDGET(collection));
 }
@@ -1885,14 +2043,16 @@ int collection_find_item(Collection *collection, gpointer data,
  */
 int collection_get_item(Collection *collection, int x, int y)
 {
-	int		scroll;
+	int		scroll = 0;
 	int		row, col;
 	int		width;
 	int		item;
 
 	g_return_val_if_fail(collection != NULL, -1);
 
+#ifndef GTK2
 	scroll = collection->vadj->value;
+#endif
 	col = x / collection->item_width;
 	row = (y + scroll) / collection->item_height;
 
@@ -1974,8 +2134,15 @@ void collection_wink_item(Collection *collection, gint item)
 	if (item == -1)
 		return;
 
+	if (!GTK_WIDGET_MAPPED(GTK_WIDGET(collection)))
+	{
+		collection->wink_on_map = item;
+		return;
+	}
+
 	collection->cursor_item_old = collection->wink_item = item;
 	collection->winks_left = 3;
+
 	collection->wink_timeout = gtk_timeout_add(70,
 					   (GtkFunction) wink_timeout,
 					   collection);
@@ -1989,6 +2156,8 @@ void collection_wink_item(Collection *collection, gint item)
  * Remove all items for which it returns TRUE. test() should
  * free the data before returning TRUE. The collection is in an
  * inconsistant state during this call (ie, when test() is called).
+ *
+ * If test is NULL, remove all items.
  */
 void collection_delete_if(Collection *collection,
 			  gboolean (*test)(gpointer item, gpointer data),
@@ -2000,14 +2169,14 @@ void collection_delete_if(Collection *collection,
 
 	g_return_if_fail(collection != NULL);
 	g_return_if_fail(IS_COLLECTION(collection));
-	g_return_if_fail(test != NULL);
 
 	cursor = collection->cursor_item;
 
 	for (in = 0; in < collection->number_of_items; in++)
 	{
-		if (!test(collection->items[in].data, data))
+		if (test && !test(collection->items[in].data, data))
 		{
+			/* Keep item */
 			if (collection->items[in].selected)
 			{
 				collection->items[out].selected = TRUE;
@@ -2016,11 +2185,22 @@ void collection_delete_if(Collection *collection,
 			else
 				collection->items[out].selected = FALSE;
 
-			collection->items[out++].data =
+			collection->items[out].data =
 				collection->items[in].data;
+			collection->items[out].view_data =
+				collection->items[in].view_data;
+			out++;
+		}
+		else 
+		{
+			/* Remove item */
+			if (collection->free_item)
+				collection->free_item(collection,
+							&collection->items[in]);
+			
+			if (cursor >= in)
+				cursor--;
 		}
-		else if (cursor >= in)
-			cursor--;
 	}
 
 	if (in != out)
@@ -2046,13 +2226,20 @@ void collection_delete_if(Collection *collection,
 		resize_arrays(collection,
 			MAX(collection->number_of_items, MINIMUM_ITEMS));
 
+#ifndef GTK2
 		collection->paint_level = PAINT_CLEAR;
+#endif
 
 		if (GTK_WIDGET_REALIZED(GTK_WIDGET(collection)))
 		{
+#ifndef GTK2
 			set_vadjustment(collection);
+#endif
 			gtk_widget_queue_draw(GTK_WIDGET(collection));
 		}
+#ifdef GTK2
+		gtk_widget_queue_resize(GTK_WIDGET(collection));
+#endif
 	}
 }
 
@@ -2175,7 +2362,6 @@ void collection_end_lasso(Collection *collection, GdkFunction fn)
 	}
 
 	abort_lasso(collection);
-
 }
 
 /* Unblock the selection_changed signal, emitting the signal if the
