@@ -1,5 +1,5 @@
 /*
- * $Id: dir.c,v 1.55 2001/09/02 14:12:36 tal197 Exp $
+ * $Id: dir.c,v 1.66 2001/12/22 19:32:12 tal197 Exp $
  *
  * Copyright (C) 2001, the ROX-Filer team.
  *
@@ -58,6 +58,7 @@
 #include "global.h"
 
 #include "dir.h"
+#include "diritem.h"
 #include "support.h"
 #include "gui_support.h"
 #include "dir.h"
@@ -77,10 +78,11 @@ static int getref(Directory *dir, gpointer data);
 static void update(Directory *dir, gchar *pathname, gpointer data);
 static void destroy(Directory *dir);
 static void set_idle_callback(Directory *dir);
-static void insert_item(Directory *dir, guchar *leafname);
+static DirItem *insert_item(Directory *dir, guchar *leafname);
 static void remove_missing(Directory *dir, GPtrArray *keep);
 static void dir_recheck(Directory *dir, guchar *path, guchar *leafname);
 static GPtrArray *hash_to_array(GHashTable *hash);
+static void dir_force_update_item(Directory *dir, gchar *leaf);
 
 /****************************************************************
  *			EXTERNAL INTERFACE			*
@@ -132,7 +134,8 @@ void dir_attach(Directory *dir, DirCallback callback, gpointer data)
 		callback(dir, DIR_END_SCAN, NULL, data);
 
 	if (dir->error)
-		delayed_error(PROJECT, dir->error);
+		delayed_error(_("Error scanning '%s':\n%s"),
+				dir->pathname, dir->error);
 }
 
 /* Undo the effect of dir_attach */
@@ -202,7 +205,7 @@ void dir_check_this(guchar *path)
 		*slash = '\0';
 
 		dir = g_fscache_lookup_full(dir_cache, real_path,
-						FSCACHE_LOOKUP_PEEK);
+						FSCACHE_LOOKUP_PEEK, NULL);
 		if (dir)
 			dir_recheck(dir, real_path, slash + 1);
 	}
@@ -210,13 +213,37 @@ void dir_check_this(guchar *path)
 	g_free(real_path);
 }
 
-void dir_rescan_with_thumbs(Directory *dir, gchar *pathname)
+/* Tell watchers that this item has changed, but don't rescan.
+ * (used when thumbnail has been created for an item)
+ */
+void dir_force_update_path(gchar *path)
 {
-	g_return_if_fail(dir != NULL);
-	g_return_if_fail(pathname != NULL);
+	gchar	*dir_path;
+	Directory *dir;
 
-	dir->do_thumbs = TRUE;
-	update(dir, pathname, NULL);
+	g_return_if_fail(path[0] == '/');
+
+	dir_path = g_dirname(path);
+
+	dir = g_fscache_lookup_full(dir_cache, dir_path, FSCACHE_LOOKUP_PEEK,
+			NULL);
+	if (dir)
+		dir_force_update_item(dir, g_basename(path));
+	
+	g_free(dir_path);
+}
+
+/* Ensure that 'leafname' is up-to-date. Returns the new/updated
+ * DirItem, or NULL if the file no longer exists.
+ */
+DirItem *dir_update_item(Directory *dir, gchar *leafname)
+{
+	DirItem *item;
+	
+	item = insert_item(dir, leafname);
+	dir_merge_new(dir);
+
+	return item;
 }
 
 static int sort_names(const void *a, const void *b)
@@ -274,12 +301,12 @@ static gboolean recheck_callback(gpointer data)
 	leaf = (guchar *) next->data;
 	g_list_free_1(next);
 
+	/* usleep(800); */
+
 	insert_item(dir, leaf);
 
 	g_free(leaf);
 
-	/* usleep(200); */
-
 	if (dir->recheck_list)
 		return TRUE;	/* Call again */
 
@@ -306,7 +333,6 @@ static gboolean recheck_callback(gpointer data)
  */
 void dir_rescan(Directory *dir, guchar *pathname)
 {
-	GList		*next;
 	GPtrArray	*names;
 	DIR		*d;
 	struct dirent	*ent;
@@ -352,20 +378,6 @@ void dir_rescan(Directory *dir, guchar *pathname)
 		g_ptr_array_add(names, g_strdup(ent->d_name));
 	}
 
-	/* Give everyone the list of names, so they can resize if needed.
-	 * Only do this if we haven't scanned before, as then the statted
-	 * items will be a better guess.
-	 */
-	if (!dir->have_scanned)
-	{
-		for (next = dir->users; next; next = next->next)
-		{
-			DirUser *user = (DirUser *) next->data;
-
-			user->callback(dir, DIR_NAMES, names, user->data);
-		}
-	}
-
 	/* Sort, so the names are scanned in a sensible order */
 	qsort(names->pdata, names->len, sizeof(guchar *), sort_names);
 
@@ -376,10 +388,23 @@ void dir_rescan(Directory *dir, guchar *pathname)
 
 	free_recheck_list(dir);
 
+	/* For each name found, add it to the recheck_list.
+	 * If the item is new, put a blank place-holder item in the directory.
+	 */
 	for (i = 0; i < names->len; i++)
-		dir->recheck_list = g_list_prepend(dir->recheck_list,
-						   names->pdata[i]);
+	{
+		guchar *name = names->pdata[i];
+		dir->recheck_list = g_list_prepend(dir->recheck_list, name);
+		if (!g_hash_table_lookup(dir->known_items, name))
+		{
+			DirItem *new;
 
+			new = diritem_new(name);
+			g_ptr_array_add(dir->new_items, new);
+		}
+	}
+	dir_merge_new(dir);
+	
 	dir->recheck_list = g_list_reverse(dir->recheck_list);
 
 	g_ptr_array_free(names, TRUE);
@@ -422,7 +447,6 @@ void dir_merge_new(Directory *dir)
 	g_ptr_array_set_size(up, 0);
 }
 
-
 /****************************************************************
  *			INTERNAL FUNCTIONS			*
  ****************************************************************/
@@ -435,8 +459,7 @@ static void free_items_array(GPtrArray *array)
 	{
 		DirItem	*item = (DirItem *) array->pdata[i];
 
-		diritem_clear(item);
-		g_free(item);
+		diritem_free(item);
 	}
 
 	g_ptr_array_free(array, TRUE);
@@ -531,7 +554,7 @@ static gint notify_timeout(gpointer data)
 	return FALSE;
 }
 
-/* Call merge_new() after a while. */
+/* Call dir_merge_new() after a while. */
 static void delayed_notify(Directory *dir)
 {
 	if (dir->notify_active)
@@ -554,85 +577,79 @@ static void remove_item(Directory *dir, DirItem *item)
 	free_items_array(gone);
 }
 
-/* Stat this item and add, update or remove it */
-static void insert_item(Directory *dir, guchar *leafname)
+/* Stat this item and add, update or remove it.
+ * Returns the new/updated item, if any.
+ * (leafname may be from the current DirItem item)
+ */
+static DirItem *insert_item(Directory *dir, guchar *leafname)
 {
 	static GString  *tmp = NULL;
 
 	DirItem		*item;
-	DirItem		new;
-	gboolean	is_new = FALSE;
-	gboolean	deleted;
+	DirItem		old;
+	gboolean	do_compare = FALSE;	/* (old is filled in) */
 
 	tmp = make_path(dir->pathname, leafname);
-	diritem_stat(tmp->str, &new, dir->do_thumbs);
-	deleted = new.base_type == TYPE_ERROR && new.lstat_errno == ENOENT;
-
 	item = g_hash_table_lookup(dir->known_items, leafname);
 
-	if (deleted)
+	if (item)
 	{
-		diritem_clear(&new);
-		if (item)
-			remove_item(dir, item);
-		return;
-	}
-
-	if (!item)
-	{
-		/* Item isn't already here... */
-		item = g_new(DirItem, 1);
-		item->leafname = g_strdup(leafname);
-		g_ptr_array_add(dir->new_items, item);
-		delayed_notify(dir);
-		is_new = TRUE;
-	}
-
-	item->may_delete = FALSE;
-
-	new.name_width = gdk_string_measure(item_font, item->leafname);
-	new.leafname = item->leafname;
-
-	if (!is_new)
-	{
-		if (item->lstat_errno == new.lstat_errno
-		 && item->base_type == new.base_type
-		 && item->flags == new.flags
-		 && item->size == new.size
-		 && item->mode == new.mode
-		 && item->atime == new.atime
-		 && item->ctime == new.ctime
-		 && item->mtime == new.mtime
-		 && item->uid == new.uid
-		 && item->gid == new.gid
-		 && item->image == new.image
-		 && item->mime_type == new.mime_type
-		 && item->name_width == new.name_width)
+		if (item->base_type != TYPE_UNKNOWN)
 		{
-			pixmap_unref(new.image);
-			return;
+			/* Preserve the old details so we can compare */
+			memcpy(&old, item, sizeof(DirItem));
+			pixmap_ref(old.image);
+			do_compare = TRUE;
 		}
+		diritem_restat(tmp->str, item);
 	}
-
-	item->image = new.image;
-	item->lstat_errno = new.lstat_errno;
-	item->base_type = new.base_type;
-	item->flags = new.flags;
-	item->size = new.size;
-	item->mode = new.mode;
-	item->uid = new.uid;
-	item->gid = new.gid;
-	item->atime = new.atime;
-	item->ctime = new.ctime;
-	item->mtime = new.mtime;
-	item->mime_type = new.mime_type;
-	item->name_width = new.name_width;
-
-	if (!is_new)
+	else
 	{
-		g_ptr_array_add(dir->up_items, item);
-		delayed_notify(dir);	/* TODO: Do we need this? */
+		/* Item isn't already here. This won't normally happen,
+		 * because blank items are added when scanning, before
+		 * we get here.
+		 */
+		item = diritem_new(leafname);
+		diritem_restat(tmp->str, item);
+		g_ptr_array_add(dir->new_items, item);
+		dir_merge_new(dir);
 	}
+
+	if (item->base_type == TYPE_ERROR && item->lstat_errno == ENOENT)
+	{
+		/* Item has been deleted */
+		remove_item(dir, item);
+		if (do_compare)
+			pixmap_unref(old.image);
+		return NULL;
+	}
+
+	if (do_compare)
+	{
+		if (item->lstat_errno == old.lstat_errno
+		 && item->base_type == old.base_type
+		 && item->flags == old.flags
+		 && item->size == old.size
+		 && item->mode == old.mode
+		 && item->atime == old.atime
+		 && item->ctime == old.ctime
+		 && item->mtime == old.mtime
+		 && item->uid == old.uid
+		 && item->gid == old.gid
+		 && item->image == old.image
+		 && item->mime_type == old.mime_type)
+		{
+			pixmap_unref(old.image);
+			return item;
+		}
+		pixmap_unref(old.image);
+
+	}
+
+	g_ptr_array_add(dir->up_items, item);
+	delayed_notify(dir);
+
+	return item;
 }
 
 static Directory *load(char *pathname, gpointer data)
@@ -652,7 +669,6 @@ static Directory *load(char *pathname, gpointer data)
 	dir->notify_active = FALSE;
 	dir->pathname = g_strdup(pathname);
 	dir->error = NULL;
-	dir->do_thumbs = FALSE;
 
 	dir->new_items = g_ptr_array_new();
 	dir->up_items = g_ptr_array_new();
@@ -711,7 +727,8 @@ static void update(Directory *dir, gchar *pathname, gpointer data)
 		dir_rescan(dir, pathname);
 
 	if (dir->error)
-		delayed_error(PROJECT, dir->error);
+		delayed_error(_("Error scanning '%s':\n%s"),
+				dir->pathname, dir->error);
 }
 
 /* If there is work to do, set the idle callback.
@@ -738,6 +755,34 @@ static void set_idle_callback(Directory *dir)
 	}
 }
 
+/* See dir_force_update_path() */
+static void dir_force_update_item(Directory *dir, gchar *leaf)
+{
+	GList *list = dir->users;
+	GPtrArray *items;
+	DirItem *item;
+
+	items = g_ptr_array_new();
+
+	item = g_hash_table_lookup(dir->known_items, leaf);
+	if (!item)
+		goto out;
+
+	g_ptr_array_add(items, item);
+
+	while (list)
+	{
+		DirUser *user = (DirUser *) list->data;
+
+		user->callback(dir, DIR_UPDATE, items, user->data);
+		
+		list = list->next;
+	}
+
+out:
+	g_ptr_array_free(items, TRUE);
+}
+
 static void dir_recheck(Directory *dir, guchar *path, guchar *leafname)
 {
 	guchar *old = dir->pathname;
