@@ -1,5 +1,5 @@
 /*
- * $Id: dir.h,v 1.22 2001/09/02 14:12:36 tal197 Exp $
+ * $Id: dir.h,v 1.26 2001/12/20 14:08:57 tal197 Exp $
  *
  * Thomas Leonard, <tal197@users.sourceforge.net>
  */
@@ -12,7 +12,6 @@
 #include <dirent.h>
 
 typedef enum {
-	DIR_NAMES,	/* Got a list of names (used for resizing) */
 	DIR_START_SCAN,	/* Set 'scanning' indicator */
 	DIR_END_SCAN,	/* Clear 'scanning' indicator */
 	DIR_ADD,	/* Add the listed items to the display */
@@ -26,9 +25,6 @@ typedef void (*DirCallback)(Directory *dir,
 			GPtrArray *items,
 			gpointer data);
 
-#include "fscache.h"
-#include "diritem.h"
-
 extern GFSCache *dir_cache;
 
 struct _DirUser
@@ -44,7 +40,6 @@ struct _Directory
 	GList	*users;		/* Functions to call on update */
 	char	*error;		/* NULL => no error */
 
-	gboolean	do_thumbs;	/* Create thumbs while scanning */
 	gboolean	notify_active;	/* Notify timeout is running */
 	gint		idle_callback;	/* Idle callback ID */
 
@@ -72,14 +67,12 @@ void dir_init(void);
 void dir_attach(Directory *dir, DirCallback callback, gpointer data);
 void dir_detach(Directory *dir, DirCallback callback, gpointer data);
 void dir_update(Directory *dir, gchar *pathname);
-void dir_rescan_with_thumbs(Directory *dir, gchar *pathname);
 int dir_item_cmp(const void *a, const void *b);
 void refresh_dirs(char *path);
-void dir_stat(guchar *path, DirItem *item, gboolean make_thumb);
-void dir_restat(guchar *path, DirItem *item, gboolean make_thumb);
-void dir_item_clear(DirItem *item);
 void dir_check_this(guchar *path);
+DirItem *dir_update_item(Directory *dir, gchar *leafname);
 void dir_rescan(Directory *dir, guchar *pathname);
 void dir_merge_new(Directory *dir);
+void dir_force_update_path(gchar *path);
 
 #endif /* _DIR_H */
