@@ -1,5 +1,5 @@
 /*
- * $Id: diritem.h,v 1.1 2001/07/25 10:26:33 tal197 Exp $
+ * $Id: diritem.h,v 1.7 2001/12/24 12:13:04 tal197 Exp $
  *
  * Thomas Leonard, <tal197@users.sourceforge.net>
  */
@@ -13,7 +13,7 @@
 typedef enum
 {
 	ITEM_FLAG_SYMLINK 	= 0x01,	/* Is a symlink */
-	ITEM_FLAG_APPDIR  	= 0x02,	/* Contains /AppInfo */
+	ITEM_FLAG_APPDIR  	= 0x02,	/* Contains an AppRun */
 	ITEM_FLAG_MOUNT_POINT  	= 0x04,	/* Is in mtab or fstab */
 	ITEM_FLAG_MOUNTED  	= 0x08,	/* Is in /etc/mtab */
 	ITEM_FLAG_EXEC_FILE  	= 0x20,	/* File, and has an X bit set */
@@ -29,17 +29,16 @@ struct _DirItem
 	mode_t		mode;
 	off_t		size;
 	time_t		atime, ctime, mtime;
-	MaskedPixmap	*image;
+	MaskedPixmap	*image;		/* NULL => leafname only so far */
 	MIME_type	*mime_type;
-	int		name_width;
 	uid_t		uid;
 	gid_t		gid;
 	int		lstat_errno;	/* 0 if details are valid */
 };
 
 void diritem_init(void);
-void diritem_stat(guchar *path, DirItem *item, gboolean make_thumb);
-void diritem_restat(guchar *path, DirItem *item, gboolean make_thumb);
-void diritem_clear(DirItem *item);
+DirItem *diritem_new(guchar *leafname);
+void diritem_restat(guchar *path, DirItem *item);
+void diritem_free(DirItem *item);
 
 #endif /* _DIRITEM_H */
