@@ -1,5 +1,5 @@
 /*
- * $Id: display.h,v 1.17 2001/10/17 10:44:21 tal197 Exp $
+ * $Id: display.h,v 1.25 2001/12/26 20:33:21 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * By Thomas Leonard, <tal197@users.sourceforge.net>.
@@ -11,25 +11,29 @@
 #define ROW_HEIGHT_LARGE 64
 
 #include <gtk/gtk.h>
-#include "collection.h"
 #include <sys/types.h>
 #include <dirent.h>
 
-typedef enum {		/* Values used in options, must start at 0 */
-	LARGE_ICONS	= 0,
-	SMALL_ICONS	= 1,
-	HUGE_ICONS	= 2,
-	UNKNOWN_STYLE
-} DisplayStyle;
+typedef struct _ViewData ViewData;
 
-typedef enum {		/* Values used in options, must start at 0 */
-	DETAILS_NONE		= 0,
-	DETAILS_SUMMARY 	= 1,
-	DETAILS_SIZE		= 2,
-	DETAILS_PERMISSIONS	= 3,
-	DETAILS_TYPE		= 4,
-	DETAILS_TIMES		= 5,
-} DetailsType;
+struct _ViewData
+{
+#ifdef GTK2
+	PangoLayout *layout;
+	PangoLayout *details;
+#endif
+	int	name_width;
+	int	name_height;
+	int	details_width;
+	int	details_height;
+#ifndef GTK2
+	int	split_pos;		/* 0 => No split */
+	int	split_width, split_height;
+	char	*details;
+#endif
+
+	MaskedPixmap *image;		/* Image; possibly thumbnail */
+};
 
 /* Prototypes */
 void display_init();
@@ -37,6 +41,7 @@ void display_set_layout(FilerWindow  *filer_window,
 			DisplayStyle style,
 			DetailsType  details);
 void display_set_hidden(FilerWindow *filer_window, gboolean hidden);
+void display_set_thumbs(FilerWindow *filer_window, gboolean thumbs);
 int sort_by_name(const void *item1, const void *item2);
 int sort_by_type(const void *item1, const void *item2);
 int sort_by_date(const void *item1, const void *item2);
@@ -45,27 +50,21 @@ void display_set_sort_fn(FilerWindow *filer_window,
 			int (*fn)(const void *a, const void *b));
 void display_set_autoselect(FilerWindow *filer_window, guchar *leaf);
 void shrink_grid(FilerWindow *filer_window);
-void calc_size(FilerWindow *filer_window, DirItem *item,
+void calc_size(FilerWindow *filer_window, CollectionItem *colitem,
 		int *width, int *height);
 
 void draw_large_icon(GtkWidget *widget,
 		     GdkRectangle *area,
 		     DirItem  *item,
+		     MaskedPixmap *image,
 		     gboolean selected);
-void draw_string(GtkWidget *widget,
-		GdkFont *font,
-		char	*string,
-		int	len,
-		int 	x,
-		int 	y,
-		int 	width,
-		int	area_width,
-		gboolean selected,
-		gboolean box);
 gboolean display_is_truncated(FilerWindow *filer_window, int i);
 void display_change_size(FilerWindow *filer_window, gboolean bigger);
-void display_guess_size(FilerWindow *filer_window,
-			GPtrArray *names,
-			int *ret_w, int *ret_h, int *ret_n);
+
+ViewData *display_create_viewdata(FilerWindow *filer_window, DirItem *item);
+void display_free_colitem(Collection *collection, CollectionItem *colitem);
+void display_update_view(FilerWindow *filer_window,
+			 DirItem *item,
+			 ViewData *view);
 
 #endif /* _DISPLAY_H */
