@@ -1,5 +1,5 @@
 /*
- * $Id: dnd.c,v 1.76 2001/10/30 19:20:43 tal197 Exp $
+ * $Id: dnd.c,v 1.85 2001/12/26 20:33:21 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2001, the ROX-Filer team.
@@ -34,10 +34,10 @@
 #include <X11/Xatom.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
-#include "collection.h"
 
 #include "global.h"
 
+#include "collection.h"
 #include "dnd.h"
 #include "type.h"
 #include "filer.h"
@@ -49,6 +49,7 @@
 #include "run.h"
 #include "pinboard.h"
 #include "dir.h"
+#include "diritem.h"
 #include "usericons.h"
 #include "menu.h"
 
@@ -175,7 +176,7 @@ void dnd_init()
 
 	item_factory = menu_create(menu_def,
 				sizeof(menu_def) / sizeof(*menu_def),
-				 "<dnd>");
+				 "<dnd>", NULL);
 	dnd_menu = gtk_item_factory_get_widget(item_factory, "<dnd>");
 }
 
@@ -243,7 +244,7 @@ GList *uri_list_to_glist(char *uri_list)
 
 		if (!linebreak || linebreak[1] != 10)
 		{
-			delayed_error("uri_list_to_glist",
+			delayed_error("uri_list_to_glist: %s",
 					_("Incorrect or missing line "
 					  "break in text/uri-list data"));
 			return list;
@@ -313,7 +314,8 @@ void drag_selection(GtkWidget *widget, GdkEventMotion *event, guchar *uri_list)
 void drag_one_item(GtkWidget		*widget,
 		   GdkEventMotion	*event,
 		   guchar		*full_path,
-		   DirItem		*item)
+		   DirItem		*item,
+		   MaskedPixmap		*image)
 {
 	guchar		*uri;
 	GdkDragContext 	*context;
@@ -328,6 +330,9 @@ void drag_one_item(GtkWidget		*widget,
 	g_return_if_fail(full_path != NULL);
 	g_return_if_fail(item != NULL);
 
+	if (!image)
+		image = item->image;
+
 	if (item->base_type == TYPE_FILE)
 	{
 		MIME_type *t = item->mime_type;
@@ -364,11 +369,11 @@ void drag_one_item(GtkWidget		*widget,
 			full_path, "\r\n", NULL);
 	g_dataset_set_data_full(context, "uri_list", uri, g_free);
 
+	g_return_if_fail(image != NULL);
+
 	gtk_drag_set_icon_pixmap(context,
 			gtk_widget_get_colormap(widget),
-			item->image->pixmap,
-			item->image->mask,
-			0, 0);
+			image->pixmap, image->mask, 0, 0);
 }
 
 static void drag_end(GtkWidget *widget,
@@ -421,7 +426,7 @@ void drag_data_get(GtkWidget          		*widget,
 			delete_once_sent = FALSE;
 			break;
 		default:
-			delayed_error("drag_data_get",
+			delayed_error("drag_data_get: %s",
 					_("Internal error - bad info type"));
 			break;
 	}
@@ -500,11 +505,13 @@ void drag_set_pinboard_dest(GtkWidget *widget)
 			    NULL);
 }
 
+#ifndef GTK2
 static void scrolled(GtkAdjustment *adj, Collection *collection)
 {
 	collection_set_cursor_item(collection, -1);
 	dnd_spring_abort();
 }
+#endif
 
 /* Called during the drag when the mouse is in a widget registered
  * as a drop target. Returns TRUE if we can accept the drop.
@@ -551,6 +558,8 @@ static gboolean drag_motion(GtkWidget		*widget,
 	if (item && type == drop_dest_dir &&
 			!(item->flags & ITEM_FLAG_APPDIR))
 	{
+#ifndef GTK2
+		/* XXX: Do we still need this under 2.0? */
 		GtkObject *vadj = GTK_OBJECT(filer_window->collection->vadj);
 
 		/* Subdir: prepare for spring-open */
@@ -566,6 +575,7 @@ static gboolean drag_motion(GtkWidget		*widget,
 						GTK_SIGNAL_FUNC(scrolled),
 						filer_window->collection);
 		}
+#endif
 		dnd_spring_load(context, filer_window);
 	}
 	else
@@ -624,7 +634,7 @@ guchar *dnd_motion_item(GdkDragContext *context, DirItem **item_p)
 		 * window background.
 		 */
 		if (item->base_type != TYPE_DIRECTORY
-				&& !(item->mime_type == &special_exec))
+				&& !(item->mime_type == special_exec))
 		{
 			item = NULL;
 			*item_p = NULL;
@@ -750,7 +760,7 @@ static gboolean drag_drop(GtkWidget 	  *widget,
 	{
 		gtk_drag_finish(context, FALSE, FALSE, time);	/* Failure */
 		
-		delayed_error(PROJECT, error);
+		delayed_error("%s", error);
 	}
 	else
 		gtk_drag_get_data(widget, context, target, time);
@@ -777,6 +787,12 @@ static void desktop_drag_data_received(GtkWidget      	*widget,
 		return;
 	}
 
+	if (pinboard_drag_in_progress)
+	{
+		pinboard_move_icons();
+		return;
+	}
+	
 	gdk_window_get_position(widget->window, &dx, &dy);
 	x += dx;
 	y += dy;
@@ -790,7 +806,7 @@ static void desktop_drag_data_received(GtkWidget      	*widget,
 		path = get_local_path((gchar *) next->data);
 		if (path)
 		{
-			pinboard_pin(path, NULL, x, y, TRUE);
+			pinboard_pin(path, NULL, x, y);
 			x += 64;
 		}
 
@@ -834,7 +850,7 @@ static void drag_data_received(GtkWidget      	*widget,
 			break;
 		default:
 			gtk_drag_finish(context, FALSE, FALSE, time);
-			delayed_error("drag_data_received",
+			delayed_error("drag_data_received: %s",
 					_("Unknown target"));
 			break;
 	}
@@ -894,7 +910,7 @@ static void got_data_xds_reply(GtkWidget 		*widget,
 	}
 
 	if (error)
-		delayed_error(PROJECT, error);
+		delayed_error("%s", error);
 }
 
 static void got_data_raw(GtkWidget 		*widget,
@@ -949,7 +965,7 @@ static void got_data_raw(GtkWidget 		*widget,
 		if (provides(context, XdndDirectSave0))
 			set_xds_prop(context, "");
 		gtk_drag_finish(context, FALSE, FALSE, time);	/* Failure */
-		delayed_error(_("Error saving file"), error);
+		delayed_error(_("Error saving file: %s"), error);
 	}
 	else
 		gtk_drag_finish(context, TRUE, FALSE, time);    /* Success! */
@@ -1041,11 +1057,11 @@ static void got_uri_list(GtkWidget 		*widget,
 		else if (context->action == GDK_ACTION_ASK)
 			prompt_action(local_paths, dest_path);
 		else if (context->action == GDK_ACTION_MOVE)
-			action_move(local_paths, dest_path, NULL);
+			action_move(local_paths, dest_path, NULL, -1);
 		else if (context->action == GDK_ACTION_COPY)
-			action_copy(local_paths, dest_path, NULL);
+			action_copy(local_paths, dest_path, NULL, -1);
 		else if (context->action == GDK_ACTION_LINK)
-			action_link(local_paths, dest_path);
+			action_link(local_paths, dest_path, NULL);
 		else
 			error = _("Unknown action requested");
 
@@ -1057,7 +1073,7 @@ static void got_uri_list(GtkWidget 		*widget,
 	if (error)
 	{
 		gtk_drag_finish(context, FALSE, FALSE, time);	/* Failure */
-		delayed_error(_("Error getting file list"), error);
+		delayed_error(_("Error getting file list: %s"), error);
 	}
 	else if (send_reply)
 		gtk_drag_finish(context, TRUE, FALSE, time);    /* Success! */
@@ -1075,18 +1091,18 @@ static void got_uri_list(GtkWidget 		*widget,
 static void menuitem_response(gpointer data, guint action, GtkWidget *widget)
 {
 	if (action == MENU_MOVE)
-		action_move(prompt_local_paths, prompt_dest_path, NULL);
+		action_move(prompt_local_paths, prompt_dest_path, NULL, -1);
 	else if (action == MENU_COPY)
-		action_copy(prompt_local_paths, prompt_dest_path, NULL);
+		action_copy(prompt_local_paths, prompt_dest_path, NULL, -1);
 	else if (action == MENU_LINK)
-		action_link(prompt_local_paths, prompt_dest_path);
+		action_link(prompt_local_paths, prompt_dest_path, NULL);
 	else if (action == MENU_SET_ICON)
 	{
 		if (g_list_length(prompt_local_paths) == 1)
 			set_icon_path(prompt_dest_path,
 				(char*) prompt_local_paths->data);
 		else
-			delayed_error(PROJECT,
+			delayed_error(
 			_("You can't use multiple files with Set Icon!"));
 	}
 } 
