@@ -1,5 +1,5 @@
 /*
- * $Id: filer.c,v 1.200 2001/10/17 10:44:21 tal197 Exp $
+ * $Id: filer.c,v 1.235 2001/12/28 14:47:36 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2001, the ROX-Filer team.
@@ -31,24 +31,28 @@
 #include <ctype.h>
 #include <math.h>
 #include <netdb.h>
+#include <sys/param.h>
 
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
-#include "collection.h"
 
 #include "global.h"
 
+#include "collection.h"
+#include "display.h"
 #include "main.h"
+#include "fscache.h"
 #include "support.h"
 #include "gui_support.h"
 #include "filer.h"
+#include "choices.h"
 #include "pixmaps.h"
 #include "menu.h"
 #include "dnd.h"
 #include "dir.h"
+#include "diritem.h"
 #include "run.h"
-#include "mount.h"
 #include "type.h"
 #include "options.h"
 #include "minibuffer.h"
@@ -56,13 +60,16 @@
 #include "toolbar.h"
 #include "bind.h"
 #include "appinfo.h"
+#include "mount.h"
 
 #define PANEL_BORDER 2
 
+static XMLwrapper *groups = NULL;
+
 FilerWindow 	*window_with_focus = NULL;
 GList		*all_filer_windows = NULL;
 
-static FilerWindow *window_with_selection = NULL;
+static FilerWindow *window_with_primary = NULL;
 
 /* Item we are about to display a tooltip for */
 static DirItem *tip_item = NULL;
@@ -75,9 +82,6 @@ static void attach(FilerWindow *filer_window);
 static void detach(FilerWindow *filer_window);
 static void filer_window_destroyed(GtkWidget    *widget,
 				   FilerWindow	*filer_window);
-static gint focus_in(GtkWidget *widget,
-			GdkEventFocus *event,
-			FilerWindow *filer_window);
 static void add_item(FilerWindow *filer_window, DirItem *item);
 static void update_display(Directory *dir,
 			DirAction	action,
@@ -87,7 +91,6 @@ static void set_scanning_display(FilerWindow *filer_window, gboolean scanning);
 static gboolean may_rescan(FilerWindow *filer_window, gboolean warning);
 static gboolean minibuffer_show_cb(FilerWindow *filer_window);
 static FilerWindow *find_filer_window(char *path, FilerWindow *diff);
-static void filer_set_title(FilerWindow *filer_window);
 static gint coll_button_release(GtkWidget *widget,
 			        GdkEventButton *event,
 			        FilerWindow *filer_window);
@@ -106,6 +109,10 @@ static void filer_size_for(FilerWindow *filer_window,
 			   int w, int h, int n, gboolean allow_shrink);
 
 static void set_unique(guchar *unique);
+static void set_selection_state(FilerWindow *collection, gboolean normal);
+static void filer_next_thumb(GtkObject *window, gchar *path);
+
+static void start_thumb_scanning(FilerWindow *filer_window);
 
 static GdkCursor *busy_cursor = NULL;
 static GdkCursor *crosshair = NULL;
@@ -183,22 +190,32 @@ static void update_item(FilerWindow *filer_window, DirItem *item)
 	int	old_w = filer_window->collection->item_width;
 	int	old_h = filer_window->collection->item_height;
 	int	w, h;
+	CollectionItem *colitem;
 	
 	if (leafname[0] == '.' && filer_window->show_hidden == FALSE)
 		return;
 
-	calc_size(filer_window, item, &w, &h); 
+	i = collection_find_item(filer_window->collection, item, dir_item_cmp);
+
+	if (i < 0)
+	{
+		g_warning("Failed to find '%s'\n", item->leafname);
+		return;
+	}
+	
+	colitem = &filer_window->collection->items[i];
+
+	display_update_view(filer_window,
+			(DirItem *) colitem->data,
+			(ViewData *) colitem->view_data);
+	
+	calc_size(filer_window, colitem, &w, &h); 
 	if (w > old_w || h > old_h)
 		collection_set_item_size(filer_window->collection,
 					 MAX(old_w, w),
 					 MAX(old_h, h));
 
-	i = collection_find_item(filer_window->collection, item, dir_item_cmp);
-
-	if (i >= 0)
-		collection_draw_item(filer_window->collection, i, TRUE);
-	else
-		g_warning("Failed to find '%s'\n", item->leafname);
+	collection_draw_item(filer_window->collection, i, TRUE);
 }
 
 /* Resize the filer window to w x h pixels, plus border (not clamped) */
@@ -255,36 +272,18 @@ static void filer_window_set_size(FilerWindow *filer_window,
 
 /* Resize the window to fit the items currently in the Directory.
  * This should be used once the Directory has been fully scanned, otherwise
- * the window will appear too small.
- * When opening a Directory that hasn't yet been scanned use the DIR_NAMES
- * instead for a quick estimate.
- *
- * This should all work unless you open a directory, get DIR_NAMES, and then
- * open it again in other window before the items get added. Oh well.
+ * the window will appear too small. When opening a directory for the first
+ * time, the names will be known but not the types and images. We can
+ * still make a good estimate of the size.
  */
 void filer_window_autosize(FilerWindow *filer_window, gboolean allow_shrink)
 {
 	Collection	*collection = filer_window->collection;
 	int 		n;
 
-	if (!filer_window->directory->have_scanned)
-		return;
-
-	/* If any items are queued to be added, add them now.
-	 * (this adds items that have been stat()d but not added to the
-	 * Collection, but not items only in recheck_list).
-	 */
-	dir_merge_new(filer_window->directory);
-
 	n = collection->number_of_items;
 	n = MAX(n, 2);
 
-#if 0
-	g_print("[ actual size  = %d x %d ]\n", 
-			collection->item_width,
-			collection->item_height);
-#endif
-	
 	filer_size_for(filer_window,
 			collection->item_width,
 			collection->item_height,
@@ -301,12 +300,23 @@ static void filer_size_for(FilerWindow *filer_window,
 	const float	r = 2.5;
 	int		t = 0;
 	int		size_limit;
-	int		space;
+	int		space = 0;
 
 	size_limit = option_get_int("filer_size_limit");
 
+	/* Get the extra height required for the toolbar and minibuffer,
+	 * if visible.
+	 */
 	if (o_toolbar != TOOLBAR_NONE)
 		t = filer_window->toolbar_frame->allocation.height;
+	if (GTK_WIDGET_VISIBLE(filer_window->minibuffer_area))
+	{
+		GtkRequisition req;
+
+		gtk_widget_size_request(filer_window->minibuffer_area, &req);
+		space = req.height + 2;
+		t += space;
+	}
 
 	max_x = (size_limit * screen_width) / 100;
 	max_rows = (size_limit * screen_height) / (h * 100);
@@ -315,7 +325,7 @@ static void filer_size_for(FilerWindow *filer_window,
 	 * 	   x = r(y + t + h),		(1)
 	 * unless that's too wide.
 	 *
-	 * t = toolbar height
+	 * t = toolbar (and minibuffer) height
 	 * r = desired (width / height) ratio
 	 *
 	 * Want to display all items:
@@ -353,8 +363,10 @@ static void filer_size_for(FilerWindow *filer_window,
 
 	/* Leave some room for extra icons, but only in Small Icons mode
 	 * otherwise it takes up too much space.
+	 * Also, don't add space if the minibuffer is open.
 	 */
-	space = filer_window->display_style == SMALL_ICONS ? h : 2;
+	if (space == 0)
+		space = filer_window->display_style == SMALL_ICONS ? h : 2;
 
 	filer_window_set_size(filer_window,
 			w * MAX(cols, 1),
@@ -367,6 +379,8 @@ static void filer_size_for(FilerWindow *filer_window,
  */
 static gint open_filer_window(FilerWindow *filer_window)
 {
+	shrink_grid(filer_window);
+
 	if (filer_window->open_timeout)
 	{
 		gtk_timeout_remove(filer_window->open_timeout);
@@ -382,22 +396,6 @@ static gint open_filer_window(FilerWindow *filer_window)
 	return FALSE;
 }
 
-/* The list of names in the directory is known, but the images for each
- * one are not. Choose a sensible size.
- */
-static void filer_auto_size_names(FilerWindow *filer_window, GPtrArray *names)
-{
-	int	w, h, n;
-
-	if (GTK_WIDGET_VISIBLE(filer_window->window))
-		if (option_get_int("filer_auto_resize") != RESIZE_ALWAYS)
-			return;
-			
-	display_guess_size(filer_window, names, &w, &h, &n);
-	
-	filer_size_for(filer_window, w, h, n, TRUE);
-}
-
 static void update_display(Directory *dir,
 			DirAction	action,
 			GPtrArray	*items,
@@ -438,11 +436,17 @@ static void update_display(Directory *dir,
 				else
 					collection_wink_item(collection,
 							cursor);
+
+				g_free(filer_window->auto_select);
+				filer_window->auto_select = NULL;
 			}
 
 			if (old_num != collection->number_of_items)
 				collection_qsort(filer_window->collection,
 						filer_window->sort_fn);
+
+			/* Open and resize if currently hidden */
+			open_filer_window(filer_window);
 			break;
 		case DIR_REMOVE:
 			collection_delete_if(filer_window->collection,
@@ -458,7 +462,6 @@ static void update_display(Directory *dir,
 				gdk_window_set_cursor(
 						filer_window->window->window,
 						NULL);
-			shrink_grid(filer_window);
 			if (filer_window->had_cursor &&
 					collection->cursor_item == -1)
 			{
@@ -468,6 +471,9 @@ static void update_display(Directory *dir,
 			set_scanning_display(filer_window, FALSE);
 			toolbar_update_info(filer_window);
 			open_filer_window(filer_window);
+
+			if (filer_window->thumb_queue)
+				start_thumb_scanning(filer_window);
 			break;
 		case DIR_UPDATE:
 			for (i = 0; i < items->len; i++)
@@ -479,9 +485,6 @@ static void update_display(Directory *dir,
 			collection_qsort(filer_window->collection,
 					filer_window->sort_fn);
 			break;
-		case DIR_NAMES:
-			filer_auto_size_names(filer_window, items);
-			break;
 	}
 }
 
@@ -510,8 +513,10 @@ static void filer_window_destroyed(GtkWidget 	*widget,
 {
 	all_filer_windows = g_list_remove(all_filer_windows, filer_window);
 
-	if (window_with_selection == filer_window)
-		window_with_selection = NULL;
+	gtk_object_set_data(GTK_OBJECT(widget), "filer_window", NULL);
+
+	if (window_with_primary == filer_window)
+		window_with_primary = NULL;
 	
 	if (window_with_focus == filer_window)
 	{
@@ -529,6 +534,12 @@ static void filer_window_destroyed(GtkWidget 	*widget,
 		filer_window->open_timeout = 0;
 	}
 
+	if (filer_window->thumb_queue)
+	{
+		g_list_foreach(filer_window->thumb_queue, (GFunc) g_free, NULL);
+		g_list_free(filer_window->thumb_queue);
+	}
+
 	g_free(filer_window->auto_select);
 	g_free(filer_window->path);
 	g_free(filer_window);
@@ -544,6 +555,7 @@ static void add_item(FilerWindow *filer_window, DirItem *item)
 	int		old_w = filer_window->collection->item_width;
 	int		old_h = filer_window->collection->item_height;
 	int		w, h;
+	int		i;
 
 	if (leafname[0] == '.')
 	{
@@ -552,13 +564,16 @@ static void add_item(FilerWindow *filer_window, DirItem *item)
 		return;
 	}
 
-	calc_size(filer_window, item, &w, &h); 
+	i = collection_insert(filer_window->collection,
+			item,
+			display_create_viewdata(filer_window, item));
+
+	calc_size(filer_window, &filer_window->collection->items[i], &w, &h); 
+
 	if (w > old_w || h > old_h)
 		collection_set_item_size(filer_window->collection,
 					 MAX(old_w, w),
 					 MAX(old_h, h));
-
-	collection_insert(filer_window->collection, item);
 }
 
 /* Returns TRUE iff the directory still exists. */
@@ -575,7 +590,7 @@ static gboolean may_rescan(FilerWindow *filer_window, gboolean warning)
 	if (!dir)
 	{
 		if (warning)
-			delayed_error(PROJECT, _("Directory missing/deleted"));
+			delayed_error(_("Directory missing/deleted"));
 		gtk_widget_destroy(filer_window->window);
 		return FALSE;
 	}
@@ -591,16 +606,16 @@ static gboolean may_rescan(FilerWindow *filer_window, gboolean warning)
 	return TRUE;
 }
 
-/* Another app has grabbed the selection */
+/* The collection widget has lost the primary selection */
 static gint collection_lose_selection(GtkWidget *widget,
 				      GdkEventSelection *event)
 {
-	if (window_with_selection &&
-			window_with_selection->collection == COLLECTION(widget))
+	if (window_with_primary &&
+			window_with_primary->collection == COLLECTION(widget))
 	{
-		FilerWindow *filer_window = window_with_selection;
-		window_with_selection = NULL;
-		collection_clear_selection(filer_window->collection);
+		FilerWindow *filer_window = window_with_primary;
+		window_with_primary = NULL;
+		set_selection_state(filer_window, FALSE);
 	}
 
 	return FALSE;
@@ -673,29 +688,40 @@ static void lose_selection(Collection 	*collection,
 {
 	FilerWindow *filer_window = (FilerWindow *) user_data;
 
-	if (window_with_selection == filer_window)
+	if (window_with_primary == filer_window)
 	{
-		window_with_selection = NULL;
+		window_with_primary = NULL;
 		gtk_selection_owner_set(NULL,
 				GDK_SELECTION_PRIMARY,
 				time);
 	}
 }
 
-static void gain_selection(Collection 	*collection,
-			   guint	time,
-			   gpointer 	user_data)
+static void selection_changed(Collection *collection,
+			      gint time,
+			      gpointer user_data)
 {
 	FilerWindow *filer_window = (FilerWindow *) user_data;
 
-	if (gtk_selection_owner_set(GTK_WIDGET(collection),
+	/* Selection has been changed -- try to grab the primary selection
+	 * if we don't have it.
+	 */
+	if (window_with_primary == filer_window)
+		return;		/* Already got it */
+
+	if (!collection->number_selected)
+		return;		/* Nothing selected */
+
+	if (filer_window->temp_item_selected == FALSE &&
+		gtk_selection_owner_set(GTK_WIDGET(collection),
 				GDK_SELECTION_PRIMARY,
 				time))
 	{
-		window_with_selection = filer_window;
+		window_with_primary = filer_window;
+		set_selection_state(filer_window, TRUE);
 	}
 	else
-		collection_clear_selection(filer_window->collection);
+		set_selection_state(filer_window, FALSE);
 }
 
 /* Open the item (or add it to the shell command minibuffer) */
@@ -769,15 +795,6 @@ static gint pointer_out(GtkWidget *widget,
 	return FALSE;
 }
 
-static gint focus_in(GtkWidget *widget,
-			GdkEventFocus *event,
-			FilerWindow *filer_window)
-{
-	window_with_focus = filer_window;
-
-	return FALSE;
-}
-
 /* Move the cursor to the next selected item in direction 'dir'
  * (+1 or -1).
  */
@@ -844,11 +861,199 @@ static void return_pressed(FilerWindow *filer_window, GdkEventKey *event)
 	filer_openitem(filer_window, item, flags);
 }
 
+/* Makes sure that 'groups' is up-to-date, reloading from file if it has
+ * changed. If no groups were loaded and there is no file then initialised
+ * groups to an empty document.
+ * Return the node for the 'name' group.
+ */
+static xmlNode *group_find(char *name)
+{
+	xmlNode *node;
+	gchar *path;
+
+	/* Update the groups, if possible */
+	path = choices_find_path_load("Groups.xml", PROJECT);
+	if (path)
+	{
+		XMLwrapper *wrapper;
+		wrapper = xml_cache_load(path);
+		if (wrapper)
+		{
+			if (groups)
+				xml_cache_unref(groups);
+			groups = wrapper;
+		}
+
+		g_free(path);
+	}
+
+	if (!groups)
+	{
+		groups = g_new(XMLwrapper, 1);
+		groups->doc = xmlNewDoc("1.0");
+		groups->ref = 1;
+		xmlDocSetRootElement(groups->doc,
+			xmlNewDocNode(groups->doc, NULL, "groups", NULL));
+		return NULL;
+	}
+
+	node = xmlDocGetRootElement(groups->doc);
+
+	for (node = node->xmlChildrenNode; node; node = node->next)
+	{
+		guchar	*gid;
+
+		gid = xmlGetProp(node, "name");
+
+		if (!gid)
+			continue;
+
+		if (strcmp(name, gid) != 0)
+			continue;
+
+		g_free(gid);
+
+		return node;
+	}
+
+	return NULL;
+}
+
+static void group_save(FilerWindow *filer_window, char *name)
+{
+	Collection *collection = filer_window->collection;
+	xmlNode	*group;
+	guchar	*save_path;
+	int	i;
+
+	group = group_find(name);
+	if (group)
+	{
+		xmlUnlinkNode(group);
+		xmlFreeNode(group);
+	}
+	group = xmlNewChild(xmlDocGetRootElement(groups->doc),
+			NULL, "group", NULL);
+	xmlSetProp(group, "name", name);
+
+	xmlNewChild(group, NULL, "directory", filer_window->path);
+
+	for (i = 0; i < collection->number_of_items; i++)
+	{
+		DirItem *item = (DirItem *) collection->items[i].data;
+		
+		if (!collection->items[i].selected)
+			continue;
+
+		xmlNewChild(group, NULL, "item", item->leafname);
+	}
+
+	save_path = choices_find_path_save("Groups.xml", PROJECT, TRUE);
+	if (save_path)
+	{
+		save_xml_file(groups->doc, save_path);
+		g_free(save_path);
+	}
+}
+
+static void group_restore(FilerWindow *filer_window, char *name)
+{
+	GHashTable *in_group;
+	Collection *collection = filer_window->collection;
+	int	j, n;
+	char	*path;
+	xmlNode	*group, *node;
+
+	group = group_find(name);
+
+	if (!group)
+	{
+		report_error(_("Group %s is not set. Select some files "
+				"and press Ctrl+%s to set the group. Press %s "
+				"on its own to reselect the files later."),
+				name, name, name);
+		return;
+	}
+
+	node = get_subnode(group, NULL, "directory");
+	g_return_if_fail(node != NULL);
+	path = xmlNodeListGetString(groups->doc, node->xmlChildrenNode, 1);
+	g_return_if_fail(path != NULL);
+
+	if (strcmp(path, filer_window->path) != 0)
+		filer_change_to(filer_window, path, NULL);
+	g_free(path);
+
+	/* If an item at the start is selected then we could lose the
+	 * primary selection after checking that item and then need to
+	 * gain it again at the end. Therefore, if anything is selected
+	 * then select the last item until the end of the search.
+	 */
+	n = collection->number_of_items;
+	if (collection->number_selected)
+		collection_select_item(collection, n - 1);
+
+	in_group = g_hash_table_new(g_str_hash, g_str_equal);
+	for (node = group->xmlChildrenNode; node; node = node->next)
+	{
+		gchar *leaf;
+		if (node->type != XML_ELEMENT_NODE)
+			continue;
+		if (strcmp(node->name, "item") != 0)
+			continue;
+
+		leaf = xmlNodeListGetString(groups->doc,
+						node->xmlChildrenNode, 1);
+		if (!leaf)
+			g_warning("Missing leafname!\n");
+		else
+			g_hash_table_insert(in_group, leaf, filer_window);
+	}
+	
+	for (j = 0; j < n; j++)
+	{
+		DirItem *item = (DirItem *) collection->items[j].data;
+
+		if (g_hash_table_lookup(in_group, item->leafname))
+			collection_select_item(collection, j);
+		else
+			collection_unselect_item(collection, j);
+	}
+
+	g_hash_table_foreach(in_group, (GHFunc) g_free, NULL);
+	g_hash_table_destroy(in_group);
+}
+
 /* Handle keys that can't be bound with the menu */
 static gint key_press_event(GtkWidget	*widget,
 			GdkEventKey	*event,
 			FilerWindow	*filer_window)
 {
+	gboolean handled;
+	char group[2] = "1";
+
+	window_with_focus = filer_window;
+
+	/* Note: Not convinced this is the way Gtk's key system is supposed
+	 * to be used...
+	 */
+	gtk_window_add_accel_group(GTK_WINDOW(filer_window->window),
+						filer_keys);
+#ifdef GTK2
+	handled = gtk_accel_groups_activate(G_OBJECT(filer_window->window),
+				event->keyval, event->state);
+#else
+	handled = gtk_accel_groups_activate(GTK_OBJECT(filer_window->window),
+				event->keyval, event->state);
+#endif
+	if (window_with_focus)
+		gtk_window_remove_accel_group(GTK_WINDOW(filer_window->window),
+						filer_keys);
+	else
+		return TRUE;	/* Window no longer exists */
+	if (handled)
+		return TRUE;
+
 	switch (event->keyval)
 	{
 		case GDK_Escape:
@@ -872,7 +1077,15 @@ static gint key_press_event(GtkWidget	*widget,
 					filer_window->collection->cursor_item);
 			break;
 		default:
-			return FALSE;
+			if (event->keyval < GDK_0 || event->keyval > GDK_9)
+				return FALSE;
+
+			group[0] = event->keyval - GDK_0 + '0';
+
+			if (event->state & GDK_CONTROL_MASK)
+				group_save(filer_window, group);
+			else
+				group_restore(filer_window, group);
 	}
 
 #ifndef GTK2
@@ -918,6 +1131,8 @@ void filer_change_to(FilerWindow *filer_window, char *path, char *from)
 
 	g_return_if_fail(filer_window != NULL);
 
+	filer_cancel_thumbnails(filer_window);
+
 	filer_tooltip_prime(NULL, NULL);
 
 	real_path = pathdup(path);
@@ -925,13 +1140,9 @@ void filer_change_to(FilerWindow *filer_window, char *path, char *from)
 
 	if (!new_dir)
 	{
-		char	*error;
-
-		error = g_strdup_printf(_("Directory '%s' is not accessible"),
+		delayed_error(_("Directory '%s' is not accessible"),
 				real_path);
 		g_free(real_path);
-		delayed_error(PROJECT, error);
-		g_free(error);
 		return;
 	}
 	
@@ -1080,6 +1291,7 @@ static void create_uri_list(FilerWindow *filer_window, GString *string)
 /* Creates and shows a new filer window.
  * If src_win != NULL then display options can be taken from that source window.
  * Returns the new filer window, or NULL on error.
+ * Note: if unique windows is in use, may return an existing window.
  */
 FilerWindow *filer_opendir(char *path, FilerWindow *src_win)
 {
@@ -1087,10 +1299,22 @@ FilerWindow *filer_opendir(char *path, FilerWindow *src_win)
 	char		*real_path;
 	DisplayStyle    dstyle;
 	DetailsType     dtype;
+	FilerWindow	*same_dir_window = NULL;
 	
 	/* Get the real pathname of the directory and copy it */
 	real_path = pathdup(path);
 
+	if (o_unique_filer_windows)
+		same_dir_window = find_filer_window(real_path, NULL);
+
+#ifdef GTK2
+	if (same_dir_window)
+	{
+		gtk_window_present(GTK_WINDOW(same_dir_window->window));
+		return same_dir_window;
+	}
+#endif
+
 	filer_window = g_new(FilerWindow, 1);
 	filer_window->minibuffer = NULL;
 	filer_window->minibuffer_label = NULL;
@@ -1103,6 +1327,7 @@ FilerWindow *filer_opendir(char *path, FilerWindow *src_win)
 	filer_window->toolbar_text = NULL;
 	filer_window->target_cb = NULL;
 	filer_window->mini_type = MINI_NONE;
+	filer_window->selection_state = GTK_STATE_INSENSITIVE;
 
 	/* Finds the entry for this directory in the dir cache, creating
 	 * a new one if needed. This does not cause a scan to start,
@@ -1112,11 +1337,7 @@ FilerWindow *filer_opendir(char *path, FilerWindow *src_win)
 						   filer_window->path);
 	if (!filer_window->directory)
 	{
-		char	*error;
-
-		error = g_strdup_printf(_("Directory '%s' not found."), path);
-		delayed_error(PROJECT, error);
-		g_free(error);
+		delayed_error(_("Directory '%s' not found."), path);
 		g_free(filer_window->path);
 		g_free(filer_window);
 		return NULL;
@@ -1126,6 +1347,8 @@ FilerWindow *filer_opendir(char *path, FilerWindow *src_win)
 	filer_window->flags = (FilerFlags) 0;
 	filer_window->details_type = DETAILS_SUMMARY;
 	filer_window->display_style = UNKNOWN_STYLE;
+	filer_window->thumb_queue = NULL;
+	filer_window->max_thumbs = 0;
 
 	if (src_win && option_get_int("display_inherit_options"))
 	{
@@ -1133,6 +1356,7 @@ FilerWindow *filer_opendir(char *path, FilerWindow *src_win)
 		dstyle = src_win->display_style;
 		dtype = src_win->details_type;
 		filer_window->show_hidden = src_win->show_hidden;
+		filer_window->show_thumbs = src_win->show_thumbs;
 	}
 	else
 	{
@@ -1146,10 +1370,15 @@ FilerWindow *filer_opendir(char *path, FilerWindow *src_win)
 		dtype = option_get_int("display_details");
 		filer_window->show_hidden =
 			option_get_int("display_show_hidden");
+		filer_window->show_thumbs =
+			option_get_int("display_show_thumbs");
 	}
 
 	/* Add all the user-interface elements & realise */
 	filer_add_widgets(filer_window);
+	if (src_win)
+		gtk_window_set_position(GTK_WINDOW(filer_window->window),
+					GTK_WIN_POS_MOUSE);
 
 	/* Connect to all the signal handlers */
 	filer_add_signals(filer_window);
@@ -1187,15 +1416,10 @@ FilerWindow *filer_opendir(char *path, FilerWindow *src_win)
 	 * for an existing one and close it if found.
 	 */
 
-	if (o_unique_filer_windows)
-	{
-		FilerWindow *fw;
-		
-		fw = find_filer_window(filer_window->path, NULL);
-
-		if (fw)
-			gtk_widget_destroy(fw->window);
-	}
+#ifndef GTK2
+	if (same_dir_window)
+		gtk_widget_destroy(same_dir_window->window);
+#endif
 
 	all_filer_windows = g_list_prepend(all_filer_windows, filer_window);
 	
@@ -1211,17 +1435,24 @@ static void filer_add_widgets(FilerWindow *filer_window)
 
 	/* Create the top-level window widget */
 	filer_window->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_position(GTK_WINDOW(filer_window->window),
-				GTK_WIN_POS_MOUSE);
 	filer_set_title(filer_window);
 
 	/* The collection is the area that actually displays the files */
-	collection = collection_new(NULL);
+	collection = collection_new();
 
+	/* This property is cleared when the window is destroyed.
+	 * You can thus ref filer_window->window and use this to see
+	 * if the window no longer exists.
+	 */
+	gtk_object_set_data(GTK_OBJECT(filer_window->window),
+			"filer_window", filer_window);
+	
 	gtk_object_set_data(GTK_OBJECT(collection),
 			"filer_window", filer_window);
 	filer_window->collection = COLLECTION(collection);
 
+	filer_window->collection->free_item = display_free_colitem;
+
 	/* Scrollbar on the right, everything else on the left */
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(filer_window->window), hbox);
@@ -1262,29 +1493,62 @@ static void filer_add_widgets(FilerWindow *filer_window)
 		gtk_widget_show_all(filer_window->toolbar_frame);
 	}
 
-	/* Now add the area for displaying the files... */
-	gtk_box_pack_start(GTK_BOX(vbox), collection, TRUE, TRUE, 0);
+	/* Now add the area for displaying the files.
+	 * If we've got Gtk+-2.0 then the collection is one huge window
+	 * that goes in a Viewport. Otherwise, the collection handles
+	 * scrolling itself.
+	 */
+#ifdef GTK2
+	{
+		GtkWidget *viewport;
+		GtkAdjustment *adj;
 
-	/* And the minibuffer (hidden by default)... */
+		adj = filer_window->collection->vadj;
+		viewport = gtk_viewport_new(NULL, adj);
+		gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewport),
+				GTK_SHADOW_NONE);
+		gtk_container_add(GTK_CONTAINER(viewport), collection);
+		gtk_widget_show_all(viewport);
+		gtk_box_pack_start(GTK_BOX(vbox), viewport, TRUE, TRUE, 0);
+		filer_window->scrollbar = gtk_vscrollbar_new(adj);
+		gtk_widget_set_usize(viewport, 4, 4);
+	}
+#else
+	gtk_box_pack_start(GTK_BOX(vbox), collection, TRUE, TRUE, 0);
+	filer_window->scrollbar =
+			gtk_vscrollbar_new(COLLECTION(collection)->vadj);
+#endif
+
+	/* And the minibuffer (hidden to start with) */
 	create_minibuffer(filer_window);
 	gtk_box_pack_start(GTK_BOX(vbox), filer_window->minibuffer_area,
 				FALSE, TRUE, 0);
 
+	/* And the thumbnail progress bar (also hidden) */
+	{
+		GtkWidget *cancel;
+
+		filer_window->thumb_bar = gtk_hbox_new(FALSE, 2);
+		gtk_box_pack_start(GTK_BOX(vbox), filer_window->thumb_bar,
+				FALSE, TRUE, 0);
+
+		filer_window->thumb_progress = gtk_progress_bar_new();
+		gtk_box_pack_start(GTK_BOX(filer_window->thumb_bar),
+				filer_window->thumb_progress, TRUE, TRUE, 0);
+
+		cancel = gtk_button_new_with_label(_("Cancel"));
+		GTK_WIDGET_UNSET_FLAGS(cancel, GTK_CAN_FOCUS);
+		gtk_box_pack_start(GTK_BOX(filer_window->thumb_bar),
+				cancel, FALSE, TRUE, 0);
+		gtk_signal_connect_object(GTK_OBJECT(cancel), "clicked",
+				GTK_SIGNAL_FUNC(filer_cancel_thumbnails),
+				(GtkObject *) filer_window);
+	}
+
 	/* Put the scrollbar on the left of everything else... */
-	filer_window->scrollbar =
-			gtk_vscrollbar_new(COLLECTION(collection)->vadj);
 	gtk_box_pack_start(GTK_BOX(hbox),
 			filer_window->scrollbar, FALSE, TRUE, 0);
 
-	/* Connect the menu's accelerator group to the window */
-	gtk_accel_group_attach(filer_keys,
-#ifdef GTK2
-			G_OBJECT(filer_window->window)
-#else
-			GTK_OBJECT(filer_window->window)
-#endif
-			);
-
 	gtk_window_set_focus(GTK_WINDOW(filer_window->window), collection);
 
 	gtk_widget_show(hbox);
@@ -1316,8 +1580,6 @@ static void filer_add_signals(FilerWindow *filer_window)
 	gtk_signal_connect(GTK_OBJECT(filer_window->window),
 			"leave-notify-event",
 			GTK_SIGNAL_FUNC(pointer_out), filer_window);
-	gtk_signal_connect(GTK_OBJECT(filer_window->window), "focus_in_event",
-			GTK_SIGNAL_FUNC(focus_in), filer_window);
 	gtk_signal_connect(GTK_OBJECT(filer_window->window), "destroy",
 			GTK_SIGNAL_FUNC(filer_window_destroyed), filer_window);
 
@@ -1327,10 +1589,10 @@ static void filer_add_signals(FilerWindow *filer_window)
 			GDK_BUTTON3_MOTION_MASK | GDK_POINTER_MOTION_MASK |
 			GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
 
-	gtk_signal_connect(collection, "gain_selection",
-			GTK_SIGNAL_FUNC(gain_selection), filer_window);
 	gtk_signal_connect(collection, "lose_selection",
 			GTK_SIGNAL_FUNC(lose_selection), filer_window);
+	gtk_signal_connect(collection, "selection_changed",
+			GTK_SIGNAL_FUNC(selection_changed), filer_window);
 	gtk_signal_connect(collection, "selection_clear_event",
 			GTK_SIGNAL_FUNC(collection_lose_selection), NULL);
 	gtk_signal_connect(collection, "selection_get",
@@ -1381,6 +1643,20 @@ void filer_update_dir(FilerWindow *filer_window, gboolean warning)
 		dir_update(filer_window->directory, filer_window->path);
 }
 
+void filer_update_all(void)
+{
+	GList	*next = all_filer_windows;
+
+	while (next)
+	{
+		FilerWindow *filer_window = (FilerWindow *) next->data;
+
+		next = next->next;
+
+		filer_update_dir(filer_window, TRUE);
+	}
+}
+
 /* Refresh the various caches even if we don't think we need to */
 void full_refresh(void)
 {
@@ -1504,15 +1780,27 @@ gboolean filer_exists(FilerWindow *filer_window)
 	return FALSE;
 }
 
-static void filer_set_title(FilerWindow *filer_window)
+/* Make sure the window title is up-to-date */
+void filer_set_title(FilerWindow *filer_window)
 {
 	guchar	*title = NULL;
-	guchar	*scanning = filer_window->scanning ? _(" (Scanning)") : "";
+	guchar	*flags = "";
+
+	if (filer_window->scanning || filer_window->show_hidden ||
+				filer_window->show_thumbs)
+	{
+		flags = g_strconcat(" (",
+				filer_window->scanning ? _("Scanning, ") : "",
+				filer_window->show_hidden ? _("All, ") : "",
+				filer_window->show_thumbs ? _("Thumbs, ") : "",
+				NULL);
+		flags[strlen(flags) - 2] = ')';
+	}
 
 	if (not_local)
 	{
 	        title = g_strconcat("//", our_host_name(),
-			    filer_window->path, scanning, NULL);
+			    filer_window->path, flags, NULL);
 	}
 	
 	if (!title && home_dir_len > 1 &&
@@ -1523,15 +1811,18 @@ static void filer_set_title(FilerWindow *filer_window)
 		if (sep == '\0' || sep == '/')
 			title = g_strconcat("~",
 					filer_window->path + home_dir_len,
-					scanning,
+					flags,
 					NULL);
 	}
 	
 	if (!title)
-		title = g_strconcat(filer_window->path, scanning, NULL);
+		title = g_strconcat(filer_window->path, flags, NULL);
 
 	gtk_window_set_title(GTK_WINDOW(filer_window->window), title);
 	g_free(title);
+
+	if (flags[0] != '\0')
+		g_free(flags);
 }
 
 /* Reconnect to the same directory (used when the Show Hidden option is
@@ -1582,6 +1873,15 @@ static void perform_action(FilerWindow *filer_window, GdkEventButton *event)
 
 	item = collection_get_item(collection, event->x, event->y);
 
+	if (item != -1 && event->button == 1 &&
+		collection->items[item].selected &&
+		filer_window->selection_state == GTK_STATE_INSENSITIVE)
+	{
+		selection_changed(filer_window->collection,
+				event->time, filer_window);
+		return;
+	}
+
 	if (filer_window->target_cb)
 	{
 		dnd_motion_ungrab();
@@ -1717,8 +2017,8 @@ static gint coll_motion_notify(GtkWidget *widget,
 		if (event->state & GDK_BUTTON1_MASK)
 		{
 			/* Select just this one */
-			collection_clear_except(collection, i);
 			filer_window->temp_item_selected = TRUE;
+			collection_clear_except(collection, i);
 		}
 		else
 		{
@@ -1732,11 +2032,22 @@ static gint coll_motion_notify(GtkWidget *widget,
 
 	if (collection->number_selected == 1)
 	{
-		DirItem	*item = (DirItem *) collection->items[i].data;
+		DirItem	 *item = (DirItem *) collection->items[i].data;
+		ViewData *view = (ViewData *) collection->items[i].view_data;
+
+		if (!item->image)
+			item = dir_update_item(filer_window->directory,
+						item->leafname);
+
+		if (!item)
+		{
+			report_error(_("Item no longer exists!"));
+			return FALSE;
+		}
 
 		drag_one_item(widget, event,
 			make_path(filer_window->path, item->leafname)->str,
-			item);
+			item, view ? view->image : NULL);
 	}
 	else
 	{
@@ -1814,6 +2125,7 @@ static gboolean filer_tooltip_activate(FilerWindow *filer_window)
 	gint 	x, y;
 	int	i;
 	GString	*tip = NULL;
+	guchar	*fullpath = NULL;
 
 	g_return_val_if_fail(tip_item != NULL, 0);
 
@@ -1842,21 +2154,39 @@ static gboolean filer_tooltip_activate(FilerWindow *filer_window)
 		g_string_append_c(tip, '\n');
 	}
 	
+	fullpath = make_path(filer_window->path, tip_item->leafname)->str;
+
+	if (tip_item->flags & ITEM_FLAG_SYMLINK)
+	{
+		char *target;
+
+		target = readlink_dup(fullpath);
+		if (target)
+		{
+			g_string_append(tip, _("Symbolic link to "));
+			g_string_append(tip, target);
+			g_string_append_c(tip, '\n');
+			g_free(target);
+		}
+	}
+	
 	if (tip_item->flags & ITEM_FLAG_APPDIR)
 	{
-		AppInfo *info;
+		XMLwrapper *info;
 		xmlNode *node;
 
-		info = appinfo_get(make_path(filer_window->path,
-					tip_item->leafname)->str, tip_item);
+		info = appinfo_get(fullpath, tip_item);
 		if (info && ((node = appinfo_get_section(info, "Summary"))))
 		{
 			guchar *str;
 			str = xmlNodeListGetString(node->doc,
 					node->xmlChildrenNode, 1);
-			g_string_append(tip, str);
-			g_string_append_c(tip, '\n');
-			g_free(str);
+			if (str)
+			{
+				g_string_append(tip, str);
+				g_string_append_c(tip, '\n');
+				g_free(str);
+			}
 		}
 	}
 
@@ -1965,3 +2295,104 @@ static void filer_tooltip_prime(FilerWindow *filer_window, DirItem *item)
 					filer_window);
 	}
 }
+
+static void set_selection_state(FilerWindow *filer_window, gboolean normal)
+{
+	GtkStateType old_state;
+
+	old_state = filer_window->selection_state;
+	filer_window->selection_state = normal
+			? GTK_STATE_SELECTED : GTK_STATE_INSENSITIVE;
+
+	if (old_state != filer_window->selection_state
+	    && filer_window->collection->number_selected)
+		gtk_widget_queue_draw(GTK_WIDGET(filer_window->collection));
+}
+
+void filer_cancel_thumbnails(FilerWindow *filer_window)
+{
+	gtk_widget_hide(filer_window->thumb_bar);
+
+	g_list_foreach(filer_window->thumb_queue, (GFunc) g_free, NULL);
+	g_list_free(filer_window->thumb_queue);
+	filer_window->thumb_queue = NULL;
+	filer_window->max_thumbs = 0;
+}
+
+/* Generate the next thumb for this window. The collection object is
+ * unref'd when there is nothing more to do.
+ * If the collection no longer has a filer window, nothing is done.
+ */
+static gboolean filer_next_thumb_real(GtkObject *window)
+{
+	FilerWindow *filer_window;
+	gchar	*path;
+	int	done, total;
+
+	filer_window = gtk_object_get_data(window, "filer_window");
+
+	if (!filer_window)
+	{
+		gtk_object_unref(window);
+		return FALSE;
+	}
+		
+	if (!filer_window->thumb_queue)
+	{
+		filer_cancel_thumbnails(filer_window);
+		gtk_object_unref(window);
+		return FALSE;
+	}
+
+	total = filer_window->max_thumbs;
+	done = total - g_list_length(filer_window->thumb_queue);
+
+	path = (gchar *) filer_window->thumb_queue->data;
+
+	pixmap_background_thumb(path, (GFunc) filer_next_thumb, window);
+
+	filer_window->thumb_queue = g_list_remove(filer_window->thumb_queue,
+						  path);
+	g_free(path);
+
+	gtk_progress_set_percentage(GTK_PROGRESS(filer_window->thumb_progress),
+					done / (float) total);
+
+	return FALSE;
+}
+
+/* path is the thumb just loaded, if any.
+ * collection is unref'd (eventually).
+ */
+static void filer_next_thumb(GtkObject *window, gchar *path)
+{
+	if (path)
+		dir_force_update_path(path);
+
+	gtk_idle_add((GtkFunction) filer_next_thumb_real, window);
+}
+
+static void start_thumb_scanning(FilerWindow *filer_window)
+{
+	if (GTK_WIDGET_VISIBLE(filer_window->thumb_bar))
+		return;		/* Already scanning */
+
+	gtk_widget_show_all(filer_window->thumb_bar);
+
+	gtk_object_ref(GTK_OBJECT(filer_window->window));
+	filer_next_thumb(GTK_OBJECT(filer_window->window), NULL);
+}
+
+/* Set this image to be loaded some time in the future */
+void filer_create_thumb(FilerWindow *filer_window, gchar *path)
+{
+	filer_window->max_thumbs++;
+
+	filer_window->thumb_queue = g_list_append(filer_window->thumb_queue,
+						  g_strdup(path));
+
+	if (filer_window->scanning)
+		return;			/* Will start when scan ends */
+
+	start_thumb_scanning(filer_window);
+}
