@@ -1,5 +1,5 @@
 /*
- * $Id: filer.h,v 1.67 2001/09/25 14:30:39 tal197 Exp $
+ * $Id: filer.h,v 1.71 2001/12/22 19:32:12 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * By Thomas Leonard, <tal197@users.sourceforge.net>.
@@ -9,7 +9,6 @@
 #define _FILER_H
 
 #include <gtk/gtk.h>
-#include "collection.h"
 
 enum {
 	RESIZE_STYLE = 0,
@@ -29,12 +28,9 @@ typedef enum
 {
 	FILER_NEEDS_RESCAN	= 0x01, /* Call may_rescan after scanning */
 	FILER_UPDATING		= 0x02, /* (scanning) items may already exist */
+	FILER_CREATE_THUMBS	= 0x04, /* Create thumbs when scan ends */
 } FilerFlags;
 
-#include "mount.h"
-#include "minibuffer.h"
-#include "display.h"
-
 typedef void (*TargetFunc)(FilerWindow *filer_window, int item, gpointer data);
 
 struct _FilerWindow
@@ -75,6 +71,13 @@ struct _FilerWindow
 	GtkWidget	*scrollbar;
 
 	gint		open_timeout;	/* Will resize and show window... */
+
+	GtkStateType	selection_state;	/* for drawing selection */
+	
+	gboolean	show_thumbs;
+	GList		*thumb_queue;		/* paths to thumbnail */
+	GtkWidget	*thumb_bar, *thumb_progress;
+	int		max_thumbs;		/* total for this batch */
 };
 
 extern FilerWindow 	*window_with_focus;
@@ -105,5 +108,8 @@ void filer_target_mode(FilerWindow	*filer_window,
 			char		*reason);
 void filer_window_autosize(FilerWindow *filer_window, gboolean allow_shrink);
 GList *filer_selected_items(FilerWindow *filer_window);
+void filer_create_thumb(FilerWindow *filer_window, gchar *pathname);
+void filer_cancel_thumbnails(FilerWindow *filer_window);
+void filer_set_title(FilerWindow *filer_window);
 
 #endif /* _FILER_H */
