@@ -1,5 +1,5 @@
 /*
- * $Id: find.c,v 1.12 2001/02/07 17:58:40 tal197 Exp $
+ * $Id: find.c,v 1.14 2001/12/12 14:56:38 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2001, the ROX-Filer team.
@@ -49,6 +49,7 @@ static FindCondition *parse_system(guchar **expression);
 static FindCondition *parse_condition(guchar **expression);
 static FindCondition *parse_match(guchar **expression);
 static FindCondition *parse_comparison(guchar **expression);
+static FindCondition *parse_dash(guchar **expression);
 static FindCondition *parse_is(guchar **expression);
 static Eval *parse_eval(guchar **expression);
 static Eval *parse_variable(guchar **expression);
@@ -567,6 +568,10 @@ static FindCondition *parse_condition(guchar **expression)
 		return cond;
 	}
 
+	cond = parse_dash(expression);
+	if (cond)
+		return cond;
+
 	cond = parse_is(expression);
 	if (cond)
 		return cond;
@@ -665,6 +670,71 @@ static FindCondition *parse_comparison(guchar **expression)
 	return cond;
 }
 
+static FindCondition *parse_dash(guchar **expression)
+{
+	guchar *exp = *expression;
+	FindCondition *cond, *retval = NULL;
+	IsTest	 test;
+	int i = 1;
+
+	if (NEXT != '-')
+		return NULL;
+
+	while (exp[i] && !isspace(exp[i]))
+	{
+		switch (exp[i])
+		{
+			case 'f': test = IS_REG; break;;
+			case 'l': test = IS_LNK; break;;
+			case 'd': test = IS_DIR; break;;
+			case 'b': test = IS_BLK; break;;
+			case 'c': test = IS_CHR; break;;
+			case 'D': test = IS_DEV; break;;
+			case 'p': test = IS_FIFO; break;;
+			case 'S': test = IS_SOCK; break;;
+			case 'u': test = IS_SUID; break;;
+			case 'g': test = IS_SGID; break;;
+			case 'k': test = IS_STICKY; break;;
+			case 'r': test = IS_READABLE; break;;
+			case 'w': test = IS_WRITEABLE; break;;
+			case 'x': test = IS_EXEC; break;;
+			case 'o': test = IS_MINE; break;;
+			case 'z': test = IS_EMPTY; break;;
+			default:
+				  if (retval)
+					  find_condition_free(retval);
+				  return NULL;
+		}
+		i++;
+
+		cond = g_new(FindCondition, 1);
+		cond->test = &test_is;
+		cond->free = (FindFree) &g_free;
+		cond->data1 = NULL;
+		cond->data2 = NULL;
+		cond->value = test;
+
+		if (retval)
+		{
+			FindCondition *new;
+
+			new = g_new(FindCondition, 1);
+			new->test = &test_AND;
+			new->free = &free_branch;
+			new->data1 = retval;
+			new->data2 = cond;
+
+			retval = new;
+		}
+		else
+			retval = cond;
+	}
+
+	(*expression) += i;
+
+	return retval;
+}
+
 /* Returns NULL if expression is not an is-expression */
 static FindCondition *parse_is(guchar **expression)
 {
