@@ -1,5 +1,5 @@
 /*
- * $Id: fscache.c,v 1.12 2001/07/25 15:30:14 tal197 Exp $
+ * $Id: fscache.c,v 1.15 2001/12/22 19:32:12 tal197 Exp $
  *
  * FSCache - a glib-style object for caching files
  * Copyright (C) 2001, the ROX-Filer team.
@@ -30,6 +30,8 @@
 
 #include "config.h"
 
+#include "global.h"
+
 #include "fscache.h"
 
 #define UPTODATE(data, info)				\
@@ -138,16 +140,39 @@ void g_fscache_data_unref(GFSCache *cache, gpointer data)
  */
 gpointer g_fscache_lookup(GFSCache *cache, char *pathname)
 {
-	return g_fscache_lookup_full(cache, pathname, FSCACHE_LOOKUP_CREATE);
+	return g_fscache_lookup_full(cache, pathname,
+			FSCACHE_LOOKUP_CREATE, NULL);
 }
 
-/* As g_fscache_lookup, but 'load' flags control what happens if the data
+/* Force this already-loaded item into the cache. The cache will
+ * ref the object if it wants to keep it.
+ */
+void g_fscache_insert(GFSCache *cache, char *pathname, gpointer obj)
+{
+	GFSCacheData	*data;
+
+	data = g_fscache_lookup_full(cache, pathname,
+					FSCACHE_LOOKUP_INIT, NULL);
+
+	if (!data)
+		return;
+
+	if (data->data && cache->unref)
+		cache->unref(data->data, cache->user_data);
+	data->data = obj;
+	if (cache->ref)
+		cache->ref(data->data, cache->user_data);
+}
+
+/* As g_fscache_lookup, but 'lookup_type' controls what happens if the data
  * is out-of-date.
- * TRUE => Data is updated / reloaded
- * FALSE => Returns NULL
+ * If found is not NULL, use it to indicate whether the cache has an entry
+ * for this file (a NULL return could indicate that the data is cached, but
+ * the data is NULL).
  */
 gpointer g_fscache_lookup_full(GFSCache *cache, char *pathname,
-				FSCacheLookup lookup_type)
+				FSCacheLookup lookup_type,
+				gboolean *found)
 {
 	struct stat 	info;
 	GFSCacheKey	key;
@@ -157,18 +182,26 @@ gpointer g_fscache_lookup_full(GFSCache *cache, char *pathname,
 	g_return_val_if_fail(pathname != NULL, NULL);
 
 	if (mc_stat(pathname, &info))
+	{
+		if (found)
+			*found = FALSE;
 		return NULL;
+	}
 
 	key.device = info.st_dev;
 	key.inode = info.st_ino;
 
 	data = g_hash_table_lookup(cache->inode_to_stats, &key);
 
+	if (found)
+		*found = (data != NULL);
+
 	if (data)
 	{
 		/* We've cached this file already */
 
-		if (lookup_type == FSCACHE_LOOKUP_PEEK)
+		if (lookup_type == FSCACHE_LOOKUP_PEEK ||
+		    lookup_type == FSCACHE_LOOKUP_INIT)
 			goto out;	/* Never update on peeks */
 		
 		/* Is it up-to-date? */
@@ -193,7 +226,8 @@ gpointer g_fscache_lookup_full(GFSCache *cache, char *pathname,
 	{
 		GFSCacheKey *new_key;
 
-		if (lookup_type != FSCACHE_LOOKUP_CREATE)
+		if (lookup_type != FSCACHE_LOOKUP_CREATE &&
+		    lookup_type != FSCACHE_LOOKUP_INIT)
 			return NULL;
 		
 		new_key = g_memdup(&key, sizeof(key));
@@ -208,16 +242,19 @@ gpointer g_fscache_lookup_full(GFSCache *cache, char *pathname,
 	data->length = info.st_size;
 	data->mode = info.st_mode;
 
-	if (data->data == NULL)
+	if (data->data == NULL && lookup_type != FSCACHE_LOOKUP_INIT)
 	{
 		/* Create the object for the file (ie, not an update) */
 		if (cache->load)
 			data->data = cache->load(pathname, cache->user_data);
 	}
 out:
+	data->last_lookup = time(NULL);
+	if (lookup_type == FSCACHE_LOOKUP_INIT)
+		return data;
+
 	if (cache->ref)
 		cache->ref(data->data, cache->user_data);
-	data->last_lookup = time(NULL);
 	return data->data;
 }
 
