@@ -1,5 +1,5 @@
 /*
- * $Id: fscache.h,v 1.8 2001/07/25 15:30:14 tal197 Exp $
+ * $Id: fscache.h,v 1.11 2001/12/22 19:32:12 tal197 Exp $
  *
  * Thomas Leonard, <tal197@users.sourceforge.net>
  */
@@ -14,7 +14,6 @@
 #include <time.h>
 #include <glib.h>
 
-typedef struct _GFSCache GFSCache;
 typedef struct _GFSCacheKey GFSCacheKey;
 typedef struct _GFSCacheData GFSCacheData;
 typedef gpointer (*GFSLoadFunc)(char *pathname, gpointer user_data);
@@ -57,6 +56,7 @@ typedef enum {
 	FSCACHE_LOOKUP_CREATE,	/* Load if missing. Update as needed. */
 	FSCACHE_LOOKUP_ONLY_NEW,/* Return NULL if not present AND uptodate */
 	FSCACHE_LOOKUP_PEEK,	/* Lookup; don't load or update */
+	FSCACHE_LOOKUP_INIT,	/* Internal use */
 } FSCacheLookup;
 
 GFSCache *g_fscache_new(GFSLoadFunc load,
@@ -68,7 +68,8 @@ GFSCache *g_fscache_new(GFSLoadFunc load,
 void g_fscache_destroy(GFSCache *cache);
 gpointer g_fscache_lookup(GFSCache *cache, char *pathname);
 gpointer g_fscache_lookup_full(GFSCache *cache, char *pathname,
-				FSCacheLookup lookup_type);
+				FSCacheLookup lookup_type,
+				gboolean *found);
 void g_fscache_may_update(GFSCache *cache, char *pathname);
 void g_fscache_update(GFSCache *cache, char *pathname);
 void g_fscache_purge(GFSCache *cache, gint age);
@@ -76,4 +77,6 @@ void g_fscache_purge(GFSCache *cache, gint age);
 void g_fscache_data_ref(GFSCache *cache, gpointer data);
 void g_fscache_data_unref(GFSCache *cache, gpointer data);
 
+void g_fscache_insert(GFSCache *cache, char *pathname, gpointer obj);
+
 #endif /* _FSCACHE_H */
