@@ -1,5 +1,5 @@
 /*
- * $Id: gui_support.c,v 1.37 2001/10/30 19:20:43 tal197 Exp $
+ * $Id: gui_support.c,v 1.51 2001/12/26 20:41:05 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2001, the ROX-Filer team.
@@ -34,20 +34,23 @@
 #include <X11/Xatom.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdk.h>
-#include "collection.h"
 
 #include "global.h"
 
 #include "main.h"
 #include "gui_support.h"
+#include "support.h"
 #include "pixmaps.h"
+#include "choices.h"
 
+#ifndef GTK2
 GdkFont	   	*item_font = NULL;
 GdkFont	   	*fixed_font = NULL;
 GtkStyle   	*fixed_style = NULL;
 gint		fixed_width;
-GdkColor 	red = {0, 0xffff, 0, 0};
+#endif
 GdkGC		*red_gc = NULL;	/* Not automatically initialised */
+GdkColor 	red = {0, 0xffff, 0, 0};
 gint		screen_width, screen_height;
 
 static GdkAtom xa_cardinal;
@@ -56,42 +59,37 @@ static GtkWidget *current_dialog = NULL;
 
 void gui_support_init()
 {
+#ifndef GTK2
 	GtkWidget *tmp;
 
-	fixed_font = gdk_font_load("fixed");
-
 	/* Create a window and get its font rather than using
 	 * the default style (allows customisation via .gtkrc)
 	 */
 	tmp = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_widget_realize(tmp);
-#ifdef GTK2
-	item_font = gtk_style_get_font(gtk_widget_get_style(tmp));
-#else
 	item_font = gtk_widget_get_style(tmp)->font;
-#endif
 	gdk_font_ref(item_font);
 	gtk_widget_destroy(tmp);
 
+	fixed_font = gdk_font_load("fixed");
+
 	fixed_style = gtk_style_copy(gtk_widget_get_default_style());
-#ifdef GTK2
-	gtk_style_set_font(fixed_style, fixed_font);
-#else
 	fixed_style->font = fixed_font;
-#endif
 
 	fixed_width = gdk_string_width(fixed_font, "m");
-
-	xa_cardinal  = gdk_atom_intern("CARDINAL", FALSE);
+#endif
 
 	gdk_color_alloc(gtk_widget_get_default_colormap(), &red);
 
+	xa_cardinal  = gdk_atom_intern("CARDINAL", FALSE);
+
 	/* This call starts returning strange values after a while, so get
 	 * the result here during init.
 	 */
 	gdk_window_get_size(GDK_ROOT_PARENT(), &screen_width, &screen_height);
 }
 
+#ifndef GTK2
 static void choice_clicked(GtkWidget *widget, gpointer number)
 {
 	int	*choice_return;
@@ -102,6 +100,7 @@ static void choice_clicked(GtkWidget *widget, gpointer number)
 	if (choice_return)
 		*choice_return = GPOINTER_TO_INT(number);
 }
+#endif
 
 /* Open a modal dialog box showing a message.
  * The user can choose from a selection of buttons at the bottom.
@@ -116,12 +115,14 @@ int get_choice(char *title,
 	       int number_of_buttons, ...)
 {
 	GtkWidget	*dialog;
-	GtkWidget	*vbox, *action_area, *separator;
-	GtkWidget	*text, *text_container;
-	GtkWidget	*button = NULL, *default_button;
+	GtkWidget	*button = NULL;
 	int		i, retval;
 	va_list	ap;
+#ifndef GTK2
+	GtkWidget	*vbox, *action_area, *separator;
+	GtkWidget	*text, *text_container;
 	int		choice_return;
+#endif
 
 	if (current_dialog)
 	{
@@ -130,11 +131,17 @@ int get_choice(char *title,
 		return -1;
 	}
 
+#ifdef GTK2
+	current_dialog = dialog = gtk_message_dialog_new(NULL,
+					GTK_DIALOG_MODAL,
+					GTK_MESSAGE_QUESTION,
+					GTK_BUTTONS_NONE,
+					"%s", message);
+	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
+#else
 	current_dialog = dialog = gtk_window_new(GTK_WINDOW_DIALOG);
 	GTK_WIDGET_UNSET_FLAGS(dialog, GTK_CAN_FOCUS);
 	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
-	gtk_window_set_title(GTK_WINDOW(dialog), title);
-	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
 	gtk_container_set_border_width(GTK_CONTAINER(dialog), 2);
 
 	vbox = gtk_vbox_new(FALSE, 0);
@@ -156,12 +163,16 @@ int get_choice(char *title,
 	gtk_box_pack_start(GTK_BOX(vbox),
 			text_container,
 			TRUE, TRUE, 0);
+#endif
 
 	va_start(ap, number_of_buttons);
 
-	default_button = NULL;
 	for (i = 0; i < number_of_buttons; i++)
 	{
+#ifdef GTK2
+		button = gtk_dialog_add_button(GTK_DIALOG(current_dialog),
+				va_arg(ap, char *), i);
+#else
 		button = gtk_button_new_with_label(va_arg(ap, char *));
 		GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
 		gtk_misc_set_padding(GTK_MISC(GTK_BIN(button)->child), 16, 2);
@@ -170,17 +181,26 @@ int get_choice(char *title,
 		gtk_box_pack_start(GTK_BOX(action_area), button, TRUE, TRUE, 0);
 		gtk_signal_connect(GTK_OBJECT(button), "clicked",
 			GTK_SIGNAL_FUNC(choice_clicked), GINT_TO_POINTER(i));
-		if (!default_button)
-			default_button = button;
+#endif
 	}
 
-	gtk_window_set_focus(GTK_WINDOW(dialog), default_button);
-	gtk_window_set_default(GTK_WINDOW(dialog), default_button);
-	gtk_container_set_focus_child(GTK_CONTAINER(action_area),
-			default_button);
+	gtk_window_set_title(GTK_WINDOW(dialog), title);
+	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
+
+	gtk_window_set_focus(GTK_WINDOW(dialog), button);
+	gtk_window_set_default(GTK_WINDOW(dialog), button);
+#ifndef GTK2
+	gtk_container_set_focus_child(GTK_CONTAINER(action_area), button);
+#endif
 
 	va_end(ap);
 
+#ifdef GTK2
+	retval = gtk_dialog_run(GTK_DIALOG(dialog));
+	if (retval == GTK_RESPONSE_NONE)
+		retval = -1;
+	gtk_widget_destroy(dialog);
+#else
 	gtk_object_set_data(GTK_OBJECT(dialog), "choice_return",
 				&choice_return);
 	gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
@@ -197,21 +217,41 @@ int get_choice(char *title,
 
 	if (retval != -1)
 		gtk_widget_destroy(dialog);
+#endif
 
 	current_dialog = NULL;
 
 	return retval;
 }
 
-/* Display a message in a window */
-void report_error(char *title, char *message)
+/* Display a message in a window with "ROX-Filer" as title */
+void report_error(char *message, ...)
 {
+        va_list args;
+	gchar *s;
+
 	g_return_if_fail(message != NULL);
 
-	if (!title)
-		title = _("Error");
+	va_start(args, message);
+	s = g_strdup_vprintf(message, args);
+	va_end(args);
 
-	get_choice(title, message, 1, "OK");
+#ifdef GTK2
+	{
+		GtkWidget *dialog;
+		dialog = gtk_message_dialog_new(NULL,
+				GTK_DIALOG_MODAL,
+				GTK_MESSAGE_ERROR,
+				GTK_BUTTONS_OK,
+				"%s", s);
+		gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
+		gtk_dialog_run(GTK_DIALOG(dialog));
+		gtk_widget_destroy(dialog);
+	}
+#else
+	get_choice(PROJECT, s, 1, _("OK"));
+#endif
+	g_free(s);
 }
 
 void set_cardinal_property(GdkWindow *window, GdkAtom prop, guint32 value)
@@ -223,11 +263,16 @@ void set_cardinal_property(GdkWindow *window, GdkAtom prop, guint32 value)
 /* NB: Also used for pinned icons.
  * TODO: Set the level here too.
  */
-void make_panel_window(GdkWindow *window)
+void make_panel_window(GtkWidget *widget)
 {
 	static gboolean need_init = TRUE;
-	static GdkAtom	xa_state, xa_atom, xa_net_state;
+	static GdkAtom	xa_state, xa_atom, xa_net_state, xa_hints;
 	static GdkAtom	state_list[3];
+	GdkWindow *window = widget->window;
+	gint32  values[2];
+	
+
+	g_return_if_fail(window != NULL);
 
 	if (override_redirect)
 	{
@@ -241,12 +286,16 @@ void make_panel_window(GdkWindow *window)
 		xa_state = gdk_atom_intern("_WIN_STATE", FALSE);
 		xa_atom = gdk_atom_intern("ATOM", FALSE);
 		xa_net_state = gdk_atom_intern("_NET_WM_STATE", FALSE);
+		xa_hints = gdk_atom_intern("WM_HINTS", FALSE);
 
+		/* Note: Starting with Gtk+-1.3.12, Gtk+ converts GdkAtoms
+		 * to X atoms automatically when the type is ATOM.
+		 */
 		state_list[0] = gdk_atom_intern("_NET_WM_STATE_STICKY", FALSE);
 		state_list[1] = gdk_atom_intern("_NET_WM_STATE_SKIP_PAGER",
-							FALSE);
+						FALSE);
 		state_list[2] = gdk_atom_intern("_NET_WM_STATE_SKIP_TASKBAR",
-							FALSE);
+						FALSE);
 		
 		need_init = FALSE;
 	}
@@ -254,6 +303,10 @@ void make_panel_window(GdkWindow *window)
 	gdk_window_set_decorations(window, 0);
 	gdk_window_set_functions(window, 0);
 
+	/* Note: DON'T do gtk_window_stick(). Setting the state via
+	 * gdk will override our other atoms (pager/taskbar).
+	 */
+
 	/* Don't hide panel/pinboard windows initially (WIN_STATE_HIDDEN).
 	 * Needed for IceWM - Christopher Arndt <chris.arndt@web.de>
 	 */
@@ -263,6 +316,14 @@ void make_panel_window(GdkWindow *window)
 
 	gdk_property_change(window, xa_net_state, xa_atom, 32,
 			GDK_PROP_MODE_APPEND, (guchar *) state_list, 3);
+
+	g_return_if_fail(window != NULL);
+
+	values[0] = 1;		/* InputHint */
+	values[1] = False;
+
+	gdk_property_change(window, xa_hints, xa_hints, 32,
+			GDK_PROP_MODE_REPLACE, (guchar *) values, 2);
 }
 
 gint hide_dialog_event(GtkWidget *widget, GdkEvent *event, gpointer window)
@@ -276,10 +337,9 @@ static gboolean error_idle_cb(gpointer data)
 {
 	char	**error = (char **) data;
 	
-	report_error(error[0], error[1]);
-	g_free(error[0]);
-	g_free(error[1]);
-	error[0] = error[1] = NULL;
+	report_error("%s", *error);
+	g_free(*error);
+	*error = NULL;
 
 	if (--number_of_windows == 0)
 		gtk_main_quit();
@@ -287,28 +347,40 @@ static gboolean error_idle_cb(gpointer data)
 	return FALSE;
 }
 
-/* Display an error next time we are idle */
-void delayed_error(char *title, char *error)
+/* Display an error with "ROX-Filer" as title next time we are idle.
+ * If multiple errors are reported this way before the window is opened,
+ * all are displayed in a single window.
+ * If an error is reported while the error window is open, it is discarded.
+ */
+void delayed_error(char *error, ...)
 {
-	static char *delayed_error_data[2] = {NULL, NULL};
-	gboolean already_open;
-	
+	static char *delayed_error_data = NULL;
+	char *old, *new;
+	va_list args;
+
 	g_return_if_fail(error != NULL);
 
-	already_open = delayed_error_data[1] != NULL;
+	old = delayed_error_data;
 
-	g_free(delayed_error_data[0]);
-	g_free(delayed_error_data[1]);
+	va_start(args, error);
+	new = g_strdup_vprintf(error, args);
+	va_end(args);
 
-	delayed_error_data[0] = g_strdup(title);
-	delayed_error_data[1] = g_strdup(error);
-	
-	if (already_open)
-		return;
+	if (old)
+	{
+		delayed_error_data = g_strconcat(old,
+				_("\n--- next error ---\n"),
+				new, NULL);
+		g_free(old);
+		g_free(new);
+	}
+	else
+	{
+		delayed_error_data = new;
+		gtk_idle_add(error_idle_cb, &delayed_error_data);
 
-	gtk_idle_add(error_idle_cb, delayed_error_data);
-
-	number_of_windows++;
+		number_of_windows++;
+	}
 }
 
 /* Load the file into memory. Return TRUE on success.
@@ -316,6 +388,21 @@ void delayed_error(char *title, char *error)
  */
 gboolean load_file(char *pathname, char **data_out, long *length_out)
 {
+#ifdef GTK2
+	gsize len;
+	GError *error;
+	
+	if (!g_file_get_contents(pathname, data_out, &len, &error))
+	{
+		delayed_error("%s", error ? error->message : pathname);
+		g_error_free(error);
+		return FALSE;
+	}
+		
+	if (length_out)
+		*length_out = len;
+	return TRUE;
+#else
 	FILE		*file;
 	long		length;
 	char		*buffer;
@@ -325,12 +412,7 @@ gboolean load_file(char *pathname, char **data_out, long *length_out)
 
 	if (!file)
 	{
-		guchar	*message;
-
-		message = g_strdup_printf("open(%s): %s",
-				pathname, g_strerror(errno));
-		delayed_error(PROJECT, message);
-		g_free(message);
+		delayed_error("open(%s): %s", pathname, g_strerror(errno));
 		return FALSE;
 	}
 
@@ -345,12 +427,8 @@ gboolean load_file(char *pathname, char **data_out, long *length_out)
 
 		if (ferror(file))
 		{
-			guchar *tmp;
-
-			tmp = g_strdup_printf("%s: %s\n",
+			delayed_error(_("Error reading file %s: %s"),
 					pathname, g_strerror(errno));
-			delayed_error(_("Error reading file"), tmp);
-			g_free(tmp);
 			g_free(buffer);
 		}
 		else
@@ -362,13 +440,13 @@ gboolean load_file(char *pathname, char **data_out, long *length_out)
 		}
 	}
 	else
-		delayed_error(PROJECT,
-			_("Can't allocate memory for buffer to "
+		delayed_error(_("Can't allocate memory for buffer to "
 				"load this file"));
 
 	fclose(file);
 
 	return retval;
+#endif
 }
 
 GtkWidget *new_help_button(HelpFunc show_help, gpointer data)
@@ -413,10 +491,7 @@ void parse_file(char *path, ParseFunc *parse_line)
 
 			if (error && !seen_error)
 			{
-				GString *message;
-
-				message = g_string_new(NULL);
-				g_string_sprintf(message,
+				delayed_error(
 		_("Error in '%s' file at line %d: "
 		"\n\"%s\"\n"
 		"This may be due to upgrading from a previous version of "
@@ -425,8 +500,6 @@ void parse_file(char *path, ParseFunc *parse_line)
 					path,
 					line_number,
 					error);
-				delayed_error(PROJECT, message->str);
-				g_string_free(message, TRUE);
 				seen_error = TRUE;
 			}
 
@@ -695,11 +768,6 @@ void centre_window(GdkWindow *window, int x, int y)
 		CLAMP(y, DECOR_BORDER, screen_height - h - DECOR_BORDER));
 }
 
-static void set_to_null(gpointer *data)
-{
-	*data = NULL;
-}
-
 /* Non-NULL => selector window is open */
 static GtkColorSelectionDialog *current_csel_box = NULL;
 
@@ -855,3 +923,55 @@ void destroy_on_idle(GtkWidget *widget)
 	gtk_widget_ref(widget);
 	gtk_idle_add((GtkFunction) idle_destroy_cb, widget);
 }
+
+/* Spawn a child process (as spawn_full), and report errors.
+ * TRUE on success.
+ */
+gboolean rox_spawn(gchar *dir, gchar **argv)
+{
+#ifdef GTK2
+	GError	*error = NULL;
+	
+	if (!g_spawn_async_with_pipes(dir, argv, NULL,
+			G_SPAWN_DO_NOT_REAP_CHILD |
+			G_SPAWN_SEARCH_PATH,
+			NULL, NULL,		/* Child setup fn */
+			NULL,			/* Child PID */
+			NULL, NULL, NULL,	/* Standard pipes */
+			&error))
+	{
+		delayed_error("%s", error ? error->message : "(null)");
+		g_error_free(error);
+
+		return FALSE;
+	}
+#else
+	if (spawn_full(argv, dir, NULL) == 0)
+	{
+		report_error("Failed to start child process:\n%s",
+				g_strerror(errno));
+		return FALSE;
+	}
+#endif
+	return TRUE;
+}
+
+/* Called before gtk_init(). Override default styles with our defaults,
+ * and override them with user choices, if any.
+ */
+void add_default_styles(void)
+{
+	gchar	*rc_file;
+	
+	rc_file = g_strconcat(app_dir, "/Styles", NULL);
+	gtk_rc_add_default_file(rc_file);
+	g_free(rc_file);
+
+	rc_file = choices_find_path_load("Styles", PROJECT);
+	if (rc_file)
+	{
+		gtk_rc_add_default_file(rc_file);
+		g_free(rc_file);
+	}
+}
+
