@@ -1,5 +1,5 @@
 /*
- * $Id: i18n.c,v 1.16 2001/05/09 10:53:26 tal197 Exp $
+ * $Id: i18n.c,v 1.20 2001/12/16 14:58:42 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2001, the ROX-Filer team.
@@ -25,9 +25,6 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
-#ifdef HAVE_LOCALE_H
-#  include <locale.h>
-#endif
 
 #include "global.h"
 
@@ -38,10 +35,12 @@
 #include "gui_support.h"
 #include "main.h"
 
+/* Used to stop the translation-changed message on startup */
+static gboolean doing_init;
+
 /* Static Prototypes */
-static char *load_trans(guchar *lang);
 static void set_trans(guchar *lang);
-static void save(void);
+static void trans_changed(guchar *lang);
 
 /****************************************************************
  *			EXTERNAL INTERFACE			*
@@ -51,27 +50,15 @@ static void save(void);
 /* Set things up for internationalisation */
 void i18n_init(void)
 {
-	guchar		*trans;
+	gtk_set_locale();
 
-#ifdef HAVE_LOCALE_H
-	setlocale(LC_ALL, "");
-#endif
-	
-	trans = choices_find_path_load("Translation", "ROX-Filer");
-
-	if (trans)
-	{
-		parse_file(trans, load_trans);
-		g_free(trans);
-	}
-	else
-		load_trans("From LANG");
-
-	option_add_saver(save);
+	doing_init = TRUE;
+	option_add_string("i18n_translation", "From LANG", trans_changed);
+	doing_init = FALSE;
 }
 
 /* These two stolen from dia :-).
- * Slight modification though: '/%s/' means 'same as above' so that
+ * Slight modification though: '>' means 'same as above' so that
  * if a translation is missing it doesn't muck up the whole menu structure!
  */
 GtkItemFactoryEntry *translate_entries(GtkItemFactoryEntry *entries, gint n)
@@ -162,26 +149,10 @@ void free_translated_entries(GtkItemFactoryEntry *entries, gint n)
 static void trans_changed(guchar *lang)
 {
 	set_trans(lang);
-	delayed_error(PROJECT,
-		_("You must restart the filer for the new language "
-		  "setting to take full effect"));
-}
-
-/* Just read the Translation file on startup */
-static char *load_trans(guchar *lang)
-{
-	static gboolean init = FALSE;
-
-	if (!init)
-	{
-		init = TRUE;
-		option_add_string("i18n_translation", lang, trans_changed);
-		option_set_save("i18n_translation", FALSE);
-	}
-
-	set_trans(lang);
-
-	return NULL;
+	if (!doing_init)
+		delayed_error(
+			_("You must restart the filer for the new language "
+			  "setting to take full effect"));
 }
 
 /* Load the 'Messages/<name>.gmo' translation.
@@ -220,34 +191,3 @@ static void set_trans(guchar *lang)
 	g_free(path);
 	g_free(lang2);
 }
-
-static void save(void)
-{
-	guchar	*path;
-	int	len, err;
-	FILE	*f;
-	guchar	*lang;
-
-	path = choices_find_path_save("Translation", "ROX-Filer", TRUE);
-	if (!path)
-		return;
-
-	lang = option_get_static_string("i18n_translation");
-
-	f = fopen(path, "wb");
-	g_free(path);
-	if (!f)
-	{
-		delayed_error(PROJECT, g_strerror(errno));
-		return;
-	}
-
-	len = strlen(lang);
-
-	err = (fwrite(lang, 1, len, f) < len) |
-		(fwrite("\n", 1, 1, f) < 1)   |
-		  (fclose(f) != 0);
-
-	if (err)
-		delayed_error(PROJECT, g_strerror(errno));
-}
