@@ -1,5 +1,5 @@
 /*
- * $Id: icon.c,v 1.26 2001/10/10 14:28:05 tal197 Exp $
+ * $Id: icon.c,v 1.35 2001/12/24 12:13:04 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2001, the ROX-Filer team.
@@ -37,6 +37,7 @@
 #include "pinboard.h"
 #include "panel.h"
 #include "icon.h"
+#include "diritem.h"
 #include "menu.h"
 #include "appmenu.h"
 #include "dnd.h"
@@ -55,8 +56,6 @@ static GtkWidget	*icon_file_menu;	/* The file submenu */
 static GtkWidget	*icon_file_item;	/* 'File' label */
 static GtkWidget	*file_shift_item;	/* 'Shift Open' label */
 
-static GtkTooltips	*tooltips = NULL;
-
 /* Widget which holds the selection when we have it */
 static GtkWidget *selection_invisible = NULL;
 static guint losing_selection = 0;	/* > 0 => Don't send events */
@@ -137,7 +136,7 @@ void icon_init(void)
 
 	item_factory = menu_create(menu_def,
 				sizeof(menu_def) / sizeof(*menu_def),
-				 "<icon>");
+				 "<icon>", NULL);
 
 	tmp = g_strconcat("<icon>/", _("File"), NULL);
 	icon_menu = gtk_item_factory_get_widget(item_factory, "<icon>");
@@ -249,14 +248,14 @@ void icon_may_update(Icon *icon)
 
 	g_return_if_fail(icon != NULL);
 
-	image = icon->item.image;
-	flags = icon->item.flags;
+	image = icon->item->image;
+	flags = icon->item->flags;
 
 	pixmap_ref(image);
 	mount_update(FALSE);
-	diritem_restat(icon->path, &icon->item, FALSE);
+	diritem_restat(icon->path, icon->item);
 
-	if (icon->item.image != image || icon->item.flags != flags)
+	if (icon->item->image != image || icon->item->flags != flags)
 	{
 		/* Appearance changed; need to redraw */
 		if (icon->panel)
@@ -301,7 +300,7 @@ static void check_has(gpointer key, GList *icons, CheckData *check)
 void icon_set_tip(Icon *icon)
 {
 	GtkWidget	*widget;
-	AppInfo		*ai;
+	XMLwrapper	*ai;
 	xmlNode 	*node;
 
 	g_return_if_fail(icon != NULL);
@@ -319,26 +318,29 @@ void icon_set_tip(Icon *icon)
 	if (icon->panel && icon->socket)
 		ai = NULL;
 	else
-		ai = appinfo_get(icon->path, &icon->item);
+		ai = appinfo_get(icon->path, icon->item);
 
 	if (ai && ((node = appinfo_get_section(ai, "Summary"))))
 	{
 		guchar *str;
 		str = xmlNodeListGetString(node->doc,
 				node->xmlChildrenNode, 1);
-		gtk_tooltips_set_tip(tooltips, widget, str, NULL);
-		g_free(str);
+		if (str)
+		{
+			gtk_tooltips_set_tip(tooltips, widget, str, NULL);
+			g_free(str);
+		}
 	}
 	else if (icon->panel && (!panel_want_show_text(icon)) && !icon->socket)
 	{
 		gtk_tooltips_set_tip(tooltips, widget,
-				icon->item.leafname, NULL);
+				icon->item->leafname, NULL);
 	}
 	else
 		gtk_tooltips_set_tip(tooltips, widget, NULL, NULL);
 
 	if (ai)
-		appinfo_unref(ai);
+		xml_cache_unref(ai);
 }
 
 /* Returns TRUE if any icon links to this item (or any item inside
@@ -389,7 +391,7 @@ void icon_show_menu(GdkEventButton *event, Icon *icon, Panel *panel)
 		(icon_selection || menu_icon) ? FALSE : TRUE,
 		4, 1);
 
-	menu_show_shift_action(file_shift_item, icon ? &icon->item : NULL,
+	menu_show_shift_action(file_shift_item, icon ? icon->item : NULL,
 			FALSE);
 
 	/* Shade the File/Edit/Show items unless an item was clicked */
@@ -399,17 +401,17 @@ void icon_show_menu(GdkEventButton *event, Icon *icon, Panel *panel)
 
 		menu_set_items_shaded(icon_menu, FALSE, 1, 3);
 		menu_set_items_shaded(icon_file_menu, FALSE, 0, 5);
-		if (!can_set_run_action(&icon->item))
+		if (!can_set_run_action(icon->item))
 			menu_set_items_shaded(icon_file_menu, TRUE, 3, 1);
 
 		tmp = g_strdup_printf("%s '%s'",
-				basetype_name(&icon->item),
-				icon->item.leafname);
+				basetype_name(icon->item),
+				icon->item->leafname);
 		gtk_label_set_text(GTK_LABEL(icon_file_item), tmp);
 		g_free(tmp);
 
 		/* Check for app-specific menu */
-		appmenu_add(icon->path, &icon->item, icon_menu);
+		appmenu_add(icon->path, icon->item, icon_menu);
 	}
 	else
 	{
@@ -548,7 +550,7 @@ void icon_destroyed(Icon *icon)
 				g_list_remove(current_pinboard->icons, icon);
 	}
 	
-	diritem_clear(&icon->item);
+	diritem_free(icon->item);
 	g_free(icon->path);
 	g_free(icon->src_path);
 	g_free(icon);
@@ -594,13 +596,13 @@ static void rename_activate(GtkWidget *dialog)
 	new_src = gtk_entry_get_text(GTK_ENTRY(src));
 	
 	if (*new_name == '\0')
-		report_error(PROJECT,
+		report_error(
 			_("The label must contain at least one character!"));
 	else if (*new_src == '\0')
-		report_error(PROJECT,
+		report_error(
 			_("The location must contain at least one character!"));
 	else if (strpbrk(new_name, "<>"))
-		report_error(PROJECT,
+		report_error(
 			_("Sorry, but the name must not contain < or >"));
 	else
 	{
@@ -608,7 +610,7 @@ static void rename_activate(GtkWidget *dialog)
 		GdkFont	*font = icon->widget->style->font;
 #endif
 		
-		g_free(icon->item.leafname);
+		g_free(icon->item->leafname);
 		g_free(icon->src_path);
 
 		icon->src_path = g_strdup(new_src);
@@ -618,13 +620,13 @@ static void rename_activate(GtkWidget *dialog)
 		icon->path = icon_convert_path(new_src);
 		icon_hash_path(icon);
 		
-		icon->item.leafname = g_strdup(new_name);
+		icon->item->leafname = g_strdup(new_name);
 #ifndef GTK2
-		icon->item.name_width = gdk_string_measure(font, new_name);
+		icon->name_width = gdk_string_measure(font, new_name);
 #endif
 		/* XXX: Set name_width in size_and_shape? */
 
-		diritem_restat(icon->path, &icon->item, FALSE);
+		diritem_restat(icon->path, icon->item);
 
 		callback(icon);
 		gtk_widget_destroy(dialog);
@@ -746,7 +748,7 @@ static void remove_items(gpointer data, guint action, GtkWidget *widget)
 	
 	if (!next)
 	{
-		delayed_error(PROJECT,
+		delayed_error(
 			_("You must first select some items to remove"));
 		return;
 	}
@@ -785,15 +787,14 @@ static void file_op(gpointer data, guint action, GtkWidget *widget)
 {
 	if (!menu_icon)
 	{
-		delayed_error(PROJECT,
-			_("You must open the menu over an item"));
+		delayed_error(_("You must open the menu over an item"));
 		return;
 	}
 
 	switch (action)
 	{
 		case ACTION_SHIFT:
-			run_diritem(menu_icon->path, &menu_icon->item,
+			run_diritem(menu_icon->path, menu_icon->item,
 					NULL, NULL, TRUE);
 			break;
 		case ACTION_EDIT:
@@ -804,22 +805,22 @@ static void file_op(gpointer data, guint action, GtkWidget *widget)
 			open_to_show(menu_icon->path);
 			break;
 		case ACTION_HELP:
-			show_item_help(menu_icon->path, &menu_icon->item);
+			show_item_help(menu_icon->path, menu_icon->item);
 			break;
 		case ACTION_INFO:
 			infobox_new(menu_icon->path);
 			break;
 		case ACTION_RUN_ACTION:
-			if (can_set_run_action(&menu_icon->item))
+			if (can_set_run_action(menu_icon->item))
 				type_set_handler_dialog(
-						menu_icon->item.mime_type);
+						menu_icon->item->mime_type);
 			else
-				report_error(PROJECT,
+				report_error(
 				_("You can only set the run action for a "
 				"regular file"));
 			break;
 		case ACTION_SET_ICON:
-			icon_set_handler_dialog(&menu_icon->item,
+			icon_set_handler_dialog(menu_icon->item,
 						menu_icon->path);
 			break;
 	}
@@ -836,6 +837,10 @@ static void show_rename_box(GtkWidget *widget, Icon *icon, RenameFn callback)
 	GtkWidget	*dialog, *hbox, *vbox, *label, *entry, *button;
 
 	dialog = gtk_window_new(GTK_WINDOW_DIALOG);
+#ifdef GTK2
+	gtk_window_set_type_hint(GTK_WINDOW(dialog),
+			GDK_WINDOW_TYPE_HINT_DIALOG);
+#endif
 	gtk_window_set_title(GTK_WINDOW(dialog), _("Edit Item"));
 	gtk_container_set_border_width(GTK_CONTAINER(dialog), 10);
 
@@ -858,7 +863,7 @@ static void show_rename_box(GtkWidget *widget, Icon *icon, RenameFn callback)
 	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
 	entry = gtk_entry_new();
 	gtk_box_pack_start(GTK_BOX(vbox), entry, TRUE, FALSE, 2);
-	gtk_entry_set_text(GTK_ENTRY(entry), icon->item.leafname);
+	gtk_entry_set_text(GTK_ENTRY(entry), icon->item->leafname);
 	gtk_editable_select_region(GTK_EDITABLE(entry), 0, -1);
 	gtk_widget_grab_focus(entry);
 	gtk_object_set_data(GTK_OBJECT(dialog), "new_name", entry);
