@@ -1,5 +1,5 @@
 /*
- * $Id: icon.h,v 1.11 2001/09/09 15:46:36 tal197 Exp $
+ * $Id: icon.h,v 1.13 2001/12/17 14:13:00 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * By Thomas Leonard, <tal197@users.sourceforge.net>.
@@ -14,9 +14,6 @@
 # include <pango/pango.h>
 #endif
 
-#include "dir.h"
-#include "diritem.h"
-
 extern GList *icon_selection;
 extern gboolean tmp_icon_selected;
 
@@ -26,13 +23,14 @@ struct _Icon {
 	gboolean	selected;
 	guchar		*src_path;	/* Eg: ~/Apps */
 	guchar		*path;		/* Eg: /home/fred/Apps */
-	DirItem		item;
+	DirItem		*item;
 
 	/* Only used on the pinboard... */
 	GtkWidget	*win;
 	GdkBitmap	*mask;
 	int		x, y;
 	int		width, height;
+	int		name_width;
 #ifdef GTK2
 	PangoLayout	*layout;	/* The label */
 #endif
