@@ -1,5 +1,5 @@
 /*
- * $Id: main.c,v 1.109 2001/10/15 18:02:07 tal197 Exp $
+ * $Id: main.c,v 1.118 2001/12/27 20:58:08 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2001, the ROX-Filer team.
@@ -19,6 +19,12 @@
  * Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+/* main.c - parses command-line options and parameters, plus some global
+ * 	    housekeeping.
+ *
+ * New to the code and feeling lost? Read global.h now.
+ */
+
 #include "config.h"
 
 #include <stdlib.h>
@@ -31,13 +37,13 @@
 #include <fcntl.h>
 #include <pwd.h>
 #include <grp.h>
+#include <parser.h>
 
 #ifdef HAVE_GETOPT_LONG
 #  include <getopt.h>
 #endif
 
 #include <gtk/gtk.h>
-#include "collection.h"
 
 #include "global.h"
 
@@ -54,6 +60,7 @@
 #include "type.h"
 #include "pixmaps.h"
 #include "dir.h"
+#include "diritem.h"
 #include "action.h"
 #include "i18n.h"
 #include "remote.h"
@@ -62,13 +69,14 @@
 #include "toolbar.h"
 #include "bind.h"
 #include "icon.h"
-#include "appinfo.h"
 #include "panel.h"
 #include "session.h"
+#include "minibuffer.h"
 
 int number_of_windows = 0;	/* Quit when this reaches 0 again... */
 static int to_wakeup_pipe = -1;	/* Write here to get noticed */
 
+/* Information about the ROX-Filer process */
 uid_t euid;
 gid_t egid;
 int ngroups;			/* Number of supplemental groups */
@@ -80,6 +88,8 @@ guchar *show_user_message = NULL;
 int home_dir_len;
 char *home_dir, *app_dir;
 
+GtkTooltips *tooltips = NULL;
+
 #define COPYING								\
 	     N_("Copyright (C) 2001 Thomas Leonard.\n"			\
 		"ROX-Filer comes with ABSOLUTELY NO WARRANTY,\n"	\
@@ -161,7 +171,6 @@ static gboolean child_died_flag = FALSE;
 
 /* Static prototypes */
 static void show_features(void);
-static xmlDocPtr soap_new(xmlNodePtr *ret_body);
 static void soap_add(xmlNodePtr body,
 			   xmlChar *function,
 			   xmlChar *arg1_name, xmlChar *arg1_value,
@@ -177,6 +186,11 @@ static void wake_up_cb(gpointer data, gint source, GdkInputCondition condition);
 /* The value that goes with an option */
 #define VALUE (*optarg == '=' ? optarg + 1 : optarg)
 
+/* Parses the command-line to work out what the user wants to do.
+ * Tries to send the request to an already-running copy of the filer.
+ * If that fails, it initialises all the other modules and executes the
+ * request itself.
+ */
 int main(int argc, char **argv)
 {
 	int		 wakeup_pipe[2];
@@ -185,14 +199,18 @@ int main(int argc, char **argv)
 	guchar		*tmp, *dir, *slash;
 	gchar *client_id = NULL;
 	gboolean	show_user = FALSE;
-	xmlDocPtr	rpc, soap_rpc = NULL;
+	xmlDocPtr	rpc, soap_rpc = NULL, reply;
 	xmlNodePtr	body;
 
 	home_dir = g_get_home_dir();
 	home_dir_len = strlen(home_dir);
 	app_dir = g_strdup(getenv("APP_DIR"));
 
+	/* Get internationalisation up and running. This requires the
+	 * choices system, to discover the user's preferred language.
+	 */
 	choices_init();
+	options_init();
 	i18n_init();
 
 	if (!app_dir)
@@ -209,12 +227,17 @@ int main(int argc, char **argv)
 	}
 #endif
 
+	/* Sometimes we want to take special action when a child
+	 * process exits. This hash table is used to convert the
+	 * child's PID to the callback function.
+	 */
 	death_callbacks = g_hash_table_new(NULL, NULL);
 
 #ifdef HAVE_LIBVFS
 	mc_vfs_init();
 #endif
 
+	/* Find out some information about ourself */
 	euid = geteuid();
 	egid = getegid();
 	ngroups = getgroups(0, NULL);
@@ -233,6 +256,13 @@ int main(int argc, char **argv)
 	 */
 	rpc = soap_new(&body);
 
+	/* Note: must do this before checking our options,
+	 * otherwise we report an error for Gtk's options.
+	 */
+	add_default_styles();
+	gtk_init(&argc, &argv);
+
+	/* Process each option in turn */
 	while (1)
 	{
 		int	c;
@@ -340,13 +370,15 @@ int main(int argc, char **argv)
 		}
 	}
 
-	add_default_styles();
-	gtk_init(&argc, &argv);
+	tooltips = gtk_tooltips_new();
 
 	if (euid == 0 || show_user)
 		show_user_message = g_strdup_printf( _("Running as user '%s'"), 
 				user_name(euid));
 	
+	/* Add each remaining (non-option) argument to the list of files
+	 * to run.
+	 */
 	i = optind;
 	while (i < argc)
 	{
@@ -367,6 +399,9 @@ int main(int argc, char **argv)
 	}
 	else if (!body->xmlChildrenNode)
 	{
+		/* The user didn't request any action. Open the current
+		 * directory.
+		 */
 		guchar	*dir;
 
 		dir = g_get_current_dir();
@@ -376,9 +411,10 @@ int main(int argc, char **argv)
 
 	option_add_int("dnd_no_hostnames", 1, NULL);
 
+	/* Try to send the request to an already-running copy of the filer */
 	gui_support_init();
 	if (remote_init(rpc, new_copy))
-		return EXIT_SUCCESS;	/* Already running */
+		return EXIT_SUCCESS;	/* It worked - exit */
 
 	/* Put ourselves into the background (so 'rox' always works the
 	 * same, whether we're already running or not).
@@ -395,6 +431,22 @@ int main(int argc, char **argv)
 		 * it!).
 		 */
 	}
+
+	/* Close stdin. We don't need it, and it can cause problems if
+	 * a child process wants a password, etc...
+	 */
+	{
+		int fd;
+		fd = open("/dev/null", O_RDONLY);
+		if (fd > 0)
+		{
+			close(0);
+			dup2(fd, 0);
+			close(fd);
+		}
+	}
+	
+	/* Initialize the rest of the filer... */
 	
 	pixmaps_init();
 
@@ -408,17 +460,20 @@ int main(int argc, char **argv)
 	toolbar_init();
 	display_init();
 	mount_init();
-	options_init();
 	type_init();
 	action_init();
-	appinfo_init();
 
 	icon_init();
 	pinboard_init();
 	panel_init();
 
-	options_load();
+	/* Let everyone update */
+	options_notify();
 
+	/* When we get a signal, we can't do much right then. Instead,
+	 * we send a char down this pipe, which causes the main loop to
+	 * deal with the event next time we're idle.
+	 */
 	pipe(wakeup_pipe);
 	close_on_exec(wakeup_pipe[0], TRUE);
 	close_on_exec(wakeup_pipe[1], TRUE);
@@ -444,9 +499,19 @@ int main(int argc, char **argv)
 	session_init(client_id);
 	g_free(client_id);
 		
-	run_soap(rpc);
+	/* Finally, execute the request */
+	reply = run_soap(rpc);
 	xmlFreeDoc(rpc);
+	if (reply)
+	{
+		/* Write the result, if any, to stdout */
+		save_xml_file(reply, "-");
+		xmlFreeDoc(reply);
+	}
 
+	/* Enter the main loop, processing events until all our windows
+	 * are closed.
+	 */
 	if (number_of_windows > 0)
 		gtk_main();
 
@@ -475,35 +540,24 @@ void on_child_death(gint child, CallbackFn callback, gpointer data)
 static void show_features(void)
 {
 	g_printerr("\n-- %s --\n\n", _("features set at compile time"));
-	g_printerr("%s... %s\n", _("VFS support"),
+	g_printerr("%s... %s\n", _("Old VFS support"),
 #ifdef HAVE_LIBVFS
 		_("Yes")
 #else
+# ifdef LARGE_FILE_SUPPORT
+		_("No (incompatible with Large File Support)")
+# else
 		_("No (couldn't find a valid libvfs)")
+# endif
+#endif
+		);
+	g_printerr("%s... %s\n", _("GTK+-2.0 support"),
+#ifdef GTK2
+		_("Yes")
+#else
+		_("No (using Gtk+-1.2 instead)")
 #endif
 		);
-}
-
-/* Create a new SOAP message and return the document and the (empty)
- * body node.
- */
-static xmlDocPtr soap_new(xmlNodePtr *ret_body)
-{
-	xmlDocPtr  doc;
-	xmlNodePtr root;
-	xmlNs	   *env_ns;
-
-	doc = xmlNewDoc("1.0");
-	root = xmlNewDocNode(doc, NULL, "Envelope", NULL);
-	xmlDocSetRootElement(doc, root);
-	
-	env_ns = xmlNewNs(root, SOAP_ENV_NS, "env");
-	xmlSetNs(root, env_ns);
-
-	*ret_body = xmlNewTextChild(root, env_ns, "Body", NULL);
-	xmlNewNs(*ret_body, ROX_NS, "rox");
-
-	return doc;
 }
 
 static void soap_add(xmlNodePtr body,
