@@ -1,5 +1,5 @@
 /*
- * $Id: menu.h,v 1.19 2001/09/09 13:41:02 tal197 Exp $
+ * $Id: menu.h,v 1.21 2001/11/26 13:53:11 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * By Thomas Leonard, <tal197@users.sourceforge.net>.
@@ -12,16 +12,12 @@
 
 extern GtkAccelGroup	*filer_keys;
 extern GtkWidget 	*popup_menu;
-extern char		*xterm_here_value;
-
-extern GtkWidget	*display_large_menu;	/* Display->Large With... */
-extern GtkWidget	*display_small_menu;	/* Display->Small With... */
 
 void menu_init(void);
 
 void show_filer_menu(FilerWindow *filer_window, GdkEvent *event, int item);
-GtkItemFactory *menu_create(GtkItemFactoryEntry *def,
-			int n_entries, guchar *name);
+GtkItemFactory *menu_create(GtkItemFactoryEntry *def, int n_entries,
+			    guchar *name, GtkAccelGroup *keys);
 void menu_set_items_shaded(GtkWidget *menu, gboolean shaded, int from, int n);
 void position_menu(GtkMenu *menu, gint *x, gint *y,
 #ifdef GTK2
