@@ -1,5 +1,5 @@
 /*
- * $Id: minibuffer.c,v 1.36 2001/09/25 14:30:39 tal197 Exp $
+ * $Id: minibuffer.c,v 1.41 2001/12/20 19:11:51 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2001, the ROX-Filer team.
@@ -41,9 +41,10 @@
 #include "support.h"
 #include "minibuffer.h"
 #include "filer.h"
+#include "display.h"
 #include "main.h"
 #include "action.h"
-#include "dir.h"
+#include "diritem.h"
 #include "type.h"
 
 static GList *shell_history = NULL;
@@ -61,7 +62,9 @@ static gboolean matches(Collection *collection, int item, char *pattern);
 static void search_in_dir(FilerWindow *filer_window, int dir);
 static guchar *mini_contents(FilerWindow *filer_window);
 static void show_help(FilerWindow *filer_window);
-
+#ifdef GTK2
+static gboolean grab_focus(GtkWidget *minibuffer);
+#endif
 
 /****************************************************************
  *			EXTERNAL INTERFACE			*
@@ -91,12 +94,19 @@ void create_minibuffer(FilerWindow *filer_window)
 
 	mini = gtk_entry_new();
 	gtk_box_pack_start(GTK_BOX(hbox), mini, TRUE, TRUE, 0);
-	gtk_widget_set_style(mini, fixed_style);
+	gtk_widget_set_name(mini, "fixed-style");
 	gtk_signal_connect(GTK_OBJECT(mini), "key_press_event",
 			GTK_SIGNAL_FUNC(key_press_event), filer_window);
 	gtk_signal_connect(GTK_OBJECT(mini), "changed",
 			GTK_SIGNAL_FUNC(changed), filer_window);
 
+#ifdef GTK2
+	/* Grabbing focus musn't select the text... */
+	gtk_signal_connect_object(GTK_OBJECT(mini), "grab-focus",
+		GTK_SIGNAL_FUNC(grab_focus),
+		GTK_OBJECT(mini));
+#endif
+
 	filer_window->minibuffer = mini;
 	filer_window->minibuffer_label = label;
 	filer_window->minibuffer_area = hbox;
@@ -112,7 +122,6 @@ void minibuffer_show(FilerWindow *filer_window, MiniType mini_type)
 	g_return_if_fail(filer_window->minibuffer != NULL);
 
 	mini = GTK_ENTRY(filer_window->minibuffer);
-	gtk_widget_set_style(filer_window->minibuffer, fixed_style);
 
 	filer_window->mini_type = MINI_NONE;
 	gtk_label_set_text(GTK_LABEL(filer_window->minibuffer_label),
@@ -184,7 +193,15 @@ void minibuffer_hide(FilerWindow *filer_window)
 
 	if (filer_window->temp_show_hidden)
 	{
-		display_set_hidden(filer_window, FALSE);
+		Collection *collection = filer_window->collection;
+		int i = collection->cursor_item;
+		DirItem *item = NULL;
+
+		if (i >= 0 && i < collection->number_of_items)
+			item = (DirItem *) collection->items[i].data;
+		
+		if (item == NULL || item->leafname[0] != '.')
+			display_set_hidden(filer_window, FALSE);
 		filer_window->temp_show_hidden = FALSE;
 	}
 }
@@ -245,7 +262,7 @@ static void show_help(FilerWindow *filer_window)
 			break;
 	}
 
-	delayed_error(PROJECT, message);
+	delayed_error("%s", message);
 }
 
 
@@ -722,8 +739,7 @@ static void shell_return_pressed(FilerWindow *filer_window)
 	switch (child)
 	{
 		case -1:
-			delayed_error(PROJECT, _("Failed to create "
-					"child process"));
+			delayed_error(_("Failed to create child process"));
 			break;
 		case 0:	/* Child */
 			/* Ensure output is noticed - send stdout to stderr */
@@ -794,7 +810,7 @@ static void select_return_pressed(FilerWindow *filer_window, guint etime)
 	cond = find_compile(entry);
 	if (!cond)
 	{
-		delayed_error(PROJECT, "Invalid Find condition");
+		delayed_error(_("Invalid Find condition"));
 		return;
 	}
 
@@ -962,3 +978,17 @@ static guchar *mini_contents(FilerWindow *filer_window)
 	return NULL;
 }
 
+#ifdef GTK2
+static gboolean grab_focus(GtkWidget *minibuffer)
+{
+	GtkWidgetClass *class;
+	
+	class = GTK_WIDGET_CLASS(gtk_type_class(GTK_TYPE_WIDGET));
+
+	class->grab_focus(minibuffer);
+
+	gtk_signal_emit_stop_by_name(GTK_OBJECT(minibuffer), "grab_focus");
+
+	return 1;
+}
+#endif
