@@ -1,5 +1,5 @@
 /*
- * $Id: options.c,v 1.48 2001/09/20 13:40:20 tal197 Exp $
+ * $Id: options.c,v 1.58 2001/12/27 13:41:53 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2001, the ROX-Filer team.
@@ -23,32 +23,41 @@
 
 /* How it works:
  *
- * On startup, each part of the filer calls option_add_int() or a related
- * function, supplying a name for the option and a default value.
+ * On startup:
  *
- * Once everything has initialised, the options file (if any) is read,
- * which may change some or all of the values. Update callbacks are called
- * if values change from their defaults.
+ * - The <Choices>/PROJECT/Options file is read in. Each line
+ *   is a name/value pair, and these are stored in the 'loading' hash table.
  *
- * All the notify_callbacks are called.
+ * - Each part of the filer then calls option_add_int(), or a related function,
+ *   supplying the name for the option and a default value. Once an option is
+ *   registered, it is removed from the loading table.
+ *   Update callbacks are called if the default value isn't used.
+ *
+ * - option_register_widget() can be used during initialisation (any time
+ *   before the Options box is displayed) to tell the system how to render a
+ *   particular type of option.
+ *
+ * - All notify callbacks are called.
  *
  * When the user opens the Options box:
+ *
  * - The Options.xml file is read and used to create the Options dialog box.
  *   Each element in the file has a key corresponding to an option named
  *   above.
+ *
  * - For each widget in the box, the current value of the option is used to
  *   set the widget's state.
  *
- * When the user clicks Save/OK/Apply, the state of each widget is copied
- * to the options values and the update callbacks are called for those values
- * which have changed.
+ * When the user clicks Save/OK/Apply:
  *
- * All the notify_callbacks are called.
+ * - The state of each widget is copied to the options values.
  *
- * If Save or OK was clicked then the box is also closed.
+ * - The update callbacks are called for those values which have changed.
  *
- * If Save was clicked then the options are written to the filesystem
- * and the saver_callbacks are called.
+ * - If Save or OK was clicked then the box is also closed.
+ *
+ * - If Save was clicked then the options are written to the filesystem and the
+ *   saver_callbacks are called.
  */
 
 #include "config.h"
@@ -68,7 +77,6 @@
 #include "options.h"
 #include "main.h"
 #include "support.h"
-#include "toolbar.h"
 
 typedef struct _Option Option;
 
@@ -80,21 +88,8 @@ static GtkTooltips *option_tooltips = NULL;
 /* The Options window. NULL if not yet created. */
 static GtkWidget *window = NULL;
 
-/* The various widget types */
-typedef enum {
-	OPTION_MENU,
-	OPTION_ITEM,
-	OPTION_RADIO_GROUP,
-	OPTION_TOGGLE,
-	OPTION_ENTRY,
-	OPTION_SLIDER,
-	OPTION_COLOUR,
-	OPTION_TOOLS,
-} OptionType;
-
 struct _Option {
-	GtkWidget	*widget;	/* NULL => No widget yet */
-	OptionType	widget_type;
+	OptionUI	*ui;		/* NULL => No UI yet */
 	guchar		*value;
 	OptionChanged	*changed_cb;
 	gboolean	save;		/* Save to options file */
@@ -105,6 +100,17 @@ enum {BUTTON_SAVE, BUTTON_OK, BUTTON_APPLY};
 /* "filer_unique" -> (Option *) */
 static GHashTable *option_hash = NULL;
 
+/* A mapping (name -> value) for options which have been loaded by not
+ * yet registered. The options in this table cannot be used until
+ * option_add_*() is called to move them into option_hash.
+ */
+static GHashTable *loading = NULL;
+
+/* A mapping (XML name -> OptionBuildFn). When reading the Options.xml
+ * file, this table gives the function used to create the widgets.
+ */
+static GHashTable *widget_builder = NULL;
+
 /* List of functions to call after all option values are updated */
 static GList *notify_callbacks = NULL;
 
@@ -117,8 +123,14 @@ static char *process_option_line(guchar *line);
 static void build_options_window(void);
 static GtkWidget *build_frame(void);
 static void update_option_widgets(void);
-static Option *new_option(guchar *key, OptionChanged *changed);
-static guchar *tools_to_list(GtkWidget *hbox);
+static Option *new_option(guchar *key, OptionChanged *changed, guchar *def);
+
+static GList *build_toggle(OptionUI *ui, xmlNode *node, guchar *label);
+static GList *build_slider(OptionUI *ui, xmlNode *node, guchar *label);
+static GList *build_entry(OptionUI *ui, xmlNode *node, guchar *label);
+static GList *build_radio_group(OptionUI *ui, xmlNode *node, guchar *label);
+static GList *build_colour(OptionUI *ui, xmlNode *node, guchar *label);
+static GList *build_menu(OptionUI *ui, xmlNode *node, guchar *label);
 
 
 /****************************************************************
@@ -127,21 +139,52 @@ static guchar *tools_to_list(GtkWidget *hbox);
 
 void options_init(void)
 {
-	option_tooltips = gtk_tooltips_new();
-}
-
-void options_load(void)
-{
-	GList	*next;
 	char	*path;
 
+	loading = g_hash_table_new(g_str_hash, g_str_equal);
+	option_hash = g_hash_table_new(g_str_hash, g_str_equal);
+	widget_builder = g_hash_table_new(g_str_hash, g_str_equal);
+
 	path = choices_find_path_load("Options", PROJECT);
 	if (!path)
 		return;		/* Nothing to load */
 
+	/* Load in all the options set in the filer, storing them
+	 * temporarily in the loading hash table.
+	 * They get moved to option_hash when they're registered.
+	 */
 	parse_file(path, process_option_line);
 	g_free(path);
 
+	option_register_widget("toggle", build_toggle);
+	option_register_widget("slider", build_slider);
+	option_register_widget("entry", build_entry);
+	option_register_widget("radio-group", build_radio_group);
+	option_register_widget("colour", build_colour);
+	option_register_widget("menu", build_menu);
+}
+
+/* When parsing the XML file, process an element named 'name' by
+ * calling 'builder(ui, xml_node, label)'.
+ * builder returns the new widgets to add to the options box.
+ * 'name' should be a static string.
+ *
+ * Functions to set or get the widget's state can be stored in 'ui'.
+ * If the option doesn't have a name attribute in Options.xml then
+ * ui will be NULL on entry (this is used for buttons).
+ */
+void option_register_widget(char *name, OptionBuildFn builder)
+{
+	g_hash_table_insert(widget_builder, name, builder);
+}
+
+/* Call all the notify callbacks. This should happen after any options
+ * have their values changed.
+ */
+void options_notify(void)
+{
+	GList	*next;
+
 	for (next = notify_callbacks; next; next = next->next)
 	{
 		OptionNotify *cb = (OptionNotify *) next->data;
@@ -152,6 +195,15 @@ void options_load(void)
 
 void options_show(void)
 {
+	if (!option_tooltips)
+		option_tooltips = gtk_tooltips_new();
+
+	if (g_hash_table_size(loading) != 0)
+	{
+		g_printerr(PROJECT ": Some options loaded but not used:\n");
+		g_hash_table_foreach(loading, (GHFunc) puts, NULL);
+	}
+
 	if (window)
 		gtk_widget_destroy(window);
 
@@ -165,10 +217,7 @@ void options_show(void)
 /* Create a new option and register it */
 void option_add_int(guchar *key, int value, OptionChanged *changed)
 {
-	Option	*option;
-
-	option = new_option(key, changed);
-	option->value = g_strdup_printf("%d", value);
+	new_option(key, changed, g_strdup_printf("%d", value));
 }
 
 int option_get_int(guchar *key)
@@ -184,10 +233,7 @@ int option_get_int(guchar *key)
 
 void option_add_string(guchar *key, guchar *value, OptionChanged *changed)
 {
-	Option	*option;
-
-	option = new_option(key, changed);
-	option->value = g_strdup(value);
+	new_option(key, changed, g_strdup(value));
 }
 
 /* The string returned is only valid until the option value changes.
@@ -223,47 +269,50 @@ void option_add_saver(OptionNotify *callback)
 	saver_callbacks = g_list_append(saver_callbacks, callback);
 }
 
-/* Set whether this option should be saved in the options file */
-void option_set_save(guchar *key, gboolean save)
-{
-	Option	*option;
-
-	option = g_hash_table_lookup(option_hash, key);
-	g_return_if_fail(option != NULL);
-
-	option->save = save;
-}
-
 /****************************************************************
  *                      INTERNAL FUNCTIONS                      *
  ****************************************************************/
 
-static Option *new_option(guchar *key, OptionChanged *changed)
+/* 'def' is placed directly into the new Option. It will be g_free()d
+ * automatically.
+ */
+static Option *new_option(guchar *key, OptionChanged *changed, guchar *def)
 {
 	Option	*option;
+	gpointer okey, value;
 
-	if (!option_hash)
-		option_hash = g_hash_table_new(g_str_hash, g_str_equal);
+	g_return_val_if_fail(option_hash != NULL, NULL);
+	g_return_val_if_fail(g_hash_table_lookup(option_hash, key) == NULL,
+									NULL);
 	
 	option = g_new(Option, 1);
 
 	option->save = TRUE;	/* Save by default */
-	option->widget = NULL;
-	option->value = NULL;
+	option->ui = NULL;
 	option->changed_cb = changed;
-	
+
 	g_hash_table_insert(option_hash, key, option);
 
+	/* Use the value loaded from the file, if any */
+	if (g_hash_table_lookup_extended(loading, key, &okey, &value))
+	{
+		option->value = value;
+		g_hash_table_remove(loading, key);
+		g_free(okey);
+
+		if (changed && strcmp(def, option->value) != 0)
+			changed(option->value);
+
+		g_free(def);	/* Don't need the default */
+	}
+	else
+		option->value = def;
+
 	return option;
 }
 
 static GtkColorSelectionDialog *current_csel_box = NULL;
 
-static void set_to_null(gpointer *data)
-{
-	*data = NULL;
-}
-
 static void get_new_colour(GtkWidget *ok, GtkWidget *button)
 {
 	GtkWidget	*csel;
@@ -318,7 +367,6 @@ static void open_coloursel(GtkWidget *ok, GtkWidget *button)
 }
 
 /* These are used during parsing... */
-static guchar *section_name = NULL;
 static xmlDocPtr options_doc = NULL;
 
 #define DATA(node) (xmlNodeListGetString(options_doc, node->xmlChildrenNode, 1))
@@ -353,7 +401,7 @@ static int get_int(xmlNode *node, guchar *attr)
 	return retval;
 }
 
-static GtkWidget *build_radio(xmlNode *radio, GtkWidget *box, GtkWidget *prev)
+static GtkWidget *build_radio(xmlNode *radio, GtkWidget *prev)
 {
 	GtkWidget	*button;
 	GtkRadioButton	*prev_button = (GtkRadioButton *) prev;
@@ -367,7 +415,6 @@ static GtkWidget *build_radio(xmlNode *radio, GtkWidget *box, GtkWidget *prev)
 			_(label));
 	g_free(label);
 
-	gtk_box_pack_start(GTK_BOX(box), button, FALSE, TRUE, 0);
 	may_add_tip(button, radio);
 
 	gtk_object_set_data(GTK_OBJECT(button), "value",
@@ -402,12 +449,13 @@ static void show_notice(GtkObject *button)
 	text = gtk_object_get_data(button, "notice_text");
 	g_return_if_fail(text != NULL);
 
-	report_error(_("Notice"), _(text));
+	report_error("%s", _(text));
 }
 
 static void build_widget(xmlNode *widget, GtkWidget *box)
 {
 	const char *name = widget->name;
+	OptionBuildFn builder;
 	guchar	*oname;
 	Option	*option;
 	guchar	*label;
@@ -486,218 +534,50 @@ static void build_widget(xmlNode *widget, GtkWidget *box)
 	}
 
 	oname = xmlGetProp(widget, "name");
-	g_return_if_fail(oname != NULL);
 
+	if (oname)
 	{
-		guchar	*tmp;
-
-		tmp = g_strconcat(section_name, "_", oname, NULL);
-		g_free(oname);
-		option = g_hash_table_lookup(option_hash, tmp);
+		option = g_hash_table_lookup(option_hash, oname);
 
 		if (!option)
 		{
-			g_warning("No Option for '%s'!\n", tmp);
-			g_free(tmp);
+			g_warning("No Option for '%s'!\n", oname);
+			g_free(oname);
 			return;
 		}
 
-		g_free(tmp);
+		g_free(oname);
 	}
+	else
+		option = NULL;
 
-	if (strcmp(name, "toggle") == 0)
+	builder = g_hash_table_lookup(widget_builder, name);
+	if (builder)
 	{
-		GtkWidget	*toggle;
+		GList *widgets, *next;
 
-		toggle = gtk_check_button_new_with_label(_(label));
-		
-		gtk_box_pack_start(GTK_BOX(box), toggle, FALSE, TRUE, 0);
-		may_add_tip(toggle, widget);
-
-		option->widget_type = OPTION_TOGGLE;
-		option->widget = toggle;
-	}
-	else if (strcmp(name, "slider") == 0)
-	{
-		GtkAdjustment *adj;
-		GtkWidget *hbox, *slide;
-		int	min, max;
-		int	fixed;
-		int	showvalue;
-
-		min = get_int(widget, "min");
-		max = get_int(widget, "max");
-		fixed = get_int(widget, "fixed");
-		showvalue = get_int(widget, "showvalue");
-
-		adj = GTK_ADJUSTMENT(gtk_adjustment_new(0,
-					min, max, 1, 10, 0));
-
-		hbox = gtk_hbox_new(FALSE, 4);
-		gtk_box_pack_start(GTK_BOX(hbox),
-				gtk_label_new(_(label)),
-				FALSE, TRUE, 0);
-
-		slide = gtk_hscale_new(adj);
-
-		if (fixed)
-			gtk_widget_set_usize(slide, adj->upper, 24);
-		if (showvalue)
+		if (option)
 		{
-			gtk_scale_set_draw_value(GTK_SCALE(slide), TRUE);
-			gtk_scale_set_value_pos(GTK_SCALE(slide),
-						GTK_POS_LEFT);
-			gtk_scale_set_digits(GTK_SCALE(slide), 0);
+			if (option->ui)
+				g_warning("UI for option already exists!");
+			else
+			{
+				option->ui = g_new(OptionUI, 1);
+				option->ui->update_widget = NULL;
+				option->ui->read_widget = NULL;
+			}
+
+			widgets = builder(option->ui, widget, label);
 		}
-		else 
-			gtk_scale_set_draw_value(GTK_SCALE(slide), FALSE);
-		GTK_WIDGET_UNSET_FLAGS(slide, GTK_CAN_FOCUS);
+		else
+			widgets = builder(NULL, widget, label);
 
-		may_add_tip(slide, widget);
-		
-		gtk_box_pack_start(GTK_BOX(hbox), slide, !fixed, TRUE, 0);
-
-		gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, TRUE, 0);
-
-		option->widget_type = OPTION_SLIDER;
-		option->widget = slide;
-	}
-	else if (strcmp(name, "entry") == 0)
-	{
-		GtkWidget	*hbox;
-		GtkWidget	*entry;
-
-		hbox = gtk_hbox_new(FALSE, 4);
-
-		gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_(label)),
-					FALSE, TRUE, 0);
-
-		entry = gtk_entry_new();
-		gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
-		may_add_tip(entry, widget);
-
-		gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, TRUE, 0);
-
-		option->widget_type = OPTION_ENTRY;
-		option->widget = entry;
-	}
-	else if (strcmp(name, "radio-group") == 0)
-	{
-		GtkWidget	*button = NULL;
-		xmlNode		*rn;
-
-		for (rn = widget->xmlChildrenNode; rn; rn = rn->next)
+		for (next = widgets; next; next = next->next)
 		{
-			if (rn->type == XML_ELEMENT_NODE)
-				button = build_radio(rn, box, button);
+			GtkWidget *w = (GtkWidget *) next->data;
+			gtk_box_pack_start(GTK_BOX(box), w, FALSE, TRUE, 0);
 		}
-
-		option->widget_type = OPTION_RADIO_GROUP;
-		option->widget = button;
-	}
-	else if (strcmp(name, "colour") == 0)
-	{
-		GtkWidget	*hbox, *da, *button;
-		int		lpos;
-		
-		/* lpos gives the position for the label 
-		 * 0: label comes before the button
-		 * non-zero: label comes after the button
-		 */
-		lpos = get_int(widget, "lpos");
-
-		hbox = gtk_hbox_new(FALSE, 4);
-		if (lpos == 0)
-			gtk_box_pack_start(GTK_BOX(hbox),
-				gtk_label_new(_(label)),
-				FALSE, TRUE, 0);
-
-		button = gtk_button_new();
-		da = gtk_drawing_area_new();
-		gtk_drawing_area_size(GTK_DRAWING_AREA(da), 64, 12);
-		gtk_container_add(GTK_CONTAINER(button), da);
-		gtk_signal_connect(GTK_OBJECT(button), "clicked",
-				GTK_SIGNAL_FUNC(open_coloursel), button);
-
-		may_add_tip(button, widget);
-		
-		gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, TRUE, 0);
-		if (lpos)
-			gtk_box_pack_start(GTK_BOX(hbox),
-				gtk_label_new(_(label)),
-				FALSE, TRUE, 0);
-
-		gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, TRUE, 0);
-
-		option->widget_type = OPTION_COLOUR;
-		option->widget = button;
-	}
-	else if (strcmp(name, "menu") == 0)
-	{
-		GtkWidget	*hbox, *om, *option_menu;
-		xmlNode		*item;
-		GtkWidget	*menu;
-		GList		*list, *kids;
-		int		min_w = 4, min_h = 4;
-
-		hbox = gtk_hbox_new(FALSE, 4);
-		gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, TRUE, 0);
-
-		gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_(label)),
-				FALSE, TRUE, 0);
-
-		option_menu = gtk_option_menu_new();
-		gtk_box_pack_start(GTK_BOX(hbox), option_menu, FALSE, TRUE, 0);
-
-		om = gtk_menu_new();
-		gtk_option_menu_set_menu(GTK_OPTION_MENU(option_menu), om);
-
-		for (item = widget->xmlChildrenNode; item; item = item->next)
-		{
-			if (item->type == XML_ELEMENT_NODE)
-				build_menu_item(item, option_menu);
-		}
-
-		menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(option_menu));
-		list = kids = gtk_container_children(GTK_CONTAINER(menu));
-
-		while (kids)
-		{
-			GtkWidget	*item = (GtkWidget *) kids->data;
-			GtkRequisition	req;
-
-			gtk_widget_size_request(item, &req);
-			if (req.width > min_w)
-				min_w = req.width;
-			if (req.height > min_h)
-				min_h = req.height;
-			
-			kids = kids->next;
-		}
-
-		g_list_free(list);
-
-		gtk_widget_set_usize(option_menu,
-				min_w + 50,	/* Else widget doesn't work! */
-				min_h + 4);
-
-		option->widget_type = OPTION_MENU;
-		option->widget = option_menu;
-	}
-	else if (strcmp(name, "tool-options") == 0)
-	{
-		int		i = 0;
-		GtkWidget	*hbox, *tool;
-
-		hbox = gtk_hbox_new(FALSE, 0);
-
-		while ((tool = toolbar_tool_option(i++)))
-			gtk_box_pack_start(GTK_BOX(hbox), tool, FALSE, TRUE, 0);
-
-		gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, TRUE, 0);
-
-		option->widget_type = OPTION_TOOLS;
-		option->widget = hbox;
+		g_list_free(widgets);
 	}
 	else
 		g_warning("Unknown option type '%s'\n", name);
@@ -705,7 +585,6 @@ static void build_widget(xmlNode *widget, GtkWidget *box)
 	g_free(label);
 }
 
-
 static void build_sections(xmlNode *options, GtkWidget *sections_box)
 {
 	xmlNode	*section = options->xmlChildrenNode;
@@ -722,7 +601,6 @@ static void build_sections(xmlNode *options, GtkWidget *sections_box)
 			continue;
 
 		title = xmlGetProp(section, "title");
-		section_name = xmlGetProp(section, "name");
 		page = gtk_vbox_new(FALSE, 0);
 		gtk_container_set_border_width(GTK_CONTAINER(page), 4);
 
@@ -747,8 +625,6 @@ static void build_sections(xmlNode *options, GtkWidget *sections_box)
 		}
 
 		g_free(title);
-		g_free(section_name);
-		section_name = NULL;
 	}
 }
 
@@ -765,7 +641,7 @@ static void build_options_window(void)
 	options_doc = xmlParseFile(make_path(app_dir, "Options.xml")->str);
 	if (!options_doc)
 	{
-		report_error(PROJECT, "Internal error: Options.xml unreadable");
+		report_error("Internal error: Options.xml unreadable");
 		return;
 	}
 
@@ -775,11 +651,14 @@ static void build_options_window(void)
 	options_doc = NULL;
 }
 
-static void null_widget(gpointer key, gpointer value, gpointer data)
+static void null_ui(gpointer key, gpointer value, gpointer data)
 {
 	Option	*option = (Option *) value;
 
-	option->widget = NULL;
+	g_return_if_fail(option->ui != NULL);
+
+	g_free(option->ui);
+	option->ui = NULL;
 }
 
 static void options_destroyed(GtkWidget *widget, gpointer data)
@@ -788,7 +667,7 @@ static void options_destroyed(GtkWidget *widget, gpointer data)
 	{
 		window = NULL;
 
-		g_hash_table_foreach(option_hash, null_widget, NULL);
+		g_hash_table_foreach(option_hash, null_ui, NULL);
 	}
 }
 
@@ -824,9 +703,8 @@ static GtkWidget *build_frame(void)
 	save_path = choices_find_path_save("...", PROJECT, FALSE);
 	if (save_path)
 	{
-		string = g_strconcat(_("Choices will be saved as "),
-					save_path,
-					NULL);
+		string = g_strdup_printf(_("Choices will be saved as %s"),
+					save_path);
 		g_free(save_path);
 		label = gtk_label_new(string);
 		g_free(string);
@@ -877,7 +755,7 @@ static GtkWidget *build_frame(void)
 static char *process_option_line(guchar *line)
 {
 	guchar 		*eq, *c;
-	Option		*option;
+	char		*name = line;
 
 	g_return_val_if_fail(option_hash != NULL, "No registered options!");
 
@@ -893,17 +771,10 @@ static char *process_option_line(guchar *line)
 	while (*c == ' ' || *c == '\t')
 		c++;
 
-	option = g_hash_table_lookup(option_hash, line);
-	if (!option)
-		return NULL;	/* Unknown option - silently ignore */
+	if (g_hash_table_lookup(loading, name))
+		return "Duplicate option found!";
 
-	if (strcmp(option->value, line) == 0)
-		return NULL;	/* Value unchanged */
-
-	g_free(option->value);
-	option->value = g_strdup(g_strstrip(c));
-	if (option->changed_cb)
-		option->changed_cb(option->value);
+	g_hash_table_insert(loading, g_strdup(name), g_strdup(g_strstrip(c)));
 
 	return NULL;
 }
@@ -1010,50 +881,16 @@ static guchar *option_menu_get(GtkOptionMenu *om)
  */
 static void may_change_cb(gpointer key, gpointer value, gpointer data)
 {
-	Option		*option = (Option *) value;
+	Option *option = (Option *) value;
 	guchar		*new = NULL;
-	GtkWidget	*widget = option->widget;
-	GtkStyle	*style;
 
-	switch (option->widget_type)
-	{
-		case OPTION_TOGGLE:
-			new = g_strdup_printf("%d",
-				gtk_toggle_button_get_active(
-					GTK_TOGGLE_BUTTON(widget)));
-			break;
-		case OPTION_ENTRY:
-			new = gtk_editable_get_chars(GTK_EDITABLE(widget),
-					0, -1);
-			break;
-		case OPTION_SLIDER:
-			new = g_strdup_printf("%f",
-				gtk_range_get_adjustment(
-					GTK_RANGE(widget))->value);
-			break;
-		case OPTION_RADIO_GROUP:
-			new = radio_group_get_value(GTK_RADIO_BUTTON(widget));
-			break;
-		case OPTION_MENU:
-			new = g_strdup(option_menu_get(
-						GTK_OPTION_MENU(widget)));
-			break;
-		case OPTION_COLOUR:
-			style = GTK_BIN(widget)->child->style;
+	g_return_if_fail(option != NULL);
+	g_return_if_fail(option->ui != NULL);
 
-			new = g_strdup_printf("#%04x%04x%04x",
-					style->bg[GTK_STATE_NORMAL].red,
-					style->bg[GTK_STATE_NORMAL].green,
-					style->bg[GTK_STATE_NORMAL].blue);
-			break;
-		case OPTION_TOOLS:
-			new = tools_to_list(widget);
-			break;
-		default:
-			g_printerr("[ unknown widget for change '%s' ]\n",
-					(guchar *) key);
-			break;
-	}
+	if (option->ui->read_widget)
+		new = option->ui->read_widget(option->ui);
+	else
+		return;
 
 	g_return_if_fail(new != NULL);
 
@@ -1085,7 +922,8 @@ static void save_cb(gpointer key, gpointer value, gpointer data)
 	len = strlen(tmp);
 
 	if (fwrite(tmp, sizeof(char), len, stream) < len)
-		delayed_error(_("Saving options"), g_strerror(errno));
+		delayed_error(_("Could not save options: %s"),
+				  g_strerror(errno));
 
 	g_free(tmp);
 }
@@ -1097,12 +935,7 @@ static void save_options(GtkWidget *widget, gpointer data)
 
 	g_hash_table_foreach(option_hash, may_change_cb, NULL);
 
-	for (next = notify_callbacks; next; next = next->next)
-	{
-		OptionNotify *cb = (OptionNotify *) next->data;
-
-		cb();
-	}
+	options_notify();
 
 	if (button == BUTTON_SAVE)
 	{
@@ -1112,9 +945,9 @@ static void save_options(GtkWidget *widget, gpointer data)
 		path = choices_find_path_save("Options", PROJECT, TRUE);
 		if (!path)
 		{
-			delayed_error(_("Saving options"),
-				      _("Choices saving is disabled by "
-					"CHOICESPATH variable"));
+		        delayed_error(_("Could not save options: %s"),
+				          _("Choices saving is disabled by "
+					  "CHOICESPATH variable"));
 			return;
 		}
 		
@@ -1124,7 +957,8 @@ static void save_options(GtkWidget *widget, gpointer data)
 		g_hash_table_foreach(option_hash, save_cb, file);
 
 		if (fclose(file) == EOF)
-			delayed_error(_("Saving options"), g_strerror(errno));
+			delayed_error(_("Could not save options: %s"),
+					  g_strerror(errno));
 
 		for (next = saver_callbacks; next; next = next->next)
 		{
@@ -1137,105 +971,16 @@ static void save_options(GtkWidget *widget, gpointer data)
 		gtk_widget_destroy(window);
 }
 
-/* Shade the listed tools; unshade the rest. */
-static void disable_tools(GtkWidget *hbox, guchar *list)
-{
-	GList	*next, *kids;
-
-	kids = gtk_container_children(GTK_CONTAINER(hbox));
-
-	for (next = kids; next; next = next->next)
-	{
-		GtkWidget	*kid = (GtkWidget *) next->data;
-		guchar		*name;
-
-		name = gtk_object_get_data(GTK_OBJECT(kid), "tool_name");
-
-		g_return_if_fail(name != NULL);
-		
-		gtk_widget_set_sensitive(GTK_BIN(kid)->child,
-					 !in_list(name, list));
-	}
-
-	g_list_free(kids);
-}
-
-/* Return a list of the shaded tools */
-static guchar *tools_to_list(GtkWidget *hbox)
-{
-	GList	*next, *kids;
-	GString	*list;
-	guchar	*retval;
-
-	list = g_string_new(NULL);
-
-	kids = gtk_container_children(GTK_CONTAINER(hbox));
-
-	for (next = kids; next; next = next->next)
-	{
-		GtkObject	*kid = (GtkObject *) next->data;
-		guchar		*name;
-
-		if (!GTK_WIDGET_SENSITIVE(GTK_BIN(kid)->child))
-		{
-			name = gtk_object_get_data(kid, "tool_name");
-			g_return_val_if_fail(name != NULL, list->str);
-
-			if (list->len)
-				g_string_append(list, ", ");
-			g_string_append(list, name);
-		}
-	}
-
-	g_list_free(kids);
-	retval = list->str;
-	g_string_free(list, FALSE);
-
-	return retval;
-}
-
 /* Make the widget reflect the current value of the option */
 static void update_cb(gpointer key, gpointer value, gpointer data)
 {
-	Option		*option = (Option *) value;
-	GtkWidget	*widget = option->widget;
-	GdkColor	colour;
+	Option *option = (Option *) value;
 
-	g_return_if_fail(option->widget != NULL);
+	g_return_if_fail(option != NULL);
+	g_return_if_fail(option->ui != NULL);
 
-	switch (option->widget_type)
-	{
-		case OPTION_TOGGLE:
-			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
-					atoi(option->value));
-			break;
-		case OPTION_ENTRY:
-			gtk_entry_set_text(GTK_ENTRY(widget), option->value);
-			break;
-		case OPTION_RADIO_GROUP:
-			radio_group_set_value(GTK_RADIO_BUTTON(widget),
-						option->value);
-			break;
-		case OPTION_SLIDER:
-			gtk_adjustment_set_value(
-				gtk_range_get_adjustment(GTK_RANGE(widget)),
-				atoi(option->value));
-			break;
-		case OPTION_MENU:
-			option_menu_set(GTK_OPTION_MENU(widget), option->value);
-			break;
-		case OPTION_COLOUR:
-			gdk_color_parse(option->value, &colour);
-			button_patch_set_colour(widget, &colour);
-			break;
-		case OPTION_TOOLS:
-			disable_tools(widget, option->value);
-			break;
-		default:
-			g_printerr("Unknown widget for update '%s'\n",
-					(guchar *) key);
-			break;
-	}
+	if (option->ui->update_widget)
+		option->ui->update_widget(option->ui, option->value);
 }
 
 /* Reflect the values in the Option structures by changing the widgets
@@ -1245,3 +990,299 @@ static void update_option_widgets(void)
 {
 	g_hash_table_foreach(option_hash, update_cb, NULL);
 }
+
+/* Each of the following update the widget to make it show the current
+ * value of the option.
+ */
+
+static void update_toggle(OptionUI *ui, guchar *value)
+{
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ui->widget),
+			atoi(value));
+}
+
+static void update_entry(OptionUI *ui, guchar *value)
+{
+	gtk_entry_set_text(GTK_ENTRY(ui->widget), value);
+}
+
+static void update_radio_group(OptionUI *ui, guchar *value)
+{
+	radio_group_set_value(GTK_RADIO_BUTTON(ui->widget), value);
+}
+
+static void update_slider(OptionUI *ui, guchar *value)
+{
+	gtk_adjustment_set_value(
+		gtk_range_get_adjustment(GTK_RANGE(ui->widget)), atoi(value));
+}
+
+static void update_menu(OptionUI *ui, guchar *value)
+{
+	option_menu_set(GTK_OPTION_MENU(ui->widget), value);
+}
+
+static void update_colour(OptionUI *ui, guchar *value)
+{
+	GdkColor colour;
+
+	gdk_color_parse(value, &colour);
+	button_patch_set_colour(ui->widget, &colour);
+}
+
+/* Each of these read_* calls get the new (string) value of an option
+ * from the widget.
+ */
+
+static guchar *read_toggle(OptionUI *ui)
+{
+	return g_strdup_printf("%d",
+		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ui->widget)));
+}
+
+static guchar *read_entry(OptionUI *ui)
+{
+	return gtk_editable_get_chars(GTK_EDITABLE(ui->widget), 0, -1);
+}
+
+static guchar *read_slider(OptionUI *ui)
+{
+	return g_strdup_printf("%f",
+			gtk_range_get_adjustment(GTK_RANGE(ui->widget))->value);
+}
+
+static guchar *read_radio_group(OptionUI *ui)
+{
+	return radio_group_get_value(GTK_RADIO_BUTTON(ui->widget));
+}
+
+static guchar *read_menu(OptionUI *ui)
+{
+	return g_strdup(option_menu_get(GTK_OPTION_MENU(ui->widget)));
+}
+
+static guchar *read_colour(OptionUI *ui)
+{
+	GtkStyle *style = GTK_BIN(ui->widget)->child->style;
+
+	return g_strdup_printf("#%04x%04x%04x",
+			style->bg[GTK_STATE_NORMAL].red,
+			style->bg[GTK_STATE_NORMAL].green,
+			style->bg[GTK_STATE_NORMAL].blue);
+}
+
+/* These create new widgets in the options window and set the appropriate
+ * callbacks.
+ */
+
+static GList *build_toggle(OptionUI *ui, xmlNode *node, guchar *label)
+{
+	GtkWidget	*toggle;
+
+	g_return_val_if_fail(ui != NULL, NULL);
+
+	toggle = gtk_check_button_new_with_label(_(label));
+
+	may_add_tip(toggle, node);
+
+	ui->update_widget = update_toggle;
+	ui->read_widget = read_toggle;
+	ui->widget = toggle;
+
+	return g_list_append(NULL, toggle);
+}
+
+static GList *build_slider(OptionUI *ui, xmlNode *node, guchar *label)
+{
+	GtkAdjustment *adj;
+	GtkWidget *hbox, *slide;
+	int	min, max;
+	int	fixed;
+	int	showvalue;
+
+	g_return_val_if_fail(ui != NULL, NULL);
+
+	min = get_int(node, "min");
+	max = get_int(node, "max");
+	fixed = get_int(node, "fixed");
+	showvalue = get_int(node, "showvalue");
+
+	adj = GTK_ADJUSTMENT(gtk_adjustment_new(0,
+				min, max, 1, 10, 0));
+
+	hbox = gtk_hbox_new(FALSE, 4);
+	gtk_box_pack_start(GTK_BOX(hbox),
+			gtk_label_new(_(label)),
+			FALSE, TRUE, 0);
+
+	slide = gtk_hscale_new(adj);
+
+	if (fixed)
+		gtk_widget_set_usize(slide, adj->upper, 24);
+	if (showvalue)
+	{
+		gtk_scale_set_draw_value(GTK_SCALE(slide), TRUE);
+		gtk_scale_set_value_pos(GTK_SCALE(slide),
+				GTK_POS_LEFT);
+		gtk_scale_set_digits(GTK_SCALE(slide), 0);
+	}
+	else 
+		gtk_scale_set_draw_value(GTK_SCALE(slide), FALSE);
+	GTK_WIDGET_UNSET_FLAGS(slide, GTK_CAN_FOCUS);
+
+	may_add_tip(slide, node);
+
+	gtk_box_pack_start(GTK_BOX(hbox), slide, !fixed, TRUE, 0);
+
+	ui->update_widget = update_slider;
+	ui->read_widget = read_slider;
+	ui->widget = slide;
+
+	return g_list_append(NULL, hbox);
+}
+
+static GList *build_entry(OptionUI *ui, xmlNode *node, guchar *label)
+{
+	GtkWidget	*hbox;
+	GtkWidget	*entry;
+
+	g_return_val_if_fail(ui != NULL, NULL);
+
+	hbox = gtk_hbox_new(FALSE, 4);
+
+	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_(label)),
+				FALSE, TRUE, 0);
+
+	entry = gtk_entry_new();
+	gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+	may_add_tip(entry, node);
+
+	ui->update_widget = update_entry;
+	ui->read_widget = read_entry;
+	ui->widget = entry;
+
+	return g_list_append(NULL, hbox);
+}
+
+static GList *build_radio_group(OptionUI *ui, xmlNode *node, guchar *label)
+{
+	GList		*list = NULL;
+	GtkWidget	*button = NULL;
+	xmlNode		*rn;
+
+	g_return_val_if_fail(ui != NULL, NULL);
+
+	for (rn = node->xmlChildrenNode; rn; rn = rn->next)
+	{
+		if (rn->type == XML_ELEMENT_NODE)
+		{
+			button = build_radio(rn, button);
+			list = g_list_append(list, button);
+		}
+	}
+
+	ui->update_widget = update_radio_group;
+	ui->read_widget = read_radio_group;
+	ui->widget = button;
+
+	return list;
+}
+
+static GList *build_colour(OptionUI *ui, xmlNode *node, guchar *label)
+{
+	GtkWidget	*hbox, *da, *button;
+	int		lpos;
+	
+	g_return_val_if_fail(ui != NULL, NULL);
+
+	/* lpos gives the position for the label 
+	 * 0: label comes before the button
+	 * non-zero: label comes after the button
+	 */
+	lpos = get_int(node, "lpos");
+
+	hbox = gtk_hbox_new(FALSE, 4);
+	if (lpos == 0)
+		gtk_box_pack_start(GTK_BOX(hbox),
+			gtk_label_new(_(label)),
+			FALSE, TRUE, 0);
+
+	button = gtk_button_new();
+	da = gtk_drawing_area_new();
+	gtk_drawing_area_size(GTK_DRAWING_AREA(da), 64, 12);
+	gtk_container_add(GTK_CONTAINER(button), da);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			GTK_SIGNAL_FUNC(open_coloursel), button);
+
+	may_add_tip(button, node);
+	
+	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, TRUE, 0);
+	if (lpos)
+		gtk_box_pack_start(GTK_BOX(hbox),
+			gtk_label_new(_(label)),
+			FALSE, TRUE, 0);
+
+	ui->update_widget = update_colour;
+	ui->read_widget = read_colour;
+	ui->widget = button;
+
+	return g_list_append(NULL, hbox);
+}
+
+static GList *build_menu(OptionUI *ui, xmlNode *node, guchar *label)
+{
+	GtkWidget	*hbox, *om, *option_menu;
+	xmlNode		*item;
+	GtkWidget	*menu;
+	GList		*list, *kids;
+	int		min_w = 4, min_h = 4;
+
+	g_return_val_if_fail(ui != NULL, NULL);
+
+	hbox = gtk_hbox_new(FALSE, 4);
+
+	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_(label)),
+			FALSE, TRUE, 0);
+
+	option_menu = gtk_option_menu_new();
+	gtk_box_pack_start(GTK_BOX(hbox), option_menu, FALSE, TRUE, 0);
+
+	om = gtk_menu_new();
+	gtk_option_menu_set_menu(GTK_OPTION_MENU(option_menu), om);
+
+	for (item = node->xmlChildrenNode; item; item = item->next)
+	{
+		if (item->type == XML_ELEMENT_NODE)
+			build_menu_item(item, option_menu);
+	}
+
+	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(option_menu));
+	list = kids = gtk_container_children(GTK_CONTAINER(menu));
+
+	while (kids)
+	{
+		GtkWidget	*item = (GtkWidget *) kids->data;
+		GtkRequisition	req;
+
+		gtk_widget_size_request(item, &req);
+		if (req.width > min_w)
+			min_w = req.width;
+		if (req.height > min_h)
+			min_h = req.height;
+		
+		kids = kids->next;
+	}
+
+	g_list_free(list);
+
+	gtk_widget_set_usize(option_menu,
+			min_w + 50,	/* Else node doesn't work! */
+			min_h + 4);
+
+	ui->update_widget = update_menu;
+	ui->read_widget = read_menu;
+	ui->widget = option_menu;
+
+	return g_list_append(NULL, hbox);
+}
+
