@@ -1,5 +1,5 @@
 /*
- * $Id: panel.c,v 1.49 2001/09/26 14:48:06 tal197 Exp $
+ * $Id: panel.c,v 1.61 2001/12/26 20:33:21 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2001, the ROX-Filer team.
@@ -40,7 +40,7 @@
 #include "main.h"
 #include "type.h"
 #include "gui_support.h"
-#include "dir.h"
+#include "diritem.h"
 #include "pixmaps.h"
 #include "display.h"
 #include "bind.h"
@@ -65,7 +65,8 @@ static gint icon_button_release(GtkWidget *widget,
 static gint icon_button_press(GtkWidget *widget,
 			      GdkEventButton *event,
 			      Icon *icon);
-static void reposition_panel(Panel *panel, gboolean force_resize);
+static void reposition_panel(GtkWidget *window,
+				GtkAllocation *alloc, Panel *panel);
 static gint expose_icon(GtkWidget *widget,
 			GdkEventExpose *event,
 			Icon *icon);
@@ -80,7 +81,6 @@ static gint panel_button_press(GtkWidget *widget,
 			      Panel *panel);
 static void panel_post_resize(GtkWidget *box,
 			GtkRequisition *req, Panel *panel);
-static void box_resized(GtkWidget *box, GtkAllocation *alloc, Panel *panel);
 static void drag_set_panel_dest(Icon *icon);
 static void add_uri_list(GtkWidget          *widget,
                          GdkDragContext     *context,
@@ -104,7 +104,6 @@ static void drag_leave(GtkWidget	*widget,
                        GdkDragContext	*context,
 		       guint32		time,
 		       Icon	*icon);
-static GList *get_widget_list(Panel *panel);
 static GtkWidget *make_insert_frame(Panel *panel);
 static gboolean enter_icon(GtkWidget *widget,
 			   GdkEventCrossing *event,
@@ -139,6 +138,8 @@ static gint slide_from_value = 0;
 #define SHOW_ICON 2
 static int panel_style = SHOW_APPS_SMALL;
 
+static int closing_panel = 0;	/* Don't panel_save; destroying! */
+
 /****************************************************************
  *			EXTERNAL INTERFACE			*
  ****************************************************************/
@@ -165,7 +166,9 @@ Panel *panel_new(guchar *name, PanelSide side)
 	{
 		if (name)
 			number_of_windows++;
+		closing_panel++;
 		gtk_widget_destroy(current_panel[side]->window);
+		closing_panel--;
 		if (name)
 			number_of_windows--;
 	}
@@ -176,8 +179,8 @@ Panel *panel_new(guchar *name, PanelSide side)
 	panel = g_new(Panel, 1);
 	panel->name = g_strdup(name);
 	panel->side = side;
-	panel->height = 0;
 	panel->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	gtk_window_set_policy(GTK_WINDOW(panel->window), FALSE, FALSE, TRUE);
 	gtk_window_set_wmclass(GTK_WINDOW(panel->window), "ROX-Panel", PROJECT);
 	gtk_widget_set_events(panel->window,
 			GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
@@ -202,11 +205,12 @@ Panel *panel_new(guchar *name, PanelSide side)
 		guchar	*leaf;
 
 		leaf = g_strconcat("pan_", name, NULL);
-		load_path = choices_find_path_load(leaf, "ROX-Filer");
+		load_path = choices_find_path_load(leaf, PROJECT);
 		g_free(leaf);
 	}
 
 	vp = gtk_viewport_new(NULL, NULL);
+	gtk_container_set_resize_mode(GTK_CONTAINER(vp), GTK_RESIZE_PARENT);
 	gtk_viewport_set_shadow_type(GTK_VIEWPORT(vp), GTK_SHADOW_OUT);
 	gtk_container_add(GTK_CONTAINER(panel->window), vp);
 
@@ -241,7 +245,7 @@ Panel *panel_new(guchar *name, PanelSide side)
 	gtk_box_pack_start(GTK_BOX(box), frame, TRUE, TRUE, 4);
 
 	gtk_widget_realize(panel->window);
-	make_panel_window(panel->window->window);
+	make_panel_window(panel->window);
 
 	gtk_widget_show_all(vp);
 	
@@ -270,12 +274,12 @@ Panel *panel_new(guchar *name, PanelSide side)
 	gtk_widget_queue_resize(box);
 	gtk_signal_connect_after(GTK_OBJECT(panel->window), "size-request",
 		GTK_SIGNAL_FUNC(panel_post_resize), (GtkObject *) panel);
-	gtk_signal_connect_after(GTK_OBJECT(box), "size-allocate",
-			GTK_SIGNAL_FUNC(box_resized), (GtkObject *) panel);
+	gtk_signal_connect_after(GTK_OBJECT(panel->window), "size-allocate",
+			GTK_SIGNAL_FUNC(reposition_panel), (GtkObject *) panel);
 
 	number_of_windows++;
 	gtk_widget_show(panel->window);
-	
+
 	return panel;
 }
 
@@ -286,7 +290,7 @@ gboolean panel_want_show_text(Icon *icon)
 	if (panel_style == SHOW_ICON)
 		return FALSE;
 
-	if (icon->item.flags & ITEM_FLAG_APPDIR)
+	if (icon->item->flags & ITEM_FLAG_APPDIR)
 		return FALSE;
 
 	return TRUE;
@@ -296,7 +300,7 @@ void panel_icon_renamed(Icon *icon)
 {
 	GtkLabel *label = GTK_LABEL(icon->label);
 
-	gtk_label_set_text(label, icon->item.leafname);
+	gtk_label_set_text(label, icon->item->leafname);
 }
 
 
@@ -311,7 +315,7 @@ static int panel_delete(GtkWidget *widget, GdkEvent *event, Panel *panel)
 		      _("You have tried to close a panel via the window "
 			"manager - I usually find that this is accidental... "
 			"really close?"),
-			2, _("Remove"), _("Cancel")) != 0;
+			2, _("Cancel"), _("Remove")) != 1;
 }
 
 static void panel_destroyed(GtkWidget *widget, Panel *panel)
@@ -322,9 +326,11 @@ static void panel_destroyed(GtkWidget *widget, Panel *panel)
 	if (panel->side == PANEL_TOP || panel->side == PANEL_BOTTOM)
 	{
 		if (current_panel[PANEL_RIGHT])
-			reposition_panel(current_panel[PANEL_RIGHT], FALSE);
+			gtk_widget_queue_resize(
+					current_panel[PANEL_RIGHT]->window);
 		if (current_panel[PANEL_LEFT])
-			reposition_panel(current_panel[PANEL_LEFT], FALSE);
+			gtk_widget_queue_resize(
+					current_panel[PANEL_LEFT]->window);
 	}
 
 	g_free(panel->name);
@@ -408,23 +414,23 @@ static void panel_add_item(Panel *panel,
 	icon->widget = widget;
 	gtk_widget_set_name(icon->widget, "panel-icon");
 	icon->selected = FALSE;
-	diritem_stat(icon->path, &icon->item, FALSE);
 
 	if (name)
-		icon->item.leafname = g_strdup(name);
+		icon->item = diritem_new(name);
 	else
 	{
 		guchar	*slash;
 
 		slash = strrchr(icon->path, '/');
-		icon->item.leafname = g_strdup(slash && slash[1] ? slash + 1
+		icon->item = diritem_new(slash && slash[1] ? slash + 1
 								 : path);
 	}
+	diritem_restat(icon->path, icon->item);
 	
 	gtk_signal_connect_object(GTK_OBJECT(widget), "destroy",
 			  GTK_SIGNAL_FUNC(icon_destroyed), (gpointer) icon);
 
-	if (icon->item.base_type == TYPE_DIRECTORY)
+	if (icon->item->base_type == TYPE_DIRECTORY)
 		run_applet(icon);
 
 	gtk_signal_connect(GTK_OBJECT(widget), "button_release_event",
@@ -453,7 +459,7 @@ static void panel_add_item(Panel *panel,
 
 		drag_set_panel_dest(icon);
 
-		icon->label = gtk_label_new(icon->item.leafname);
+		icon->label = gtk_label_new(icon->item->leafname);
 		gtk_container_add(GTK_CONTAINER(icon->widget), icon->label);
 		gtk_misc_set_alignment(GTK_MISC(icon->label), 0.5, 1);
 		gtk_misc_set_padding(GTK_MISC(icon->label), 1, 2);
@@ -473,8 +479,8 @@ static void size_request(GtkWidget *widget, GtkRequisition *req, Icon *icon)
 {
 	int	im_width, im_height;
 
-	im_width = icon->item.image->width;
-	im_height = MIN(icon->item.image->height, ICON_HEIGHT);
+	im_width = icon->item->image->width;
+	im_height = MIN(icon->item->image->height, ICON_HEIGHT);
 
 	req->height += im_height;
 	req->width = MAX(req->width, im_width);
@@ -496,7 +502,7 @@ static gint draw_icon(GtkWidget *widget, GdkRectangle *badarea, Icon *icon)
 
 	area.x = 0;
 	area.width = width;
-	area.height = icon->item.image->height;
+	area.height = icon->item->image->height;
 
 	if (panel_want_show_text(icon))
 	{
@@ -504,12 +510,14 @@ static gint draw_icon(GtkWidget *widget, GdkRectangle *badarea, Icon *icon)
 
 		area.y = height - text_height - area.height;
 		
-		draw_large_icon(widget, &area, &icon->item, icon->selected);
+		draw_large_icon(widget, &area, icon->item,
+				icon->item->image, icon->selected);
 	}
 	else
 	{
 		area.y = (height - area.height) >> 1;
-		draw_large_icon(widget, &area, &icon->item, icon->selected);
+		draw_large_icon(widget, &area, icon->item,
+				icon->item->image, icon->selected);
 	}
 
 	return FALSE;
@@ -531,12 +539,12 @@ static void perform_action(Panel *panel, Icon *icon, GdkEventButton *event)
 		case ACT_OPEN_ITEM:
 			dnd_motion_ungrab();
 			wink_widget(icon->widget);
-			run_diritem(icon->path, &icon->item, NULL, NULL, FALSE);
+			run_diritem(icon->path, icon->item, NULL, NULL, FALSE);
 			break;
 		case ACT_EDIT_ITEM:
 			dnd_motion_ungrab();
 			wink_widget(icon->widget);
-			run_diritem(icon->path, &icon->item, NULL, NULL, TRUE);
+			run_diritem(icon->path, icon->item, NULL, NULL, TRUE);
 			break;
 		case ACT_POPUP_MENU:
 			dnd_motion_ungrab();
@@ -632,72 +640,38 @@ static gint icon_button_press(GtkWidget *widget,
 	return TRUE;
 }
 
-/* Difference between height of an icon and height of panel (or width) */
-#define MARGIN 8
-
-static void reposition_panel(Panel *panel, gboolean force_resize)
+static void reposition_panel(GtkWidget *window,
+				GtkAllocation *alloc, Panel *panel)
 {
-	int	x = 0, y = 0;
-	int	w = 32, h = 32;
-	GList	*next, *children;
+	int		x = 0, y = 0;
 	PanelSide	side = panel->side;
 
-	children = get_widget_list(panel);
-
-	for (next = children; next; next = next->next)
+	if (side == PANEL_LEFT || side == PANEL_RIGHT)
 	{
-		GtkWidget	*widget = (GtkWidget *) next->data;
-		GtkRequisition	req;
-
-		gtk_widget_get_child_requisition(widget, &req);
-
-		if (req.width > w)
-			w = req.width;
-		if (req.height > h)
-			h = req.height;
-	}
-
-	g_list_free(children);
-
-	if (side == PANEL_TOP || side == PANEL_BOTTOM)
-	{
-		w = screen_width;
-		h += MARGIN;
-	}
-	else
-	{
-		w += MARGIN;
 		if (side == PANEL_RIGHT)
-			x = screen_width - w;
-
-		h = screen_height;
+			x = screen_width - alloc->width;
 
 		if (current_panel[PANEL_TOP])
 		{
-			int	ph;
-
-			ph = current_panel[PANEL_TOP]->height;
-			y += ph;
-			h -= ph;
+			GtkWidget *win = current_panel[PANEL_TOP]->window;
+			y += win->allocation.height;
 		}
-
-		if (current_panel[PANEL_BOTTOM])
-			h -= current_panel[PANEL_BOTTOM]->height;
 	}
 
 	if (side == PANEL_BOTTOM)
-		y = screen_height - h;
+		y = screen_height - alloc->height;
 	
 	gtk_widget_set_uposition(panel->window, x, y);
-	panel->height = h;
-	gdk_window_move_resize(panel->window->window, x, y, w, h);
+	gdk_window_move(panel->window->window, x, y);
 
 	if (side == PANEL_BOTTOM || side == PANEL_TOP)
 	{
 		if (current_panel[PANEL_RIGHT])
-			reposition_panel(current_panel[PANEL_RIGHT], FALSE);
+			gtk_widget_queue_resize(
+					current_panel[PANEL_RIGHT]->window);
 		if (current_panel[PANEL_LEFT])
-			reposition_panel(current_panel[PANEL_LEFT], FALSE);
+			gtk_widget_queue_resize(
+					current_panel[PANEL_LEFT]->window);
 	}
 }
 
@@ -725,7 +699,7 @@ static gboolean drag_motion(GtkWidget		*widget,
 {
 	GdkDragAction	action = context->suggested_action;
 	char		*type = NULL;
-	DirItem		*item = &icon->item;
+	DirItem		*item = icon->item;
 
 	if (icon->selected)
 		goto out;	/* Can't drag a selection to itself */
@@ -858,7 +832,7 @@ static gboolean write_widgets(FILE *file, GList *widgets, guchar side)
 		}
 
 		g_string_sprintf(tmp, "%s%c%s\n",
-				icon->item.leafname,
+				icon->item->leafname,
 				side, icon->src_path);
 		if (fwrite(tmp->str, 1, tmp->len, file) < tmp->len)
 		{
@@ -890,7 +864,7 @@ void panel_save(Panel *panel)
 		guchar	*leaf;
 
 		leaf = g_strconcat("pan_", panel->name, NULL);
-		save = choices_find_path_save(leaf, "ROX-Filer", TRUE);
+		save = choices_find_path_save(leaf, PROJECT, TRUE);
 		g_free(leaf);
 	}
 
@@ -926,7 +900,7 @@ void panel_save(Panel *panel)
 
 	goto out;
 err:
-	delayed_error(_("Error saving panel"), g_strerror(errno));
+	delayed_error(_("Could not save panel: %s"), g_strerror(errno));
 out:
 	if (file)
 		fclose(file);
@@ -934,17 +908,6 @@ out:
 	g_free(save_new);
 }
 
-static GList *get_widget_list(Panel *panel)
-{
-	GList	*list;
-
-	list = gtk_container_children(GTK_CONTAINER(panel->before));
-	list = g_list_concat(list,
-			gtk_container_children(GTK_CONTAINER(panel->after)));
-
-	return list;
-}
-
 /* Create a frame widget which can be used to add icons to the panel */
 static GtkWidget *make_insert_frame(Panel *panel)
 {
@@ -1155,7 +1118,7 @@ static void start_drag(Icon *icon, GdkEventMotion *event)
 	g_return_if_fail(icon_selection != NULL);
 
 	if (icon_selection->next == NULL)
-		drag_one_item(widget, event, icon->path, &icon->item);
+		drag_one_item(widget, event, icon->path, icon->item, NULL);
 	else
 	{
 		guchar	*uri_list;
@@ -1194,47 +1157,53 @@ static guchar *create_uri_list(GList *list)
 
 static void applet_died(GtkWidget *socket)
 {
-	if (!GTK_OBJECT_DESTROYED(socket))
+	gboolean never_plugged;
+
+	never_plugged = (!gtk_object_get_data(GTK_OBJECT(socket), "lost_plug"))
+		      && !GTK_SOCKET(socket)->plug_window;
+
+	if (never_plugged)
+	{
+		report_error(
+			_("Applet quit without ever creating a widget!"));
 		gtk_widget_destroy(socket);
+	}
 
 	gtk_widget_unref(socket);
 }
 
-static void restart_applet(GtkWidget *button, Icon *icon)
-{
-	gtk_widget_destroy(button);
-	run_applet(icon);
-}
-
 static void socket_destroyed(GtkWidget *socket, GtkWidget *widget)
 {
-	Icon	*icon;
-	gboolean lost_widget;
+	gtk_object_set_data(GTK_OBJECT(socket), "lost_plug", "yes");
 
-	lost_widget = GTK_OBJECT_DESTROYED(widget);
-	gtk_widget_unref(widget);
+	gtk_widget_unref(socket);
 
-	if (lost_widget)
-		return;		/* We're removing the icon... */
-		
-	icon = gtk_object_get_data(GTK_OBJECT(widget), "icon");
-	icon->socket = gtk_button_new_with_label(_("Restart\nApplet"));
-	
-	gtk_signal_connect(GTK_OBJECT(icon->socket), "button_release_event",
-			GTK_SIGNAL_FUNC(icon_button_release), icon);
-	gtk_signal_connect(GTK_OBJECT(icon->socket), "button_press_event",
-			GTK_SIGNAL_FUNC(icon_button_press), icon);
+	gtk_widget_destroy(widget);	/* Remove from panel */
 
-	gtk_container_add(GTK_CONTAINER(icon->widget), icon->socket);
-	gtk_container_set_border_width(GTK_CONTAINER(icon->socket), 4);
-	gtk_misc_set_padding(GTK_MISC(GTK_BIN(icon->socket)->child), 4, 4);
-	gtk_widget_show(icon->socket);
-
-	gtk_signal_connect(GTK_OBJECT(icon->socket), "clicked",
-			GTK_SIGNAL_FUNC(restart_applet), icon);
+	if (!closing_panel)
+		panel_save(gtk_object_get_data(GTK_OBJECT(socket), "panel"));
 }
 
-/* Try to run this applet. Fills in icon->socket on success. */
+/* Try to run this applet.
+ * Cases:
+ * 
+ * - No executable AppletRun:
+ * 	icon->socket == NULL (unchanged) on return.
+ *
+ * Otherwise, create socket (setting icon->socket) and ref it twice.
+ * 
+ * - AppletRun quits without connecting a plug:
+ * 	On child death lost_plug is unset and socket is empty.
+ * 	Unref socket.
+ * 	Report error and destroy widget (to 'socket destroyed').
+ *
+ * - AppletRun quits while plug is in socket:
+ * 	Unref socket once. Socket will be destroyed later.
+ *
+ * - Socket is destroyed.
+ * 	Set lost_plug = "yes" and remove widget from panel.
+ * 	Unref socket.
+ */
 static void run_applet(Icon *icon)
 {
 	char	*argv[3];
@@ -1246,12 +1215,17 @@ static void run_applet(Icon *icon)
 		return;
 
 	icon->socket = gtk_socket_new();
+	/* Two refs held: one for child death, one for socket destroyed */
+	gtk_widget_ref(icon->socket);
+	gtk_widget_ref(icon->socket);
+	
 	gtk_container_add(GTK_CONTAINER(icon->widget), icon->socket);
 	gtk_widget_show_all(icon->socket);
 	gtk_widget_realize(icon->socket);
 
-	gtk_widget_ref(icon->widget);
 	gtk_object_set_data(GTK_OBJECT(icon->widget), "icon", icon);
+	gtk_object_set_data(GTK_OBJECT(icon->socket), "panel", icon->panel);
+
 	gtk_signal_connect(GTK_OBJECT(icon->socket), "destroy",
 			GTK_SIGNAL_FUNC(socket_destroyed), icon->widget);
 	
@@ -1259,41 +1233,34 @@ static void run_applet(Icon *icon)
 			GDK_WINDOW_XWINDOW(icon->socket->window));
 	argv[2] = NULL;
 
-	pid = spawn(argv);
-	gtk_widget_ref(icon->socket);
+	pid = spawn_full(argv, NULL, NULL);
+	
 	on_child_death(pid, (CallbackFn) applet_died, icon->socket);
 	
 	g_free(argv[1]);
 }
 
-/* When one of the panel icons resizes it will cause it's container box
- * to resize. This will cause the packing box inside the viewport to resize -
- * we get here right after that.
- */
-static void box_resized(GtkWidget *box, GtkAllocation *alloc, Panel *panel)
-{
-	reposition_panel(panel, FALSE);
-}
-
 static void panel_post_resize(GtkWidget *win, GtkRequisition *req, Panel *panel)
 {
 	if (panel->side == PANEL_TOP || panel->side == PANEL_BOTTOM)
-	{
-		if (req->width < screen_width)
-			req->width = screen_width;
-	}
+		req->width = screen_width;
 	else
 	{
 		int h = screen_height;
 
 		if (current_panel[PANEL_TOP])
-			h -= current_panel[PANEL_TOP]->height;
+		{
+			GtkWidget *win = current_panel[PANEL_TOP]->window;
+			h -= win->allocation.height;
+		}
 
 		if (current_panel[PANEL_BOTTOM])
-			h -= current_panel[PANEL_BOTTOM]->height;
+		{
+			GtkWidget *win = current_panel[PANEL_BOTTOM]->window;
+			h -= win->allocation.height;
+		}
 
-		if (req->height < h)
-			req->height = h;
+		req->height = h;
 	}
 }
 
