@@ -1,5 +1,5 @@
 /*
- * $Id: pinboard.c,v 1.62 2001/09/26 14:48:06 tal197 Exp $
+ * $Id: pinboard.c,v 1.75 2001/12/26 20:33:21 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2001, the ROX-Filer team.
@@ -42,7 +42,7 @@
 #include "support.h"
 #include "gui_support.h"
 #include "options.h"
-#include "dir.h"
+#include "diritem.h"
 #include "bind.h"
 #include "icon.h"
 #include "run.h"
@@ -85,7 +85,8 @@ static GtkWidget *proxy_invisible;
 static GdkWindow *click_proxy_gdk_window = NULL;
 static GdkAtom   win_button_proxy; /* _WIN_DESKTOP_BUTTON_PROXY */
 
-static gboolean pinboard_drag_in_progress = FALSE;
+/* The Icon that was used to start the current drag, if any */
+Icon *pinboard_drag_in_progress = NULL;
 
 /* Used when dragging icons around... */
 static gboolean pinboard_modified = FALSE;
@@ -155,6 +156,11 @@ static gboolean bg_drag_motion(GtkWidget	*widget,
                                gint		y,
                                guint		time,
 			       gpointer		data);
+static gboolean bg_drag_leave(GtkWidget		*widget,
+			      GdkDragContext	*context,
+			      guint32		time,
+			      gpointer		data);
+static void bg_expose(GdkRectangle *area);
 static void drag_end(GtkWidget *widget,
 			GdkDragContext *context,
 			Icon *icon);
@@ -194,7 +200,7 @@ void pinboard_activate(guchar *name)
 	guchar		*path, *slash;
 
 	/* Treat an empty name the same as NULL */
-	if (!*name)
+	if (name && !*name)
 		name = NULL;
 
 	if (old_board)
@@ -212,7 +218,7 @@ void pinboard_activate(guchar *name)
 
 	if (!add_root_handlers())
 	{
-		delayed_error(PROJECT, _("Another application is already "
+		delayed_error(_("Another application is already "
 					"managing the pinboard!"));
 		return;
 	}
@@ -232,7 +238,7 @@ void pinboard_activate(guchar *name)
 		guchar	*leaf;
 
 		leaf = g_strconcat("pb_", name, NULL);
-		path = choices_find_path_load(leaf, "ROX-Filer");
+		path = choices_find_path_load(leaf, PROJECT);
 		g_free(leaf);
 	}
 
@@ -247,7 +253,9 @@ void pinboard_activate(guchar *name)
 		g_free(path);
 	}
 	else
-		pinboard_pin(home_dir, "Home", 4, 4, TRUE);
+		pinboard_pin(home_dir, "Home",
+				4 + ICON_WIDTH / 2,
+				4 + ICON_HEIGHT / 2);
 	loading_pinboard--;
 }
 
@@ -258,7 +266,7 @@ void pinboard_activate(guchar *name)
  * image should be if it is TRUE.
  * 'name' is the name to use. If NULL then the leafname of path is used.
  */
-void pinboard_pin(guchar *path, guchar *name, int x, int y, gboolean corner)
+void pinboard_pin(guchar *path, guchar *name, int x, int y)
 {
 	Icon		*icon;
 	int		width, height;
@@ -278,8 +286,6 @@ void pinboard_pin(guchar *path, guchar *name, int x, int y, gboolean corner)
 
 	icon_hash_path(icon);
 
-	diritem_stat(icon->path, &icon->item, FALSE);
-
 	if (!name)
 	{
 		name = strrchr(icon->path, '/');
@@ -289,7 +295,8 @@ void pinboard_pin(guchar *path, guchar *name, int x, int y, gboolean corner)
 			name = icon->path;
 	}
 
-	icon->item.leafname = g_strdup(name);
+	icon->item = diritem_new(name);
+	diritem_restat(icon->path, icon->item);
 
 	icon->win = gtk_window_new(GTK_WINDOW_DIALOG);
 	gtk_window_set_wmclass(GTK_WINDOW(icon->win), "ROX-Pinboard", PROJECT);
@@ -309,13 +316,6 @@ void pinboard_pin(guchar *path, guchar *name, int x, int y, gboolean corner)
 	gtk_widget_realize(icon->widget);
 
 	set_size_and_shape(icon, &width, &height);
-	if (corner)
-	{
-		/* Convert from icon-corner coordinates to center coordinates */
-		MaskedPixmap	*image = icon->item.image;
-
-		offset_to_centre(icon, image->width >> 1, height, &x, &y);
-	}
 	snap_to_grid(&x, &y);
 	offset_from_centre(icon, width, height, &x, &y);
 	gtk_widget_set_uposition(icon->win, x, y);
@@ -324,7 +324,19 @@ void pinboard_pin(guchar *path, guchar *name, int x, int y, gboolean corner)
 	icon->x = x;
 	icon->y = y;
 	
-	make_panel_window(icon->win->window);
+	make_panel_window(icon->win);
+
+	/* TODO: Use gdk function when it supports this type */
+	{
+		GdkAtom desktop_type;
+
+		desktop_type = gdk_atom_intern("_NET_WM_WINDOW_TYPE_DESKTOP",
+						FALSE);
+		gdk_property_change(icon->win->window,
+			gdk_atom_intern("_NET_WM_WINDOW_TYPE", FALSE),
+			gdk_atom_intern("ATOM", FALSE), 32,
+			GDK_PROP_MODE_REPLACE, (guchar *) &desktop_type, 1);
+	}
 
 	gtk_widget_add_events(icon->widget,
 			GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
@@ -504,10 +516,10 @@ static void set_size_and_shape(Icon *icon, int *rwidth, int *rheight)
 {
 	int		width, height;
 	int		font_height;
-	MaskedPixmap	*image = icon->item.image;
+	MaskedPixmap	*image = icon->item->image;
 	int		iwidth = image->width;
 	int		iheight = image->height;
-	DirItem		*item = &icon->item;
+	DirItem		*item = icon->item;
 	int		text_x, text_y;
 #ifndef GTK2
 	GdkFont		*font;
@@ -526,19 +538,19 @@ static void set_size_and_shape(Icon *icon, int *rwidth, int *rheight)
 #ifndef GTK2
 	font = pinicon_style->font;
 	font_height = font->ascent + font->descent;
-	item->name_width = gdk_string_measure(font, item->leafname);
+	icon->name_width = gdk_string_measure(font, item->leafname);
 #else
 	{
 		PangoRectangle logical;
-		pango_layout_set_text(icon->layout, icon->item.leafname, -1);
+		pango_layout_set_text(icon->layout, icon->item->leafname, -1);
 		pango_layout_get_pixel_extents(icon->layout, NULL, &logical);
 
-		item->name_width = logical.width - logical.x;
+		icon->name_width = logical.width - logical.x;
 		font_height = logical.height - logical.y;
 	}
 #endif
 
-	width = MAX(iwidth, item->name_width + 2) + 2 * WINK_FRAME;
+	width = MAX(iwidth, icon->name_width + 2) + 2 * WINK_FRAME;
 	height = iheight + GAP + (font_height + 2) + 2 * WINK_FRAME;
 	gtk_widget_set_usize(icon->win, width, height);
 	icon->width = width;
@@ -596,7 +608,7 @@ static void set_size_and_shape(Icon *icon, int *rwidth, int *rheight)
 
 	/* Mask off an area for the text (from o_text_bg) */
 
-	text_x = (width - item->name_width) >> 1;
+	text_x = (width - icon->name_width) >> 1;
 	text_y = WINK_FRAME + iheight + GAP + 1;
 
 #ifndef GTK2
@@ -604,9 +616,9 @@ static void set_size_and_shape(Icon *icon, int *rwidth, int *rheight)
 	{
 #endif
 		gdk_draw_rectangle(icon->mask, mask_gc, TRUE,
-				(width - (item->name_width + 2)) >> 1,
+				(width - (icon->name_width + 2)) >> 1,
 				WINK_FRAME + iheight + GAP,
-				item->name_width + 2, font_height + 2);
+				icon->name_width + 2, font_height + 2);
 #ifndef GTK2
 	}
 	else
@@ -641,7 +653,7 @@ static gint draw_icon(GtkWidget *widget, GdkEventExpose *event, Icon *icon)
 	GdkFont		*font = icon->widget->style->font;
 #endif
 	int		text_x, text_y;
-	DirItem		*item = &icon->item;
+	DirItem		*item = icon->item;
 	MaskedPixmap	*image = item->image;
 	int		iwidth = image->width;
 	int		iheight = image->height;
@@ -692,7 +704,7 @@ static gint draw_icon(GtkWidget *widget, GdkEventExpose *event, Icon *icon)
 		gdk_gc_set_clip_origin(gc, 0, 0);
 	}
 
-	text_x = (icon->width - item->name_width) >> 1;
+	text_x = (icon->width - icon->name_width) >> 1;
 	text_y = WINK_FRAME + iheight + GAP + 1;
 
 	if (o_text_bg != TEXT_BG_NONE)
@@ -713,7 +725,7 @@ static gint draw_icon(GtkWidget *widget, GdkEventExpose *event, Icon *icon)
 				NULL, widget, "text",
 				text_x - 1,
 				text_y - 1,
-				item->name_width + 2,
+				icon->name_width + 2,
 				font_height + 2);
 	}
 
@@ -809,12 +821,16 @@ static void perform_action(Icon *icon, GdkEventButton *event)
 		case ACT_OPEN_ITEM:
 			dnd_motion_ungrab();
 			pinboard_wink_item(icon, TRUE);
-			run_diritem(icon->path, &icon->item, NULL, NULL, FALSE);
+			if (event->type == GDK_2BUTTON_PRESS)
+				icon_set_selected(icon, FALSE);
+			run_diritem(icon->path, icon->item, NULL, NULL, FALSE);
 			break;
 		case ACT_EDIT_ITEM:
 			dnd_motion_ungrab();
 			pinboard_wink_item(icon, TRUE);
-			run_diritem(icon->path, &icon->item, NULL, NULL, TRUE);
+			if (event->type == GDK_2BUTTON_PRESS)
+				icon_set_selected(icon, FALSE);
+			run_diritem(icon->path, icon->item, NULL, NULL, TRUE);
 			break;
 		case ACT_POPUP_MENU:
 			dnd_motion_ungrab();
@@ -918,10 +934,10 @@ static void start_drag(Icon *icon, GdkEventMotion *event)
 	
 	g_return_if_fail(icon_selection != NULL);
 
-	pinboard_drag_in_progress = TRUE;
+	pinboard_drag_in_progress = icon;
 
 	if (icon_selection->next == NULL)
-		drag_one_item(widget, event, icon->path, &icon->item);
+		drag_one_item(widget, event, icon->path, icon->item, NULL);
 	else
 	{
 		guchar	*uri_list;
@@ -938,8 +954,8 @@ static gint icon_motion_notify(GtkWidget *widget,
 			       Icon *icon)
 {
 	int	x, y;
-	int	width, height;
 	int	dx,dy;
+	int	width, height;
 
 	if (motion_state == MOTION_READY_FOR_DND)
 	{
@@ -1007,7 +1023,7 @@ static char *pin_from_file(guchar *line)
 	if (sscanf(line, " %d , %d , %n", &x, &y, &n) < 2)
 		return NULL;		/* Ignore format errors */
 
-	pinboard_pin(line + n, leaf, x, y, FALSE);
+	pinboard_pin(line + n, leaf, x, y);
 
 	g_free(leaf);
 
@@ -1044,6 +1060,9 @@ static gboolean add_root_handlers(void)
 		gtk_signal_connect(GTK_OBJECT(proxy_invisible), "drag_motion",
 				GTK_SIGNAL_FUNC(bg_drag_motion),
 				NULL);
+		gtk_signal_connect(GTK_OBJECT(proxy_invisible), "drag_leave",
+				GTK_SIGNAL_FUNC(bg_drag_leave),
+				NULL);
 	}
 
 	root = gdk_window_lookup(GDK_ROOT_WINDOW());
@@ -1058,6 +1077,7 @@ static gboolean add_root_handlers(void)
 	gdk_window_set_user_data(GDK_ROOT_PARENT(), proxy_invisible);
 	gdk_window_set_events(GDK_ROOT_PARENT(),
 			gdk_window_get_events(GDK_ROOT_PARENT()) |
+				GDK_EXPOSURE_MASK |
 				GDK_PROPERTY_CHANGE_MASK);
 
 	forward_root_clicks();
@@ -1108,7 +1128,7 @@ void pinboard_save(void)
 		guchar	*leaf;
 
 		leaf = g_strconcat("pb_", current_pinboard->name, NULL);
-		save = choices_find_path_save(leaf, "ROX-Filer", TRUE);
+		save = choices_find_path_save(leaf, PROJECT, TRUE);
 		g_free(leaf);
 	}
 
@@ -1126,7 +1146,7 @@ void pinboard_save(void)
 		Icon *icon = (Icon *) next->data;
 
 		g_string_sprintf(tmp, "<%s>, %d, %d, %s\n",
-			icon->item.leafname, icon->x, icon->y, icon->src_path);
+			icon->item->leafname, icon->x, icon->y, icon->src_path);
 		if (fwrite(tmp->str, 1, tmp->len, file) < tmp->len)
 			goto err;
 	}
@@ -1144,7 +1164,7 @@ void pinboard_save(void)
 
 	goto out;
 err:
-	delayed_error(_("Error saving pinboard"), g_strerror(errno));
+	delayed_error(_("Could not save pinboard: %s"), g_strerror(errno));
 out:
 	if (file)
 		fclose(file);
@@ -1167,6 +1187,7 @@ static GdkFilterReturn proxy_filter(GdkXEvent *xevent,
 {
 	XEvent 		*xev;
 	GdkWindow	*proxy = proxy_invisible->window;
+	GdkRectangle	area;
 
 	xev = xevent;
 
@@ -1199,6 +1220,14 @@ static GdkFilterReturn proxy_filter(GdkXEvent *xevent,
 
 			return GDK_FILTER_TRANSLATE;
 
+		case Expose:
+			area.x = xev->xexpose.x;
+			area.y = xev->xexpose.y;
+			area.width = xev->xexpose.width;
+			area.height = xev->xexpose.height;
+			bg_expose(&area);
+			return GDK_FILTER_REMOVE;
+
 		case DestroyNotify:
 			/* XXX: I have no idea why this helps, but it does! */
 			/* The proxy window was destroyed (i.e. the window
@@ -1228,10 +1257,7 @@ static void offset_from_centre(Icon *icon,
 			       int *x, int *y)
 {
 	*x -= width >> 1;
-	*y -= height;
-#ifndef GTK2
-	*y += (icon->widget->style->font->descent >> 1);
-#endif
+	*y -= height >> 1;
 	*x = CLAMP(*x, 0, screen_width - (o_clamp_icons ? width : 0));
 	*y = CLAMP(*y, 0, screen_height - (o_clamp_icons ? height : 0));
 }
@@ -1242,11 +1268,7 @@ static void offset_to_centre(Icon *icon,
 			     int *x, int *y)
 {
   *x += width >> 1;
-  *y += height;
-  
-#ifndef GTK2
-  *y -= (icon->widget->style->font->descent >> 1);
-#endif
+  *y += height >> 1;
 }
 
 /* Same as drag_set_dest(), but for pinboard icons */
@@ -1276,7 +1298,7 @@ static gboolean drag_motion(GtkWidget		*widget,
 {
 	GdkDragAction	action = context->suggested_action;
 	char		*type = NULL;
-	DirItem		*item = &icon->item;
+	DirItem		*item = icon->item;
 
 	if (gtk_drag_get_source_widget(context) == widget)
 		goto out;	/* Can't drag something to itself! */
@@ -1317,6 +1339,101 @@ out:
 	return type != NULL;
 }
 
+static gboolean pinboard_shadow = FALSE;
+static gint shadow_x, shadow_y;
+#define SHADOW_SIZE (ICON_WIDTH)
+
+static void bg_expose(GdkRectangle *area)
+{
+	GdkWindow *root = GDK_ROOT_PARENT();
+	static GdkGC *shadow_gc = NULL;
+	static GdkColor white, black;
+
+	if (!pinboard_shadow)
+	{
+		/* XXX: Should just disable the events */
+		return;
+	}
+	
+	if (!shadow_gc)
+	{
+		GdkColormap *cm;
+		
+		cm = gdk_window_get_colormap(root);
+		shadow_gc = gdk_gc_new(root);
+		gdk_color_white(cm, &white);
+		gdk_color_black(cm, &black);
+	}
+
+	gdk_gc_set_clip_rectangle(shadow_gc, area);
+	gdk_gc_set_foreground(shadow_gc, &white);
+	gdk_draw_rectangle(root, shadow_gc, FALSE, shadow_x, shadow_y,
+			SHADOW_SIZE, SHADOW_SIZE);
+	gdk_gc_set_foreground(shadow_gc, &black);
+	gdk_draw_rectangle(root, shadow_gc, FALSE, shadow_x + 1, shadow_y + 1,
+			SHADOW_SIZE - 2, SHADOW_SIZE - 2);
+	gdk_gc_set_clip_rectangle(shadow_gc, NULL);
+}
+
+/* Draw a 'shadow' under an icon being dragged, showing where
+ * it will land.
+ */
+static void pinboard_set_shadow(gboolean on)
+{
+	GdkWindow *root = GDK_ROOT_PARENT();
+
+	if (pinboard_shadow)
+	{
+		gdk_window_clear_area_e(root, shadow_x, shadow_y,
+					SHADOW_SIZE + 1, SHADOW_SIZE + 1);
+	}
+
+	if (on)
+	{
+		int	old_x = shadow_x, old_y = shadow_y;
+
+		gdk_window_get_pointer(root, &shadow_x, &shadow_y, NULL);
+		snap_to_grid(&shadow_x, &shadow_y);
+		shadow_x -= SHADOW_SIZE / 2;
+		shadow_y -= SHADOW_SIZE / 2;
+
+
+		if (pinboard_shadow && shadow_x == old_x && shadow_y == old_y)
+			return;
+
+		gdk_window_clear_area_e(root, shadow_x, shadow_y,
+					SHADOW_SIZE + 1, SHADOW_SIZE + 1);
+	}
+
+	pinboard_shadow = on;
+}
+
+/* Called when dragging some pinboard icons finishes */
+void pinboard_move_icons(void)
+{
+	int	x = shadow_x, y = shadow_y;
+	Icon	*icon = pinboard_drag_in_progress;
+	int	width, height;
+
+	g_return_if_fail(icon != NULL);
+
+	x += SHADOW_SIZE / 2;
+	y += SHADOW_SIZE / 2;
+	snap_to_grid(&x, &y);
+
+	if (icon->x == x && icon->y == y)
+		return;
+
+	icon->x = x;
+	icon->y = y;
+	gdk_window_get_size(icon->win->window, &width, &height);
+	offset_from_centre(icon, width, height, &x, &y);
+
+	gdk_window_move(icon->win->window, x, y);
+
+	pinboard_save();
+}
+
 static void drag_leave(GtkWidget	*widget,
                        GdkDragContext	*context,
 		       guint32		time,
@@ -1326,6 +1443,16 @@ static void drag_leave(GtkWidget	*widget,
 	dnd_spring_abort();
 }
 
+static gboolean bg_drag_leave(GtkWidget		*widget,
+			      GdkDragContext	*context,
+			      guint32		time,
+			      gpointer		data)
+{
+	pinboard_set_shadow(FALSE);
+	return TRUE;
+}
+
+
 static gboolean bg_drag_motion(GtkWidget	*widget,
                                GdkDragContext	*context,
                                gint		x,
@@ -1334,8 +1461,11 @@ static gboolean bg_drag_motion(GtkWidget	*widget,
 			       gpointer		data)
 {
 	/* Dragging from the pinboard to the pinboard is not allowed */
+#if 0
 	if (pinboard_drag_in_progress)
 		return FALSE;
+#endif
+	pinboard_set_shadow(TRUE);
 	
 	gdk_drag_status(context, context->suggested_action, time);
 	return TRUE;
@@ -1345,7 +1475,7 @@ static void drag_end(GtkWidget *widget,
 		     GdkDragContext *context,
 		     Icon *icon)
 {
-	pinboard_drag_in_progress = FALSE;
+	pinboard_drag_in_progress = NULL;
 	if (tmp_icon_selected)
 	{
 		icon_select_only(NULL);
