@@ -1,5 +1,5 @@
 /*
- * $Id: pixmaps.c,v 1.45 2001/10/05 11:19:12 tal197 Exp $
+ * $Id: pixmaps.c,v 1.62 2001/12/26 20:37:38 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2001, the ROX-Filer team.
@@ -33,10 +33,13 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
+#include <fcntl.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
 #include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include "collection.h"
+#include <gdk-pixbuf/gdk-pixbuf-loader.h>
 
 #include "global.h"
 
@@ -45,6 +48,8 @@
 #include "gui_support.h"
 #include "pixmaps.h"
 #include "main.h"
+#include "filer.h"
+#include "dir.h"
 
 GFSCache *pixmap_cache = NULL;
 
@@ -73,10 +78,10 @@ MaskedPixmap *im_symlink;
 MaskedPixmap *im_unmounted;
 MaskedPixmap *im_mounted;
 MaskedPixmap *im_multiple;
-MaskedPixmap *im_exec_file;
 MaskedPixmap *im_appdir;
 
 MaskedPixmap *im_help;
+MaskedPixmap *im_dirs;
 
 /* Static prototypes */
 
@@ -88,9 +93,11 @@ static int getref(MaskedPixmap *mp);
 static gint purge(gpointer data);
 static MaskedPixmap *image_from_file(char *path);
 static MaskedPixmap *get_bad_image(void);
-static MaskedPixmap *image_from_pixbuf(GdkPixbuf *pixbuf);
 static GdkPixbuf *scale_pixbuf(GdkPixbuf *src, int max_w, int max_h);
 static GdkPixbuf *scale_pixbuf_up(GdkPixbuf *src, int max_w, int max_h);
+static void got_thumb_data(GdkPixbufLoader *loader,
+				gint fd, GdkInputCondition cond);
+static GdkPixbuf *get_thumbnail_for(char *path);
 
 
 /****************************************************************
@@ -99,8 +106,8 @@ static GdkPixbuf *scale_pixbuf_up(GdkPixbuf *src, int max_w, int max_h);
 
 void pixmaps_init(void)
 {
-	gdk_rgb_init();
 #ifndef GTK2
+	gdk_rgb_init();
 	gtk_widget_push_visual(gdk_rgb_get_visual());
 #endif
 	gtk_widget_push_colormap(gdk_rgb_get_cmap());
@@ -138,10 +145,10 @@ static void load_default_pixmaps(void)
 	im_unmounted = load_pixmap("pixmaps/mount.xpm");
 	im_mounted = load_pixmap("pixmaps/mounted.xpm");
 	im_multiple = load_pixmap("pixmaps/multiple.xpm");
-	im_exec_file = load_pixmap("pixmaps/exec.xpm");
 	im_appdir = load_pixmap("pixmaps/application.xpm");
 
 	im_help = load_pixmap("pixmaps/help.xpm");
+	im_dirs = load_pixmap("pixmaps/dirs.xpm");
 }
 
 void pixmap_ref(MaskedPixmap *mp)
@@ -224,10 +231,199 @@ void pixmap_make_small(MaskedPixmap *mp)
 	mp->sm_height = mp->height;
 }
 
+#ifdef GTK2
+# define GET_LOADER(key) (g_object_get_data(G_OBJECT(loader), key))
+# define SET_LOADER(key, data) (g_object_set_data(G_OBJECT(loader), key, data))
+# define SET_LOADER_FULL(key, data, free) \
+		(g_object_set_data_full(G_OBJECT(loader), key, data, free))
+#else
+# define GET_LOADER(key) (gtk_object_get_data(GTK_OBJECT(loader), key))
+# define SET_LOADER(key, data) \
+		(gtk_object_set_data(GTK_OBJECT(loader), key, data))
+# define SET_LOADER_FULL(key, data, free) \
+		(gtk_object_set_data_full(GTK_OBJECT(loader), key, data, free))
+#endif
+
+/* Load image 'path' in the background and insert into pixmap_cache.
+ * Call callback(data, path) when done (PATH is NULL error).
+ * If the image is already uptodate, or being created already, calls the
+ * callback right away.
+ */
+void pixmap_background_thumb(gchar *path, GFunc callback, gpointer data)
+{
+	GdkPixbufLoader *loader;
+	int		fd, tag;
+	gboolean	found;
+	MaskedPixmap	*image;
+	GdkPixbuf	*pixbuf;
+
+	image = g_fscache_lookup_full(pixmap_cache, path,
+					FSCACHE_LOOKUP_ONLY_NEW, &found);
+
+	if (image || found)
+	{
+		/* Thumbnail is known, or being created */
+		callback(data, NULL);
+		return;
+	}
+
+	pixbuf = get_thumbnail_for(path);
+	if (pixbuf)
+	{
+		MaskedPixmap *image;
+
+		image = image_from_pixbuf(pixbuf);
+		gdk_pixbuf_unref(pixbuf);
+		g_fscache_insert(pixmap_cache, path, image);
+		callback(data, path);
+		return;
+	}
+
+	/* Add an entry, set to NULL, so no-one else tries to load this
+	 * image.
+	 */
+	g_fscache_insert(pixmap_cache, path, NULL);
+
+	fd = mc_open(path, O_RDONLY | O_NONBLOCK);
+	if (fd == -1)
+	{
+		callback(data, NULL);
+		return;
+	}
+
+	loader = gdk_pixbuf_loader_new();
+
+	SET_LOADER_FULL("thumb-path", g_strdup(path), g_free);
+	SET_LOADER("thumb-callback", callback);
+	SET_LOADER("thumb-callback-data", data);
+	
+	tag = gdk_input_add(fd, GDK_INPUT_READ,
+			    (GdkInputFunction) got_thumb_data, loader);
+
+	SET_LOADER("thumb-input-tag", GINT_TO_POINTER(tag));
+}
+
 /****************************************************************
  *			INTERNAL FUNCTIONS			*
  ****************************************************************/
 
+#ifdef GTK2
+/* Create a thumbnail file for this image.
+ * XXX: Thumbnails should be deleted somewhere!
+ */
+static void save_thumbnail(char *path, GdkPixbuf *full, MaskedPixmap *image)
+{
+	struct stat info;
+	int original_width, original_height;
+	GString *to;
+	char *md5, *swidth, *sheight, *ssize, *smtime, *uri;
+	mode_t old_mask;
+
+	/* If the source image was very small, don't bother saving */
+	if (gdk_pixbuf_get_width(full) * gdk_pixbuf_get_height(full) <
+			   (HUGE_WIDTH * HUGE_HEIGHT * 3))
+		return;
+
+	original_width = gdk_pixbuf_get_width(full);
+	original_height = gdk_pixbuf_get_height(full);
+
+	if (mc_stat(path, &info) != 0)
+		return;
+
+	swidth = g_strdup_printf("%d", original_width);
+	sheight = g_strdup_printf("%d", original_height);
+	ssize = g_strdup_printf("%" SIZE_FMT, info.st_size);
+	smtime = g_strdup_printf("%ld", info.st_mtime);
+
+	path = pathdup(path);
+	uri = g_strconcat("file://", path, NULL);
+	md5 = md5_hash(uri);
+	g_free(path);
+		
+	to = g_string_new(home_dir);
+	g_string_append(to, "/.thumbnails");
+	mkdir(to->str, 0700);
+	g_string_append(to, "/96x96/");
+	mkdir(to->str, 0700);
+	g_string_append(to, md5);
+	g_string_append(to, ".png");
+
+	g_free(md5);
+
+	old_mask = umask(0077);
+	gdk_pixbuf_save(image->huge_pixbuf,
+			to->str,
+			"png",
+			NULL,
+			"tEXt::Thumb::Image::Width", swidth,
+			"tEXt::Thumb::Image::Height", sheight,
+			"tEXt::Thumb::Size", ssize,
+			"tEXt::Thumb::MTime", smtime,
+			"tEXt::Thumb::URI", uri,
+			"tEXt::Software", PROJECT,
+			NULL);
+	umask(old_mask);
+
+	g_string_free(to, TRUE);
+	g_free(swidth);
+	g_free(sheight);
+	g_free(ssize);
+	g_free(smtime);
+	g_free(uri);
+}
+#endif
+
+/* Check if we have an up-to-date thumbnail for this image.
+ * If so, return it. Otherwise, returns NULL.
+ */
+static GdkPixbuf *get_thumbnail_for(char *path)
+{
+	GdkPixbuf *thumb = NULL;
+#ifdef GTK2
+	char *thumb_path, *md5, *uri;
+	char *ssize, *smtime;
+	struct stat info;
+
+	path = pathdup(path);
+	uri = g_strconcat("file://", path, NULL);
+	md5 = md5_hash(uri);
+	g_free(uri);
+	
+	thumb_path = g_strdup_printf("%s/.thumbnails/96x96/%s.png",
+					home_dir, md5);
+	g_free(md5);
+
+	thumb = gdk_pixbuf_new_from_file(thumb_path, NULL);
+	if (!thumb)
+		goto err;
+
+	/* Note that these don't need freeing... */
+	ssize = gdk_pixbuf_get_option(thumb, "tEXt::Thumb::Size");
+	if (!ssize)
+		goto err;
+
+	smtime = gdk_pixbuf_get_option(thumb, "tEXt::Thumb::MTime");
+	if (!smtime)
+		goto err;
+	
+	if (mc_stat(path, &info) != 0)
+		goto err;
+	
+	if (info.st_mtime != atol(smtime) || info.st_size != atol(ssize))
+		goto err;
+
+	goto out;
+err:
+	if (thumb)
+		gdk_pixbuf_unref(thumb);
+	thumb = NULL;
+out:
+	g_free(path);
+	g_free(thumb_path);
+#endif
+	return thumb;
+}
+
 /* Load the image 'path' and return a pointer to the resulting
  * MaskedPixmap. NULL on failure.
  */
@@ -235,16 +431,26 @@ static MaskedPixmap *image_from_file(char *path)
 {
 	GdkPixbuf	*pixbuf;
 	MaskedPixmap	*image;
-	
-	pixbuf = gdk_pixbuf_new_from_file(path
 #ifdef GTK2
-			, NULL	/* XXX: Maybe something should go here? */
-#endif
-			);
+	GError		*error = NULL;
+	
+	pixbuf = get_thumbnail_for(path);
+	if (!pixbuf)
+		pixbuf = gdk_pixbuf_new_from_file(path, &error);
+	if (!pixbuf)
+	{
+		g_print("%s\n", error ? error->message : _("Unknown error"));
+		g_error_free(error);
+		return NULL;
+	}
+#else
+	pixbuf = gdk_pixbuf_new_from_file(path);
 	if (!pixbuf)
 		return NULL;
+#endif
 
 	image = image_from_pixbuf(pixbuf);
+
 	gdk_pixbuf_unref(pixbuf);
 
 	return image;
@@ -270,10 +476,12 @@ static GdkPixbuf *scale_pixbuf(GdkPixbuf *src, int max_w, int max_h)
 		float scale_x = ((float) w) / max_w;
 		float scale_y = ((float) h) / max_h;
 		float scale = MAX(scale_x, scale_y);
+		int dest_w = w / scale;
+		int dest_h = h / scale;
 		
 		return gdk_pixbuf_scale_simple(src,
-						w / scale,
-						h / scale,
+						MAX(dest_w, 1),
+						MAX(dest_h, 1),
 						GDK_INTERP_BILINEAR);
 	}
 }
@@ -307,7 +515,7 @@ static GdkPixbuf *scale_pixbuf_up(GdkPixbuf *src, int max_w, int max_h)
 }
 
 /* Turn a full-size pixbuf into a MaskedPixmap */
-static MaskedPixmap *image_from_pixbuf(GdkPixbuf *full_size)
+MaskedPixmap *image_from_pixbuf(GdkPixbuf *full_size)
 {
 	MaskedPixmap	*mp;
 	GdkPixbuf	*huge_pixbuf, *normal_pixbuf;
@@ -425,3 +633,71 @@ static gint purge(gpointer data)
 
 	return TRUE;
 }
+
+static void got_thumb_data(GdkPixbufLoader *loader,
+				gint fd, GdkInputCondition cond)
+{
+	GFunc callback;
+	gchar *path;
+	gpointer cbdata;
+	char data[4096];
+	int  got, tag;
+
+	got = read(fd, data, sizeof(data));
+
+#ifdef GTK2
+	if (got > 0 && gdk_pixbuf_loader_write(loader, data, got, NULL))
+		return;
+#else
+	if (got > 0 && gdk_pixbuf_loader_write(loader, data, got))
+		return;
+#endif
+
+	/* Some kind of error, or end-of-file */
+
+	path = GET_LOADER("thumb-path");
+
+	tag = GPOINTER_TO_INT(GET_LOADER("thumb-input-tag"));
+	gdk_input_remove(tag);
+
+#ifdef GTK2
+	gdk_pixbuf_loader_close(loader, NULL);
+#else
+	gdk_pixbuf_loader_close(loader);
+#endif
+
+	if (got == 0)
+	{
+		MaskedPixmap *image;
+		GdkPixbuf *pixbuf;
+
+		pixbuf = gdk_pixbuf_loader_get_pixbuf(loader);
+		if (pixbuf)
+		{
+			gdk_pixbuf_ref(pixbuf);
+			image = image_from_pixbuf(pixbuf);
+
+			g_fscache_insert(pixmap_cache, path, image);
+#ifdef GTK2
+			save_thumbnail(path, pixbuf, image);
+#endif
+			gdk_pixbuf_unref(pixbuf);
+			pixmap_unref(image);
+		}
+		else
+			got = -1;
+	}
+
+	mc_close(fd);
+	
+	callback = GET_LOADER("thumb-callback");
+	cbdata = GET_LOADER("thumb-callback-data");
+
+	callback(cbdata, got != 0 ? NULL : path);
+
+#ifdef GTK2
+	g_object_unref(G_OBJECT(loader));
+#else
+	gtk_object_unref(GTK_OBJECT(loader));
+#endif
+}
