@@ -1,5 +1,5 @@
 /*
- * $Id: rox_gettext.c,v 1.5 2001/02/07 17:58:40 tal197 Exp $
+ * $Id: rox_gettext.c,v 1.11 2001/12/23 14:11:20 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2001, the ROX-Filer team.
@@ -25,6 +25,8 @@
 
 #include "config.h"
 
+#include <string.h>
+
 #include "global.h"
 
 #include "gui_support.h"
@@ -52,7 +54,7 @@ char *rox_gettext(char *from)
 {
 	char	*retval;
 	
-	if (!translate)
+	if (!translate || !from || !*from)
 		return from;
 
 	retval = g_hash_table_lookup(translate, from);
@@ -79,15 +81,18 @@ void rox_add_translations(char *path)
 	long		size;
 	gboolean	swap;		/* TRUE => reverse byte-order of ints */
 	int		n, n_total;
+#ifdef GTK2
+	char		*charset = NULL;
+#endif
 
 	if (load_file(path, &data, &size) == FALSE)
-		return;
+		goto out;
 
 	if (size < 20)
 	{
-		delayed_error(PROJECT, _("Invalid .gmo translation file "
-					"(too short)"));
-		return;
+		delayed_error(_("Invalid .gmo translation file "
+					"(too short): %s"), path);
+		goto out;
 	}
 
 	magic = *((guint *) data);
@@ -98,30 +103,86 @@ void rox_add_translations(char *path)
 		swap = TRUE;
 	else
 	{
-		delayed_error(PROJECT, _("Invalid .gmo translation file "
-					"(GNU magic number not found)\n"));
-		return;
+		delayed_error(_("Invalid .gmo translation file "
+					"(GNU magic number not found): %s"),
+				  path);
+		goto out;
 	}
 
 	if (WORD(data + 4) != 0)
 		g_warning("rox_add_translations: expected format revision 0");
 
 	if (!translate)
+	{
+#ifdef GTK2
+		translate = g_hash_table_new_full(g_str_hash, g_str_equal,
+						  g_free, g_free);
+#else
 		translate = g_hash_table_new(g_str_hash, g_str_equal);
+#endif
+	}
 
 	n_total = WORD(data + 8);
 	from_base = data + WORD(data + 12);
 	to_base = data + WORD(data + 16);
 
+#ifdef GTK2
+	/* Find the charset used, so we can convert to UTF-8 */
 	for (n = 0; n < n_total; n++)
 	{
 		char	*from = data + WORD(from_base + (n << 3) + 4);
-		char	*to   = data + WORD(to_base + (n << 3) + 4);
+		char	*to_raw   = data + WORD(to_base + (n << 3) + 4);
 
-		g_hash_table_insert(translate, from, to);
+		if (*from)
+			continue;
+
+		charset = strstr(to_raw, "charset=");
+		if (charset)
+		{
+			char *tmp;
+
+			charset += 8;
+			tmp = strchr(charset, '\n');
+			if (tmp)
+				charset = g_strndup(charset, tmp - charset);
+			else
+				charset = g_strdup(charset);
+		}
+		break;
+	}
+	if (!charset)
+	{
+		g_warning("Missing charset=... in translation!");
+		charset = g_strdup("iso-8859-1");
+	}
+#endif
+	
+	for (n = 0; n < n_total; n++)
+	{
+		char	*from = data + WORD(from_base + (n << 3) + 4);
+		char	*to_raw   = data + WORD(to_base + (n << 3) + 4);
+#ifdef GTK2
+		char	*to;
+
+		to = g_convert_with_fallback(to_raw, -1,
+					"UTF-8",
+					charset,
+					"#",
+					NULL, NULL, NULL);
+		if (!to)
+			to = g_strdup(to_raw);
+
+		g_hash_table_insert(translate, g_strdup(from), to);
+#else
+		g_hash_table_insert(translate, from, to_raw);
+#endif
 	}
 
-	/* Note: Do not free the file data! */
+out:
+#ifdef GTK2
+	g_free(data);
+	g_free(charset);
+#endif
 }
 
 /****************************************************************
