@@ -1,5 +1,5 @@
 /*
- * $Id: run.c,v 1.35 2001/10/14 12:12:24 tal197 Exp $
+ * $Id: run.c,v 1.43 2001/12/20 14:08:57 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2001, the ROX-Filer team.
@@ -33,9 +33,11 @@
 #include "support.h"
 #include "gui_support.h"
 #include "filer.h"
+#include "display.h"
 #include "main.h"
 #include "type.h"
 #include "dir.h"
+#include "diritem.h"
 #include "action.h"
 #include "icon.h"
 
@@ -72,8 +74,7 @@ void run_app(char *path)
 	apprun = g_string_new(path);
 	argv[0] = g_string_append(apprun, "/AppRun")->str;
 
-	if (!spawn_full(argv, home_dir))
-		report_error(PROJECT, "Failed to fork() child process");
+	rox_spawn(home_dir, argv);
 	
 	g_string_free(apprun, TRUE);
 }
@@ -90,7 +91,7 @@ void run_with_files(char *path, GList *uri_list)
 
 	if (stat(path, &info))
 	{
-		delayed_error(PROJECT, _("Program not found - deleted?"));
+		delayed_error(_("Program %s not found - deleted?"), path);
 		return;
 	}
 
@@ -116,8 +117,7 @@ void run_with_files(char *path, GList *uri_list)
 	
 	argv[argc++] = NULL;
 
-	if (!spawn_full(argv, home_dir))
-		delayed_error(PROJECT, _("Failed to fork() child process"));
+	rox_spawn(home_dir, argv);
 }
 
 /* Run the program as '<path> -', piping the data to it via stdin.
@@ -132,7 +132,7 @@ void run_with_data(char *path, gpointer data, gulong length)
 
 	if (stat(path, &info))
 	{
-		delayed_error(PROJECT, _("Program not found - deleted?"));
+		delayed_error(_("Program %s not found - deleted?"), path);
 		return;
 	}
 
@@ -143,7 +143,7 @@ void run_with_data(char *path, gpointer data, gulong length)
 	
 	if (pipe(fds))
 	{
-		delayed_error("pipe() failed", g_strerror(errno));
+		delayed_error("pipe: %s", g_strerror(errno));
 		return;
 	}
 	close_on_exec(fds[1], TRUE);
@@ -152,7 +152,7 @@ void run_with_data(char *path, gpointer data, gulong length)
 	switch (fork())
 	{
 		case -1:
-			delayed_error("fork() failed", g_strerror(errno));
+			delayed_error("fork: %s", g_strerror(errno));
 			close(fds[1]);
 			break;
 		case 0:
@@ -217,7 +217,7 @@ gboolean run_diritem(guchar *full_path,
 				GList	*paths;
 
 				paths = g_list_prepend(NULL, full_path);
-				action_mount(paths, filer_window == NULL);
+				action_mount(paths, filer_window == NULL, -1);
 				g_list_free(paths);
 				if (item->flags & ITEM_FLAG_MOUNTED ||
 						!filer_window)
@@ -230,7 +230,7 @@ gboolean run_diritem(guchar *full_path,
 				filer_opendir(full_path, src_window);
 			return TRUE;
 		case TYPE_FILE:
-			if ((item->mime_type == &special_exec) && !edit)
+			if ((item->mime_type == special_exec) && !edit)
 			{
 				char	*argv[] = {NULL, NULL};
 				guchar	*dir = filer_window ? filer_window->path
@@ -238,26 +238,18 @@ gboolean run_diritem(guchar *full_path,
 
 				argv[0] = full_path;
 
-				if (spawn_full(argv, dir))
-					return TRUE;
-				else
-				{
-					report_error(PROJECT,
-						_("Failed to fork() child"));
-					return FALSE;
-				}
+				return rox_spawn(dir, argv);
 			}
 
-			return open_file(full_path, edit ? &text_plain
+			return open_file(full_path, edit ? text_plain
 						  : item->mime_type);
 		case TYPE_ERROR:
-			delayed_error(full_path,
-					_("File doesn't exist, or I can't "
-					  "access it"));
+			delayed_error(_("File doesn't exist, or I can't "
+					  "access it: %s"), full_path);
 			return FALSE;
 		default:
-			delayed_error(full_path,
-					_("I don't know how to open that"));
+		        delayed_error(
+				_("I don't know how to open '%s'"), full_path);
 			return FALSE;
 	}
 }
@@ -266,12 +258,13 @@ gboolean run_diritem(guchar *full_path,
 gboolean run_by_path(guchar *full_path)
 {
 	gboolean retval;
-	DirItem	item;
+	DirItem	*item;
 
 	/* XXX: Loads an image - wasteful */
-	diritem_stat(full_path, &item, FALSE);
-	retval = run_diritem(full_path, &item, NULL, NULL, FALSE);
-	diritem_clear(&item);
+	item = diritem_new(NULL);
+	diritem_restat(full_path, item);
+	retval = run_diritem(full_path, item, NULL, NULL, FALSE);
+	diritem_free(item);
 	
 	return retval;
 }
@@ -282,17 +275,20 @@ void show_item_help(guchar *path, DirItem *item)
 	{
 		case TYPE_FILE:
 			if (item->flags & ITEM_FLAG_EXEC_FILE)
-				delayed_error(_("Executable file"),
-				      _("This is a file with an eXecute bit "
+				delayed_error(
+				      _("Executable file:\n"
+					"This is a file with an eXecute bit "
 					"set - it can be run as a program."));
 			else
-				delayed_error(_("File"), _(
+				delayed_error(
+				      _("File:\n"
 					"This is a data file. Try using the "
 					"Info menu item to find out more..."));
 			break;
 		case TYPE_DIRECTORY:
 			if (item->flags & ITEM_FLAG_MOUNT_POINT)
-				delayed_error(_("Mount point"), _(
+				delayed_error(
+				_("Mount point:\n"
 				"A mount point is a directory which another "
 				"filing system can be mounted on. Everything "
 				"on the mounted filesystem then appears to be "
@@ -302,23 +298,27 @@ void show_item_help(guchar *path, DirItem *item)
 			break;
 		case TYPE_CHAR_DEVICE:
 		case TYPE_BLOCK_DEVICE:
-			delayed_error(_("Device file"), _(
+			delayed_error(
+				_("Device file:\n"
 				"Device files allow you to read from or write "
 				"to a device driver as though it was an "
 				"ordinary file."));
 			break;
 		case TYPE_PIPE:
-			delayed_error(_("Named pipe"), _(
+			delayed_error(
+				_("Named pipe:\n"
 				"Pipes allow different programs to "
 				"communicate. One program writes data to the "
 				"pipe while another one reads it out again."));
 			break;
 		case TYPE_SOCKET:
-			delayed_error(_("Socket"), _(
+			delayed_error(
+				_("Socket:\n"
 				"Sockets allow processes to communicate."));
 			break;
 		default:
-			delayed_error(_("Unknown type"),  _(
+			delayed_error(
+				_("Unknown type:\n"
 				"I couldn't find out what kind of file this "
 				"is. Maybe it doesn't exist anymore or you "
 				"don't have search permission on the directory "
@@ -350,7 +350,11 @@ void open_to_show(guchar *path)
 		*slash = '\0';
 		new = filer_opendir(dir, NULL);
 		if (new)
+		{
+			if (slash[1] == '.')
+				display_set_hidden(new, TRUE);
 			display_set_autoselect(new, slash + 1);
+		}
 	}
 
 	g_free(dir);
@@ -401,7 +405,8 @@ static void write_data(gpointer data, gint fd, GdkInputCondition cond)
 		{
 			if (errno == EAGAIN)
 				return;
-			delayed_error(_("ROX-Filer - Sending data to program"),
+			delayed_error(
+					_("Could not send data to program: %s"),
 					g_strerror(errno));
 			goto finish;
 		}
@@ -431,7 +436,8 @@ static gboolean follow_symlink(char *full_path,
 	got = readlink(full_path, path, MAXPATHLEN);
 	if (got < 0)
 	{
-		delayed_error(PROJECT, g_strerror(errno));
+		delayed_error(_("Could not read link: %s"),
+				  g_strerror(errno));
 		return FALSE;
 	}
 
@@ -457,9 +463,9 @@ static gboolean follow_symlink(char *full_path,
 	if (!slash)
 	{
 		g_free(real);
-		delayed_error(PROJECT,
+		delayed_error(
 			_("Broken symlink (or you don't have permission "
-			  "to follow it)."));
+			  "to follow it): %s"), full_path);
 		return FALSE;
 	}
 
@@ -488,23 +494,18 @@ static gboolean follow_symlink(char *full_path,
 /* Load this file into an appropriate editor */
 static gboolean open_file(guchar *path, MIME_type *type)
 {
-	GString		*message;
-
 	g_return_val_if_fail(type != NULL, FALSE);
 
 	if (type_open(path, type))
 		return TRUE;
 
-	message = g_string_new(NULL);
-	g_string_sprintf(message,
+	report_error(
 		_("No run action specified for files of this type (%s/%s) - "
 		"you can set a run action by choosing `Set Run Action' "
 		"from the File menu, or you can just drag the file to an "
 		"application"),
 		type->media_type,
 		type->subtype);
-	report_error(PROJECT, message->str);
-	g_string_free(message, TRUE);
 
 	return FALSE;
 }
@@ -522,13 +523,14 @@ static void dir_show_help(DirItem *item, char *path)
 	if (mc_stat(help_dir, &info) == 0)
 		filer_opendir(help_dir, NULL);
 	else if (item->flags & ITEM_FLAG_APPDIR)
-		delayed_error(_("Application"),
-			_("This is an application directory - you can "
+		delayed_error(
+			_("Application:\n"
+			"This is an application directory - you can "
 			"run it as a program, or open it (hold down "
 			"Shift while you open it). Most applications provide "
 			"their own help here, but this one doesn't."));
 	else
-		delayed_error(_("Directory"), _(
+		delayed_error(_("Directory:\n"
 				"This is a directory. It contains an index to "
 				"other items - open it to see the list."));
 }
