@@ -1,5 +1,5 @@
 /*
- * $Id: toolbar.c,v 1.21 2001/09/25 14:30:39 tal197 Exp $
+ * $Id: toolbar.c,v 1.26 2001/12/16 16:37:18 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2001, the ROX-Filer team.
@@ -27,6 +27,7 @@
 
 #include "global.h"
 
+#include "collection.h"
 #include "toolbar.h"
 #include "options.h"
 #include "support.h"
@@ -34,10 +35,12 @@
 #include "menu.h"
 #include "dnd.h"
 #include "filer.h"
+#include "display.h"
 #include "pixmaps.h"
 #include "bind.h"
 #include "type.h"
 #include "dir.h"
+#include "diritem.h"
 
 typedef struct _Tool Tool;
 
@@ -76,6 +79,8 @@ static void toolbar_refresh_clicked(GtkWidget *widget,
 static void toolbar_size_clicked(GtkWidget *widget, FilerWindow *filer_window);
 static void toolbar_details_clicked(GtkWidget *widget,
 				    FilerWindow *filer_window);
+static void toolbar_hidden_clicked(GtkWidget *widget,
+				   FilerWindow *filer_window);
 static GtkWidget *add_button(GtkWidget *box, Tool *tool,
 				FilerWindow *filer_window);
 static GtkWidget *create_toolbar(FilerWindow *filer_window);
@@ -97,6 +102,7 @@ static void coll_selection_changed(Collection *collection, guint time,
 static void recreate_toolbar(FilerWindow *filer_window);
 static void toggle_shaded(GtkWidget *widget);
 static void option_notify(void);
+static GList *build_tool_options(OptionUI *ui, xmlNode *node, guchar *label);
 
 static Tool all_tools[] = {
 	{N_("Close"), "close", N_("Close filer window"),
@@ -123,6 +129,10 @@ static Tool all_tools[] = {
 	 toolbar_details_clicked, DROP_NONE, TRUE,
 	 NULL, NULL},
 	
+	{N_("Hidden"), "hidden", N_("Show/hide hidden files"),
+	 toolbar_hidden_clicked, DROP_NONE, TRUE,
+	 NULL, NULL},
+	
 	{N_("Help"), "help", N_("Show ROX-Filer help"),
 	 toolbar_help_clicked, DROP_NONE, TRUE,
 	 NULL, NULL},
@@ -158,6 +168,8 @@ void toolbar_init(void)
 			g_free(path);
 		}
 	}
+
+	option_register_widget("tool-options", build_tool_options);
 }
 
 /* Returns a button which can be used to turn a tool on and off.
@@ -226,7 +238,15 @@ void toolbar_update_info(FilerWindow *filer_window)
 
 static void toolbar_help_clicked(GtkWidget *widget, FilerWindow *filer_window)
 {
-	filer_opendir(make_path(app_dir, "Help")->str, NULL);
+	GdkEvent	*event;
+
+	event = gtk_get_current_event();
+	if (event->type == GDK_BUTTON_RELEASE &&
+			((GdkEventButton *) event)->button != 1)
+		filer_change_to(filer_window,
+				make_path(app_dir, "Help")->str, NULL);
+	else
+		filer_opendir(make_path(app_dir, "Help")->str, NULL);
 }
 
 static void toolbar_refresh_clicked(GtkWidget *widget,
@@ -303,13 +323,21 @@ static void toolbar_details_clicked(GtkWidget *widget,
 				    FilerWindow *filer_window)
 {
 	if (filer_window->details_type == DETAILS_NONE)
-		display_set_layout(filer_window, SMALL_ICONS, DETAILS_SUMMARY);
+		display_set_layout(filer_window,
+				filer_window->display_style,
+				DETAILS_SUMMARY);
 	else
 		display_set_layout(filer_window,
-				option_get_int("display_size"),
+				filer_window->display_style,
 				DETAILS_NONE);
 }
 
+static void toolbar_hidden_clicked(GtkWidget *widget,
+				   FilerWindow *filer_window)
+{
+	display_set_hidden(filer_window, !filer_window->show_hidden);
+}
+
 static GtkWidget *create_toolbar(FilerWindow *filer_window)
 {
 	GtkWidget	*box;
@@ -669,3 +697,75 @@ static void option_notify(void)
 		}
 	}
 }
+
+static void update_tools(OptionUI *ui, guchar *value)
+{
+	GList	*next, *kids;
+
+	kids = gtk_container_children(GTK_CONTAINER(ui->widget));
+
+	for (next = kids; next; next = next->next)
+	{
+		GtkWidget	*kid = (GtkWidget *) next->data;
+		guchar		*name;
+
+		name = gtk_object_get_data(GTK_OBJECT(kid), "tool_name");
+
+		g_return_if_fail(name != NULL);
+		
+		gtk_widget_set_sensitive(GTK_BIN(kid)->child,
+					 !in_list(name, value));
+	}
+
+	g_list_free(kids);
+}
+
+static guchar *read_tools(OptionUI *ui)
+{
+	GList	*next, *kids;
+	GString	*list;
+	guchar	*retval;
+
+	list = g_string_new(NULL);
+
+	kids = gtk_container_children(GTK_CONTAINER(ui->widget));
+
+	for (next = kids; next; next = next->next)
+	{
+		GtkObject	*kid = (GtkObject *) next->data;
+		guchar		*name;
+
+		if (!GTK_WIDGET_SENSITIVE(GTK_BIN(kid)->child))
+		{
+			name = gtk_object_get_data(kid, "tool_name");
+			g_return_val_if_fail(name != NULL, list->str);
+
+			if (list->len)
+				g_string_append(list, ", ");
+			g_string_append(list, name);
+		}
+	}
+
+	g_list_free(kids);
+	retval = list->str;
+	g_string_free(list, FALSE);
+
+	return retval;
+}
+
+static GList *build_tool_options(OptionUI *ui, xmlNode *node, guchar *label)
+{
+	int		i = 0;
+	GtkWidget	*hbox, *tool;
+
+	hbox = gtk_hbox_new(FALSE, 0);
+
+	while ((tool = toolbar_tool_option(i++)))
+		gtk_box_pack_start(GTK_BOX(hbox), tool, FALSE, TRUE, 0);
+
+	ui->update_widget = update_tools;
+	ui->read_widget = read_tools;
+	ui->widget = hbox;
+
+	return g_list_append(NULL, hbox);
+}
