@@ -1,5 +1,5 @@
 /*
- * $Id: type.c,v 1.65 2001/10/16 14:45:15 tal197 Exp $
+ * $Id: type.c,v 1.81 2001/12/27 13:41:53 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2001, the ROX-Filer team.
@@ -37,6 +37,7 @@
 #include "global.h"
 
 #include "string.h"
+#include "fscache.h"
 #include "main.h"
 #include "pixmaps.h"
 #include "run.h"
@@ -44,11 +45,12 @@
 #include "choices.h"
 #include "type.h"
 #include "support.h"
-#include "dir.h"
+#include "diritem.h"
 #include "dnd.h"
+#include "options.h"
+#include "filer.h"
 
-#if defined(HAVE_REGEX_H) && \
-		defined(HAVE_RE_COMPILE_PATTERN) && defined(HAVE_RE_SEARCH)
+#ifdef HAVE_REGEX_H
 # define USE_REGEX 1
 #else
 # undef USE_REGEX
@@ -57,15 +59,42 @@
 #ifdef USE_REGEX
 /* A regexp rule, which maps matching filenames to a MIME-types */
 typedef struct pattern {
-  MIME_type *type;
-  struct re_pattern_buffer buffer;
+	MIME_type *type;
+	regex_t buffer;
 } Pattern;
 #endif
 
+static gboolean	o_display_colour_types = TRUE;
+
+/* Colours for file types (same order as base types) */
+static gchar *opt_type_colours[][2] = {
+	{"display_err_colour",  "#ff0000"},
+	{"display_unkn_colour", "#000000"},
+	{"display_dir_colour",  "#000080"},
+	{"display_pipe_colour", "#444444"},
+	{"display_sock_colour", "#ff00ff"},
+	{"display_file_colour", "#000000"},
+	{"display_cdev_colour", "#000000"},
+	{"display_bdev_colour", "#000000"},
+	{"display_exec_colour", "#006000"},
+	{"display_adir_colour", "#006000"}
+};
+#define NUM_TYPE_COLOURS\
+		(sizeof(opt_type_colours) / sizeof(opt_type_colours[0]))
+
+/* Parsed colours for file types */
+static GdkColor	type_colours[NUM_TYPE_COLOURS];
+
 /* Static prototypes */
+static void alloc_type_colours(void);
 static char *import_extensions(guchar *line);
 static void import_for_dir(guchar *path);
 char *get_action_save_path(GtkWidget *dialog);
+static void edit_mime_types(guchar *unused);
+static void reread_mime_files(guchar *unused);
+static MIME_type *get_mime_type(const gchar *type_name, gboolean can_create);
+static GList *build_type_reread(OptionUI *ui, xmlNode *node, guchar *label);
+static GList *build_type_edit(OptionUI *ui, xmlNode *node, guchar *label);
 
 /* Maps extensions to MIME_types (eg 'png'-> MIME_type *).
  * Extensions may contain dots; 'tar.gz' matches '*.tar.gz', etc.
@@ -77,33 +106,130 @@ static char *current_type = NULL;	/* (used while reading file) */
 #ifdef USE_REGEX
 static GList *patterns = NULL;		/* [(regexp -> MIME type)] */
 #endif
+/* Hash of all allocated MIME types, indexed by "media/subtype".
+ * MIME_type structs are never freed; this table prevents memory leaks
+ * when rereading the config files.
+ */
+static GHashTable *type_hash = NULL;
 
 /* Most things on Unix are text files, so this is the default type */
-MIME_type text_plain 		= {"text", "plain", NULL};
+MIME_type *text_plain;
+MIME_type *special_directory;
+MIME_type *special_pipe;
+MIME_type *special_socket;
+MIME_type *special_block_dev;
+MIME_type *special_char_dev;
+MIME_type *special_exec;
+MIME_type *special_unknown;
 
-MIME_type special_directory 	= {"special", "directory", NULL};
-MIME_type special_pipe 		= {"special", "pipe", NULL};
-MIME_type special_socket 	= {"special", "socket", NULL};
-MIME_type special_block_dev 	= {"special", "block-device", NULL};
-MIME_type special_char_dev 	= {"special", "char-device", NULL};
-MIME_type special_exec 		= {"special", "executable", NULL};
-MIME_type special_unknown 	= {"special", "unknown", NULL};
-
-void type_init()
+void type_init(void)
 {
 	int		i;
 	GPtrArray	*list;
 	
-	re_set_syntax(RE_SYNTAX_POSIX_EGREP);
-
 	extension_hash = g_hash_table_new(g_str_hash, g_str_equal);
+	type_hash = g_hash_table_new(g_str_hash, g_str_equal);
+
+	text_plain = get_mime_type("text/plain", TRUE);
+	special_directory = get_mime_type("special/directory", TRUE);
+	special_pipe = get_mime_type("special/pipe", TRUE);
+	special_socket = get_mime_type("special/socket", TRUE);
+	special_block_dev = get_mime_type("special/block-device", TRUE);
+	special_char_dev = get_mime_type("special/char-device", TRUE);
+	special_exec = get_mime_type("special/executable", TRUE);
+	special_unknown = get_mime_type("special/unknown", TRUE);
 
 	current_type = NULL;
 
 	list = choices_list_dirs("MIME-info");
-	for (i = 0; i < list->len; i++)
+	for (i = list->len - 1; i >= 0; i--)
 		import_for_dir((gchar *) g_ptr_array_index(list, i));
 	choices_free_list(list);
+
+	option_register_widget("type-edit", build_type_edit);
+	option_register_widget("type-reread", build_type_reread);
+	
+	option_add_int("display_colour_types", o_display_colour_types, NULL);
+	
+	for (i = 0; i < NUM_TYPE_COLOURS; i++)
+		option_add_string(opt_type_colours[i][0],
+				  opt_type_colours[i][1],
+				  NULL);
+	alloc_type_colours();
+
+	option_add_notify(alloc_type_colours);
+}
+
+/* Returns the MIME_type structure for the given type name. It is looked
+ * up in type_hash and returned if found. If not found (and can_create is
+ * TRUE) then a new MIME_type is made, added to type_hash and returned.
+ * NULL is returned if type_name is not in type_hash and can_create is
+ * FALSE, or if type_name does not contain a '/' character.
+ */
+static MIME_type *get_mime_type(const gchar *type_name, gboolean can_create)
+{
+        MIME_type *mtype;
+	gchar *slash;
+
+	mtype = g_hash_table_lookup(type_hash, type_name);
+	if (mtype || !can_create)
+		return mtype;
+
+	slash = strchr(type_name, '/');
+	g_return_val_if_fail(slash != NULL, NULL);     /* XXX: Report nicely */
+
+	mtype = g_new(MIME_type, 1);
+	mtype->media_type = g_strndup(type_name, slash - type_name);
+	mtype->subtype = g_strdup(slash + 1);
+	mtype->image = NULL;
+
+	g_hash_table_insert(type_hash, g_strdup(type_name), mtype);
+
+	return mtype;
+}
+
+#ifdef USE_REGEX
+static void pattern_delete(gpointer patt, gpointer udata)
+{
+	Pattern *pattern = (Pattern *) patt;
+
+	regfree(&pattern->buffer);
+
+	g_free(pattern);
+}
+#endif
+
+static gboolean extension_delete(gpointer key, gpointer value, gpointer udata)
+{
+	/* key is a g_strdup'ed string */
+	g_free(key);
+
+	/* value is also preserved in type_hash, so don't delete */
+
+	return TRUE; /* Removes this entry */
+}
+
+/* XXX: Dup code */
+void type_reread(void)
+{
+	GPtrArray	*list;
+	int i;
+
+	g_hash_table_foreach_remove(extension_hash, extension_delete, NULL);
+#ifdef USE_REGEX
+	g_list_foreach(patterns, pattern_delete, NULL);
+	g_list_free(patterns);
+	patterns = NULL;
+#endif
+
+	current_type = NULL;
+
+	list = choices_list_dirs("MIME-info");
+	for (i = list->len - 1; i >= 0; i--)
+		import_for_dir((gchar *) g_ptr_array_index(list, i));
+	choices_free_list(list);
+
+	filer_update_all();
 }
 
 /* Parse every file in 'dir' */
@@ -138,16 +264,11 @@ static void import_for_dir(guchar *path)
 static void add_ext(char *type_name, char *ext)
 {
 	MIME_type *new;
-	char	  *slash;
 
-	slash = strchr(type_name, '/');
-	g_return_if_fail(slash != NULL);	/* XXX: Report nicely */
+	new = get_mime_type(type_name, TRUE);
+	
+	g_return_if_fail(new != NULL);
 			
-	new = g_new(MIME_type, 1);
-	new->media_type = g_strndup(type_name, slash - type_name);
-	new->subtype = g_strdup(slash + 1);
-	new->image = NULL;
-
 	g_hash_table_insert(extension_hash, g_strdup(ext), new);
 }
 
@@ -167,17 +288,16 @@ static void add_regex(char *type_name, char *reg)
 	pattern->buffer.fastmap = NULL;
 	pattern->buffer.translate = NULL;
 
-	if (re_compile_pattern(reg, strlen(reg), &pattern->buffer))
+        if (regcomp(&pattern->buffer, reg, REG_EXTENDED | REG_NOSUB))
 	{
 		regfree(&pattern->buffer);
 		g_free(pattern);
 		return;
 	}
 	
-	new = g_new(MIME_type, 1);
-	new->media_type = g_strndup(type_name, slash - type_name);
-	new->subtype = g_strdup(slash + 1);
-	new->image = NULL;
+	new = get_mime_type(type_name, TRUE);
+	
+	g_return_if_fail(new != NULL);
 
 	pattern->type = new;
 
@@ -298,7 +418,7 @@ MIME_type *type_get_type(guchar *path)
 	if (!type)
 	{
 		if (base == TYPE_FILE && exec)
-			type = &special_exec;
+			type = special_exec;
 		else
 			type = mime_type_from_base_type(base);
 	}
@@ -351,8 +471,7 @@ MIME_type *type_from_path(char *path)
 	{
 		Pattern *pattern = (Pattern *) patt->data;
 
-		if (re_search(&pattern->buffer,
-					leafname, len, 0, len, NULL) >= 0)
+                if (regexec(&pattern->buffer, leafname, 0, NULL, 0) == 0)
 			return pattern->type;
 	}
 #endif
@@ -384,7 +503,7 @@ gboolean type_open(char *path, MIME_type *type)
 {
 	char	*argv[] = {NULL, NULL, NULL};
 	char	*open;
-	gboolean	retval = TRUE;
+	gboolean	retval;
 	struct stat	info;
 
 	argv[1] = path;
@@ -395,7 +514,7 @@ gboolean type_open(char *path, MIME_type *type)
 
 	if (stat(open, &info))
 	{
-		report_error(PROJECT, g_strerror(errno));
+		report_error("stat(%s): %s", open, g_strerror(errno));
 		g_free(open);
 		return FALSE;
 	}
@@ -405,12 +524,7 @@ gboolean type_open(char *path, MIME_type *type)
 	else
 		argv[0] = open;
 
-	if (!spawn_full(argv, home_dir))
-	{
-		report_error(PROJECT,
-				_("Failed to fork() child process"));
-		retval = FALSE;
-	}
+	retval = rox_spawn(home_dir, argv);
 
 	if (argv[0] != open)
 		g_free(argv[0]);
@@ -473,8 +587,9 @@ MaskedPixmap *type_to_icon(MIME_type *type)
 
 	if (!type->image)
 	{
+		/* One ref from the type structure, one returned */
 		type->image = im_unknown;
-		pixmap_ref(type->image);
+		pixmap_ref(im_unknown);
 	}
 
 	type->image_time = now;
@@ -499,10 +614,9 @@ GdkAtom type_to_atom(MIME_type *type)
 
 void show_shell_help(gpointer data)
 {
-	report_error(PROJECT,
-		_("Enter a shell command which will load \"$1\" into "
-		"a suitable program. Eg:\n\n"
-		"gimp \"$1\""));
+	report_error(_("Enter a shell command which will load \"$1\" into "
+			"a suitable program. Eg:\n\n"
+			"gimp \"$1\""));
 }
 
 /* Called if the user clicks on the OK button */
@@ -542,7 +656,7 @@ static void set_shell_action(GtkWidget *dialog)
 		error = errno;
 
 	if (error)
-		report_error(PROJECT, g_strerror(errno));
+		report_error(g_strerror(errno));
 
 	g_free(tmp);
 	g_free(path);
@@ -554,7 +668,7 @@ static void set_shell_action(GtkWidget *dialog)
  * dialog. Make sure it's an application, and make that the default
  * handler.
  */
-void drag_app_dropped(GtkWidget		*frame,
+void drag_app_dropped(GtkWidget		*eb,
 		      GdkDragContext    *context,
 		      gint              x,
 		      gint              y,
@@ -565,7 +679,7 @@ void drag_app_dropped(GtkWidget		*frame,
 {
 	GList	*uris;
 	guchar	*app = NULL;
-	DirItem	item;
+	DirItem	*item;
 
 	if (!selection_data->data)
 		return; 		/* Timeout? */
@@ -578,15 +692,16 @@ void drag_app_dropped(GtkWidget		*frame,
 
 	if (!app)
 	{
-		delayed_error(PROJECT,
+		delayed_error(
 			_("You should drop a single (local) application "
 			"onto the drop box - that application will be "
 			"used to load files of this type in future"));
 		return;
 	}
 
-	diritem_stat(app, &item, FALSE);
-	if (item.flags & (ITEM_FLAG_APPDIR | ITEM_FLAG_EXEC_FILE))
+	item = diritem_new(NULL);
+	diritem_restat(app, item);
+	if (item->flags & (ITEM_FLAG_APPDIR | ITEM_FLAG_EXEC_FILE))
 	{
 		guchar	*path;
 
@@ -595,7 +710,7 @@ void drag_app_dropped(GtkWidget		*frame,
 		if (path)
 		{
 			if (symlink(app, path))
-				delayed_error("symlink failed",
+				delayed_error("symlink: %s",
 						g_strerror(errno));
 			else
 				destroy_on_idle(dialog);
@@ -604,11 +719,11 @@ void drag_app_dropped(GtkWidget		*frame,
 		g_free(path);
 	}
 	else
-		delayed_error(PROJECT,
+		delayed_error(
 			_("This is not a program! Give me an application "
 			"instead!"));
 
-	diritem_clear(&item);
+	diritem_free(item);
 }
 
 /* Find the current command which is used to run files of this type.
@@ -649,14 +764,100 @@ out:
 	return command;
 }
 
+/* Find the current command which is used to run files of this type,
+ * and return a textual description of it.
+ * g_free() the result.
+ */
+gchar *describe_current_command(MIME_type *type)
+{
+	char *handler;
+	char *desc = NULL;
+	struct stat info;
+	char *target;
+
+	g_return_val_if_fail(type != NULL, NULL);
+
+	if (type == special_exec)
+		return g_strdup(_("Execute file"));
+
+	handler = handler_for(type);
+
+	if (!handler)
+		return g_strdup(_("No run action defined"));
+
+	target = readlink_dup(handler);
+	if (target)
+	{
+		/* Cope with relative paths (shouldn't normally be needed) */
+
+		if (target[0] == '/')
+		{
+			g_free(handler);
+			handler = target;
+		}
+		else
+		{
+			gchar *dir;
+
+			dir = g_dirname(handler);
+			g_free(handler);
+			handler = g_strconcat(dir, "/", target, NULL);
+			g_free(target);
+			g_free(dir);
+		}
+	}
+
+	if (mc_stat(handler, &info) !=0 )
+	{
+		desc = g_strdup_printf(_("Error in handler %s: %s"), handler,
+					g_strerror(errno));
+		goto out;
+	}
+
+	if (S_ISDIR(info.st_mode))
+	{
+		gchar *tmp;
+		uid_t dir_uid = info.st_uid;
+
+		tmp = make_path(handler, "AppRun")->str;
+
+		if (mc_lstat(tmp, &info) != 0 || info.st_uid != dir_uid
+			|| !(info.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)))
+			desc = g_strdup_printf(
+				_("Invalid application %s (bad AppRun)"),
+				handler);
+		/* Else, just report handler... */
+
+		goto out;
+	}
+
+	/* It's not an application directory, and it's not a symlink... */
+
+	if (access(handler, X_OK) != 0)
+	{
+		desc = g_strdup_printf(_("Non-executable %s"), handler);
+		goto out;
+	}
+
+	desc = get_current_command(type);
+out:
+	if (!desc)
+		desc = handler;
+	else
+		g_free(handler);
+
+	return desc;
+}
+
 /* Display a dialog box allowing the user to set the default run action
  * for this type.
  */
 void type_set_handler_dialog(MIME_type *type)
 {
 	guchar		*tmp;
+	gchar           *handler;
 	GtkWidget	*dialog, *vbox, *frame, *hbox, *entry, *label, *button;
-	GtkWidget	*radio;
+	GtkWidget	*radio, *eb;
 	GtkTargetEntry 	targets[] = {
 		{"text/uri-list", 0, TARGET_URI_LIST},
 	};
@@ -664,6 +865,10 @@ void type_set_handler_dialog(MIME_type *type)
 	g_return_if_fail(type != NULL);
 
 	dialog = gtk_window_new(GTK_WINDOW_DIALOG);
+#ifdef GTK2
+	gtk_window_set_type_hint(GTK_WINDOW(dialog),
+			GDK_WINDOW_TYPE_HINT_DIALOG);
+#endif
 	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE);
 	gtk_object_set_data(GTK_OBJECT(dialog), "mime_type", type);
 
@@ -692,17 +897,38 @@ void type_set_handler_dialog(MIME_type *type)
 	frame = gtk_frame_new(NULL);
 	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 4);
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
-	gtk_container_set_border_width(GTK_CONTAINER(frame), 4);
+	eb = gtk_event_box_new();
+	gtk_container_add(GTK_CONTAINER(frame), eb);
 
-	gtk_drag_dest_set(frame, GTK_DEST_DEFAULT_ALL,
+	gtk_container_set_border_width(GTK_CONTAINER(eb), 4);
+
+	handler = handler_for(type);
+	if (handler)
+	{
+		char *link;
+
+		link = readlink_dup(handler);
+		if (link)
+		{
+			char *msg;
+
+			msg = g_strdup_printf(_("Currently %s"), link);
+			gtk_tooltips_set_tip(tooltips, eb, msg, NULL);
+			g_free(link);
+			g_free(msg);
+		}
+		g_free(handler);
+	}
+
+	gtk_drag_dest_set(eb, GTK_DEST_DEFAULT_ALL,
 			targets, sizeof(targets) / sizeof(*targets),
 			GDK_ACTION_COPY);
-	gtk_signal_connect(GTK_OBJECT(frame), "drag_data_received",
+	gtk_signal_connect(GTK_OBJECT(eb), "drag_data_received",
 			GTK_SIGNAL_FUNC(drag_app_dropped), dialog);
 
 	label = gtk_label_new(_("Drop a suitable\napplication here"));
 	gtk_misc_set_padding(GTK_MISC(label), 10, 20);
-	gtk_container_add(GTK_CONTAINER(frame), label);
+	gtk_container_add(GTK_CONTAINER(eb), label);
 
 	hbox = gtk_hbox_new(FALSE, 4);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 4);
@@ -730,7 +956,7 @@ void type_set_handler_dialog(MIME_type *type)
 
 	hbox = gtk_hbox_new(TRUE, 4);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
-
+	
 	/* If possible, fill in the entry box with the current command */
 	tmp = get_current_command(type);
 	if (tmp)
@@ -768,7 +994,7 @@ void type_set_handler_dialog(MIME_type *type)
  */
 char *get_action_save_path(GtkWidget *dialog)
 {
-	guchar		*tmp, *path = NULL;
+	guchar		*path = NULL;
 	struct stat 	info;
 	guchar 		*type_name = NULL;
 	MIME_type	*type;
@@ -788,7 +1014,7 @@ char *get_action_save_path(GtkWidget *dialog)
 	path = choices_find_path_save("", PROJECT, FALSE);
 	if (!path)
 	{
-		report_error(PROJECT,
+		report_error(
 		_("Choices saving is disabled by CHOICESPATH variable"));
 		goto out;
 	}
@@ -804,7 +1030,7 @@ char *get_action_save_path(GtkWidget *dialog)
 			if (get_choice(PROJECT,
 				_("A run action already exists and is quite "
 				"a big program - are you sure you want to "
-				"delete it?"), 2, "Delete", "Cancel") != 0)
+				"delete it?"), 2, "Cancel", "Delete") != 1)
 			{
 				g_free(path);
 				path = NULL;
@@ -814,10 +1040,8 @@ char *get_action_save_path(GtkWidget *dialog)
 		
 		if (unlink(path))
 		{
-			tmp = g_strdup_printf( _("Can't remove %s: %s"),
+			report_error(_("Can't remove %s: %s"),
 				path, g_strerror(errno));
-			report_error(PROJECT, tmp);
-			g_free(tmp);
 			g_free(path);
 			path = NULL;
 			goto out;
@@ -834,19 +1058,19 @@ MIME_type *mime_type_from_base_type(int base_type)
 	switch (base_type)
 	{
 		case TYPE_FILE:
-			return &text_plain;
+			return text_plain;
 		case TYPE_DIRECTORY:
-			return &special_directory;
+			return special_directory;
 		case TYPE_PIPE:
-			return &special_pipe;
+			return special_pipe;
 		case TYPE_SOCKET:
-			return &special_socket;
+			return special_socket;
 		case TYPE_BLOCK_DEVICE:
-			return &special_block_dev;
+			return special_block_dev;
 		case TYPE_CHAR_DEVICE:
-			return &special_char_dev;
+			return special_char_dev;
 	}
-	return &special_unknown;
+	return special_unknown;
 }
 
 /* Takes the st_mode field from stat() and returns the base type.
@@ -867,7 +1091,7 @@ int mode_to_base_type(int st_mode)
 	else if (S_ISSOCK(st_mode))
 		return TYPE_SOCKET;
 
-	return TYPE_UNKNOWN;
+	return TYPE_ERROR;
 }
 
 /* Returns TRUE is this is something that is run by looking up its type
@@ -878,6 +1102,126 @@ gboolean can_set_run_action(DirItem *item)
 	g_return_val_if_fail(item != NULL, FALSE);
 
 	return item->base_type == TYPE_FILE &&
-		!(item->mime_type == &special_exec);
+		!(item->mime_type == special_exec);
 }
 
+/* Open all <Choices>/type directories and display a message */
+static void open_choices_dirs(gchar *type, gchar *what)
+{
+	guchar *dir;
+	GPtrArray *list;
+	int i;
+
+	dir = choices_find_path_save("", type, TRUE);
+	g_free(dir);
+	list = choices_list_dirs(type);
+
+	for (i = list->len - 1; i >= 0; i--)
+		filer_opendir(list->pdata[i], NULL);
+
+	choices_free_list(list);
+}
+
+/* To edit the MIME types, open a filer window for <Choices>/MIME-info */
+static void edit_mime_types(guchar *unused)
+{
+	open_choices_dirs("MIME-info", "the files defining MIME types");
+}
+
+static void reread_mime_files(guchar *unused)
+{
+	type_reread();
+}
+
+static GList *build_type_reread(OptionUI *none, xmlNode *node, guchar *label)
+{
+	GtkWidget *button, *align;
+
+	align = gtk_alignment_new(0.1, 0, 0.1, 0);
+	button = gtk_button_new_with_label(_(label));
+	gtk_container_add(GTK_CONTAINER(align), button);
+
+	gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
+			GTK_SIGNAL_FUNC(reread_mime_files), NULL);
+
+	return g_list_append(NULL, align);
+}
+
+static GList *build_type_edit(OptionUI *none, xmlNode *node, guchar *label)
+{
+	GtkWidget *button, *align;
+
+	align = gtk_alignment_new(0.1, 0, 0.1, 0);
+	button = gtk_button_new_with_label(_(label));
+	gtk_container_add(GTK_CONTAINER(align), button);
+
+	gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
+			GTK_SIGNAL_FUNC(edit_mime_types), NULL);
+
+	return g_list_append(NULL, align);
+}
+
+/* Parse file type colours and allocate/free them as necessary */
+static void alloc_type_colours(void)
+{
+	gboolean	success[NUM_TYPE_COLOURS];
+	int		change_count = 0;	/* No. needing realloc */
+	int		i;
+	static gboolean	allocated = FALSE;
+
+	o_display_colour_types = option_get_int("display_colour_types");
+
+	/* Parse colours */
+	for (i = 0; i < NUM_TYPE_COLOURS; i++)
+	{
+		GdkColor *c = &type_colours[i];
+		gushort r = c->red;
+		gushort g = c->green;
+		gushort b = c->blue;
+
+		gdk_color_parse(
+			option_get_static_string(opt_type_colours[i][0]),
+			&type_colours[i]);
+
+		if (allocated && (c->red != r || c->green != g || c->blue != b))
+			change_count++;
+	}
+	
+	/* Free colours if they were previously allocated and
+	 * have changed or become unneeded.
+	 */
+	if (allocated && (change_count || !o_display_colour_types))
+	{
+		gdk_colormap_free_colors(gdk_rgb_get_cmap(),
+					 type_colours, NUM_TYPE_COLOURS);
+		allocated = FALSE;
+	}
+
+	/* Allocate colours, unless they are still allocated (=> they didn't
+	 * change) or we don't want them anymore.
+	 * XXX: what should be done if allocation fails?
+	 */
+	if (!allocated && o_display_colour_types)
+	{
+		gdk_colormap_alloc_colors(gdk_rgb_get_cmap(),
+				type_colours, NUM_TYPE_COLOURS,
+				FALSE, TRUE, success);
+		allocated = TRUE;
+	}
+}
+
+/* Return a pointer to a (static) colour for this item. If colouring is
+ * off, returns normal.
+ */
+GdkColor *type_get_colour(DirItem *item, GdkColor *normal)
+{
+	if (!o_display_colour_types)
+		return normal;
+
+	if (item->flags & ITEM_FLAG_EXEC_FILE)
+		return &type_colours[8];
+	else if (item->flags & ITEM_FLAG_APPDIR)
+		return &type_colours[9];
+	else
+		return &type_colours[item->base_type];
+}
